@@ -26,8 +26,7 @@
 
 
 (declare
-  (unit driver)
-  (disable-warning var))
+  (unit driver))
 
 (include "compiler-namespace")
 (include "tweaks")
@@ -92,7 +91,7 @@
 	(upap #f)
 	(ssize (or (memq 'nursery options) (memq 'stack-size options))) )
 
-    (define (cputime) (##sys#fudge 6))
+    (define (cputime) (current-milliseconds))
 
     (define (dribble fstr . args)
       (when verbose (printf "~?~%~!" fstr args)))
@@ -178,6 +177,7 @@
     (when (and (memq 'emit-all-import-libraries options)
 	       (not a-only))
       (set! all-import-libraries #t))
+    (set! enable-module-registration (not (memq 'no-module-registration options)))
     (when (memq 'lambda-lift options) (set! do-lambda-lifting #t))
     (when (memq 'scrutinize options) (set! do-scrutinize #t))
     (when (memq 't debugging-chicken) (##sys#start-timer))
@@ -196,9 +196,7 @@
       (set! local-definitions #t))
     (when (memq 'inline-global options)
       (set! enable-inline-files #t)
-      (set! inline-locally #t)
-      (set! inline-globally #t))
-    (set! disabled-warnings (map string->symbol (collect-options 'disable-warning)))
+      (set! inline-locally #t))
     (when (or verbose do-scrutinize)
       (set! ##sys#notices-enabled #t))
     (when (memq 'no-warnings options) 
@@ -228,8 +226,6 @@
       (dribble "Identifiers and symbols are case insensitive")
       (register-feature! 'case-insensitive)
       (case-sensitive #f) )
-    (when (memq 'compress-literals options)
-      (compiler-warning 'usage "`the -compress-literals' option is obsolete") )
     (when kwstyle
       (let ([val (option-arg kwstyle)])
 	(cond [(string=? "prefix" val) (keyword-style #:prefix)]
@@ -265,6 +261,8 @@
       (set! no-bound-checks #t) )
     (when (memq 'no-procedure-checks options)
       (set! no-procedure-checks #t) )
+    (when (memq 'no-procedure-checks-for-toplevel-bindings options)
+      (set! no-global-procedure-checks #t) )
     (when (memq 'no-procedure-checks-for-usual-bindings options)
       (for-each 
        (lambda (v)
@@ -281,6 +279,9 @@
     (for-each 
      register-feature!
      (append-map (cut string-split <> ",") (collect-options 'feature)))
+    (for-each 
+     unregister-feature!
+     (append-map (cut string-split <> ",") (collect-options 'no-feature)))
 
     ;; Load extensions:
     (set! ##sys#features (cons #:compiler-extension ##sys#features))
@@ -363,7 +364,9 @@
 	   (newline) )
 	  ((not filename)
 	   (print-version #t)
-	   (display "\nEnter \"chicken -help\" for information on how to use it.\n") )
+	   (display "\nEnter `chicken -help' for information on how to use the compiler,\n")
+	   (display "or try `csc' for a more convenient interface.\n")
+	   (display "\nRun `csi' to start the interactive interpreter.\n"))
 	  (else
 
 	   ;; Display header:
@@ -524,7 +527,6 @@
 		    (concatenate (map cdr req))) )
 		 (let ((ifs (collect-options 'consult-inline-file)))
 		   (unless (null? ifs)
-		     (set! inline-globally #t)
 		     (set! inline-locally #t)
 		     (for-each 
 		      (lambda (ilf)
@@ -585,10 +587,13 @@
 			   
 			   (else
 			    (print-node "optimized" '|7| node2)
-			    (when inline-output-file
+			    ;; inlining into a file with interrupts enabled would
+			    ;; change semantics
+			    (when (and inline-output-file insert-timer-checks)
 			      (let ((f inline-output-file))
 				(dribble "Generating global inline file `~a' ..." f)
 				(emit-global-inline-file f db) ) )
+			    (check-for-unsafe-toplevel-procedure-calls node2 db)
 			    (begin-time)
 			    (set! node2 (perform-closure-conversion node2 db))
 			    (end-time "closure conversion")
