@@ -38,7 +38,8 @@
 
 (define default-declarations
   '((always-bound
-     ##sys#standard-input ##sys#standard-output ##sys#standard-error)
+     ##sys#standard-input ##sys#standard-output ##sys#standard-error
+     ##sys#undefined-value)
     (bound-to-procedure
      ##sys#for-each ##sys#map ##sys#print ##sys#setter
      ##sys#setslot ##sys#dynamic-wind ##sys#call-with-values
@@ -87,15 +88,18 @@
     disable-stack-overflow-checks raw 
     emit-external-prototypes-first release local inline-global
     analyze-only dynamic scrutinize no-argc-checks no-procedure-checks
+    no-procedure-checks-for-toplevel-bindings
     no-bound-checks no-procedure-checks-for-usual-bindings no-compiler-syntax
     no-parentheses-synonyms no-symbol-escape r5rs-syntax emit-all-import-libraries
-    setup-mode unboxing) )
+    setup-mode unboxing no-module-registration) )
 
 (define valid-compiler-options-with-argument
   '(debug 
     output-file include-path heap-size stack-size unit uses keyword-style require-extension 
-    inline-limit profile-name disable-warning parenthesis-synonyms
-    prelude postlude prologue epilogue nursery extend feature types
+    inline-limit profile-name 
+    disable-warning			; OBSOLETE
+    parenthesis-synonyms
+    prelude postlude prologue epilogue nursery extend feature no-feature types
     emit-import-library emit-inline-file static-extension consult-inline-file
     heap-growth heap-shrinkage heap-initial-size ffi-define ffi-include-path) )
 
@@ -200,6 +204,31 @@
    (lset-union eq? default-standard-bindings default-extended-bindings)
    non-foldable-bindings) )
 
+(for-each
+ (cut mark-variable <> '##compiler#pure 'standard)
+ '(not boolean? eq? eqv? equal? pair? null? list? zero?
+       char? eof-object? symbol? number? complex? real? integer? rational? string?
+       procedure?))
+ 
+(for-each
+ (cut mark-variable <> '##compiler#pure 'extended)
+ '(fx+ fx- fx* fx/ fxmod
+       fx= fx> fx< fx>= fx<= fixnum? fxneg fxmax fxmin identity
+    fxand fxnot fxior fxxor fxshr fxshl fxodd? fxeven?
+    void not-pair? atom? any? u8vector? s8vector? u16vector? s16vector?
+    u32vector? s32vector? f32vector? f64vector?
+    locative?) )
+    
+(for-each
+ (cut mark-variable <> '##compiler#pure '#t)
+ '(##sys#slot ##sys#block-ref ##sys#size ##sys#byte
+    ##sys#pointer? ##sys#generic-structure? ##sys#fits-in-int? ##sys#fits-in-unsigned-int? ##sys#flonum-in-fixnum-range? 
+    ##sys#fudge ##sys#immediate?
+    ##sys#bytevector? ##sys#pair?
+    ##sys#eq? ##sys#list? ##sys#vector? ##sys#eqv? ##sys#get-keyword
+    ##sys#void
+    ##sys#permanent?))
+
 
 ;;; Rewriting-definitions for this platform:
 
@@ -471,7 +500,7 @@
 
 (let ()
   (define (rewrite-c-w-v db classargs cont callargs)
-   ;; (call-with-values <var1> <var2>) -> (let ((k (lambda (r) (<var2> <k0> r)))) (<var1> k))
+   ;; (call-with-values <var1> <var2>) -> (let ((k (lambda (r) [<var2> <k0> r]))) [<var1> k])
    ;; - if <var2> is a known lambda of a single argument
    (and (= 2 (length callargs))
 	(let ((arg1 (car callargs))
@@ -483,7 +512,7 @@
 		 (and (eq? '##core#lambda (node-class val))
 		      (let ((llist (third (node-parameters val))))
 			(and (proper-list? llist)
-			     (= 2 (length (third (node-parameters val))))
+			     (= 2 (length llist))
 			     (let ((tmp (gensym))
 				   (tmpk (gensym 'r)) )
 			       (debugging 'o "removing single-valued `call-with-values'" (node-parameters val))
@@ -537,6 +566,8 @@
 (rewrite 'cdddar 2 1 "C_u_i_cdddar" #f)
 (rewrite 'cddddr 2 1 "C_u_i_cddddr" #f)
 
+(rewrite 'caar 2 1 "C_i_caar" #t)
+(rewrite 'cdar 2 1 "C_i_cdar" #t)
 (rewrite 'cddr 2 1 "C_i_cddr" #t)
 (rewrite 'cdddr 2 1 "C_i_cdddr" #t)
 (rewrite 'cddddr 2 1 "C_i_cddddr" #t)
@@ -603,8 +634,6 @@
 (rewrite '##sys#size 2 1 "C_block_size" #t)
 (rewrite 'fxnot 2 1 "C_fixnum_not" #t)
 (rewrite 'fx* 2 2 "C_fixnum_times" #t)
-(rewrite 'fx/ 2 2 "C_fixnum_divide" #f)
-(rewrite 'fxmod 2 2 "C_fixnum_modulo" #f)
 (rewrite 'fx= 2 2 "C_eqp" #t)
 (rewrite 'fx> 2 2 "C_fixnum_greaterp" #t)
 (rewrite 'fx< 2 2 "C_fixnum_lessp" #t)
@@ -615,8 +644,8 @@
 (rewrite 'fp< 2 2 "C_flonum_lessp" #f)
 (rewrite 'fp>= 2 2 "C_flonum_greater_or_equal_p" #f)
 (rewrite 'fp<= 2 2 "C_flonum_less_or_equal_p" #f)
-(rewrite 'fxmax 2 2 "C_i_fixnum_max" #f)
-(rewrite 'fxmin 2 2 "C_i_fixnum_min" #f)
+(rewrite 'fxmax 2 2 "C_i_fixnum_max" #t)
+(rewrite 'fxmin 2 2 "C_i_fixnum_min" #t)
 (rewrite 'fpmax 2 2 "C_i_flonum_max" #f)
 (rewrite 'fpmin 2 2 "C_i_flonum_min" #f)
 (rewrite 'char-numeric? 2 1 "C_u_i_char_numericp" #t)
@@ -815,11 +844,11 @@
 
 (rewrite 'cons 16 2 "C_a_i_cons" #t 3)
 (rewrite '##sys#cons 16 2 "C_a_i_cons" #t 3)
-(rewrite 'list 16 #f "C_a_i_list" #t '(3))
+(rewrite 'list 16 #f "C_a_i_list" #t '(3) #t)
 (rewrite '##sys#list 16 #f "C_a_i_list" #t '(3))
-(rewrite 'vector 16 #f "C_a_i_vector" #t #t)
+(rewrite 'vector 16 #f "C_a_i_vector" #t #t #t)
 (rewrite '##sys#vector 16 #f "C_a_i_vector" #t #t)
-(rewrite '##sys#make-structure 16 #f "C_a_i_record" #t #t)
+(rewrite '##sys#make-structure 16 #f "C_a_i_record" #t #t #t)
 (rewrite 'string 16 #f "C_a_i_string" #t #t) ; the last #t is actually too much, but we don't care
 (rewrite 'address->pointer 16 1 "C_a_i_address_to_pointer" #f 2)
 (rewrite 'pointer->address 16 1 "C_a_i_pointer_to_address" #f words-per-flonum)
@@ -869,6 +898,8 @@
 (rewrite 'fxxor 17 2 "C_fixnum_xor" "C_fixnum_xor")
 (rewrite 'fxand 17 2 "C_fixnum_and" "C_u_fixnum_and")
 (rewrite 'fxior 17 2 "C_fixnum_or" "C_u_fixnum_or")
+(rewrite 'fx/ 17 2 "C_fixnum_divide" "C_u_fixnum_divide")
+(rewrite 'fxmod 17 2 "C_fixnum_modulo" "C_u_fixnum_modulo")
 
 (rewrite
  'arithmetic-shift 8
