@@ -27,7 +27,7 @@
 (require-library setup-download setup-api)
 (require-library srfi-1 posix data-structures utils regex ports extras srfi-13 files)
 (require-library chicken-syntax)	; in case an import library reexports chicken syntax
-
+(require-library chicken-ffi-syntax)	; same reason, also for filling modules.db
 
 (module main ()
 
@@ -62,6 +62,10 @@
       "irregex.import.so"
       "types.db"))
 
+  (define-constant +defaults-version+ 1)
+  (define-constant +module-db+ "modules.db")
+  (define-constant +defaults-file+ "setup.defaults")
+
   (define *program-path*
     (or (and-let* ((p (get-environment-variable "CHICKEN_PREFIX")))
           (make-pathname p "bin") )
@@ -69,8 +73,6 @@
 
   (define *keep* #f)
   (define *force* #f)
-  (define *prefix* #f)
-  (define *host-extension* #f)
   (define *run-tests* #f)
   (define *retrieve-only* #f)
   (define *no-install* #f)
@@ -86,9 +88,22 @@
   (define *mappings* '())
   (define *deploy* #f)
   (define *trunk* #f)
+  (define *csc-features* '())
+  (define *csc-nonfeatures* '())
+  (define *prefix* #f)
+  (define *aliases* '())
+  (define *cross-chicken* (feature? #:cross-chicken))
+  (define *host-extension* *cross-chicken*)
+  (define *target-extension* *cross-chicken*)
+  (define *debug-setup* #f)
+  (define *keep-going* #f)
 
-  (define-constant +module-db+ "modules.db")
-  (define-constant +defaults-file+ "setup.defaults")
+  (define (get-prefix)
+    (cond ((and *cross-chicken*
+		(not *host-extension*))
+	   (or *prefix*
+	       (foreign-value "C_TARGET_PREFIX" c-string)))
+	  (else *prefix*)))
 
   (define (load-defaults)
     (let ((deff (make-pathname (chicken-home) +defaults-file+)))
@@ -101,6 +116,17 @@
 		(unless (and (list? x) (positive? (length x)))
 		  (broken x))
 		(case (car x)
+		  ((version)
+		   (cond ((not (pair? (cdr x))) (broken x))
+			 ((not (= (cadr x) +defaults-version+))
+			  (error 
+			   (sprintf 
+			       "version of installed `~a' does not match setup-API version (~a)"
+			     +defaults-file+
+			     +defaults-version+)
+			   (cadr x)))
+			 ;; ignored
+			 ))
 		  ((server)
 		   (set! *default-sources* 
 		     (append *default-sources* (list (cdr x)))))
@@ -114,9 +140,26 @@
 			       (let-values (((from to) (split-at m p)))
 				 (cons from (cdr to)))))
 			   (cdr x)))))
+		  ((alias)
+		   (set! *aliases*
+		     (append 
+		      *aliases*
+		      (map (lambda (a)
+			     (if (and (list? a) (= 2 (length a)) (every string? a))
+				 (cons (car a) (cadr a))
+				 (broken x)))
+			   (cdr x)))))
 		  (else (broken x))))
-	      (read-file deff)))
-             (pair? *default-sources*) ) ) )
+	      (read-file deff))))
+      (pair? *default-sources*) ))
+
+  (define (resolve-location name)
+    (cond ((assoc name *aliases*) => 
+	   (lambda (a)
+	     (let ((new (cdr a)))
+	       (print "resolving alias `" name "' to: " new)
+	       (resolve-location new))))
+	  (else name)))
 
   (define (known-default-sources)
     (if (and *default-location* *default-transport*)
@@ -240,8 +283,9 @@
       (if (null? defs)
           (values #f "")
           (let* ([def (car defs)]
-                 [locn (cadr (or (assq 'location def)
-                                 (error "missing location entry" def)))]
+                 [locn (resolve-location
+			(cadr (or (assq 'location def)
+				  (error "missing location entry" def))))]
                  [trans (cadr (or (assq 'transport def)
                                   (error "missing transport entry" def)))])
             (let-values ([(dir ver) (try-extension name version trans locn)])
@@ -312,7 +356,8 @@
                                    (or *force*
                                        (yes-or-no?
                                         (make-replace-extension-question e+d+v upgrade)
-                                        "no") ) )
+                                        "no"
+					abort: abort-setup) ) )
                           (let ([ueggs (unzip1 upgrade)])
                             (print " upgrade: " (string-intersperse ueggs ", "))
                             (for-each
@@ -331,7 +376,7 @@
   (define (check-platform name meta)
     (define (fail)
       (error "extension is not targeted for this system" name))
-    (unless (feature? #:cross-chicken)
+    (unless *cross-chicken*
       (and-let* ((platform (assq 'platform meta)))
 	(let loop ((p (cadr platform)))
 	  (cond ((symbol? p) 
@@ -350,24 +395,49 @@
     (conc
      *csi*
      " -bnq "
-     (if (and (feature? #:cross-chicken) ; disable -setup-mode when cross-compiling,
+     (if (and *cross-chicken* ; disable -setup-mode when cross-compiling,
 	      (not *host-extension*)) ; host-repo must always take precedence
-	 ""				
+	 ""
 	 "-setup-mode ")
-     "-e \"(require-library setup-api)\" -e \"(import setup-api)\""
-     (sprintf " -e \"(extension-name-and-version '(\\\"~a\\\" \\\"~a\\\"))\"" (car e+d+v) (caddr e+d+v))
+     "-e \"(require-library setup-api)\" -e \"(import setup-api)\" "
+     (if *debug-setup*
+	 ""
+	 "-e \"(setup-error-handling)\" ")
+     (sprintf "-e \"(extension-name-and-version '(\\\"~a\\\" \\\"~a\\\"))\""
+       (car e+d+v) (caddr e+d+v))
      (if (sudo-install) " -e \"(sudo-install #t)\"" "")
      (if *keep* " -e \"(keep-intermediates #t)\"" "")
      (if (and *no-install* (not dep?)) " -e \"(setup-install-mode #f)\"" "")
      (if *host-extension* " -e \"(host-extension #t)\"" "")
-     (if *prefix* 
-	 (sprintf " -e \"(destination-prefix \\\"~a\\\")\"" 
-	   (normalize-pathname *prefix* 'unix))
+     (let ((prefix (get-prefix)))
+       (if prefix
+	   (sprintf " -e \"(destination-prefix \\\"~a\\\")\"" 
+	     (normalize-pathname prefix 'unix))
+	   ""))
+     (if (pair? *csc-features*)
+	 (sprintf " -e \"(extra-features '~s)\"" *csc-features*)
+	 "")
+     (if (pair? *csc-nonfeatures*)
+	 (sprintf " -e \"(extra-nonfeatures '~s)\"" *csc-nonfeatures*)
 	 "")
      (if *deploy* " -e \"(deployment-mode #t)\"" "")
      #\space
      (shellpath (make-pathname (cadr e+d+v) (car e+d+v) "setup"))) )
 
+  (define-syntax keep-going
+    (syntax-rules ()
+      ((_ (name mode) body ...)
+       (let ((tmp (lambda () body ...)))
+	 (if *keep-going*
+	     (handle-exceptions ex
+		 (begin
+		   (print mode " extension `" name "' failed:")
+		   (print-error-message ex)
+		   (print "\nnevertheless trying to continue ...")
+		   #f)
+	       (tmp))
+	     (tmp))))))
+
   (define (install eggs)
     (retrieve eggs)
     (unless *retrieve-only*
@@ -386,25 +456,53 @@
 			   (not (yes-or-no?
 				 (string-append
 				  "You specified `-no-install', but this extension has dependencies"
-				  " that are required for building. Do you still want to install them?"))))
+				  " that are required for building.\nDo you still want to install them?")
+				 abort: abort-setup)))
 		 (print "aborting installation.")
 		 (cleanup)
 		 (exit 1)))
 	     (print "installing " (car e+d+v) #\: (caddr e+d+v) " ...")
-	     (print "changing current directory to " (cadr e+d+v))
-	     (parameterize ((current-directory (cadr e+d+v)))
-	       (let ((cmd (make-install-command e+d+v (> i 1))))
-		 (print "  " cmd)
-		 ($system cmd))
-	       (when (and *run-tests*
-			  (not isdep)
-			  (file-exists? "tests")
-			  (directory? "tests")
-			  (file-exists? "tests/run.scm") )
-		 (set! *running-test* #t)
-		 (current-directory "tests")
-		 (command "~a -s run.scm ~a" *csi* (car e+d+v))
-		 (set! *running-test* #f)))))
+	     (let ((tmpcopy (and *target-extension*
+				 *host-extension*
+				 (create-temporary-directory)))
+		   (eggdir (cadr e+d+v)))
+	       (when tmpcopy
+		 (print "copying sources for target installation")
+		 (command 
+		  "~a ~a ~a"
+		  (if *windows-shell* "xcopy" "cp -r")
+		  (make-pathname eggdir "*")
+		  tmpcopy))
+	       (let ((setup
+		      (lambda (dir)
+			(print "changing current directory to " dir)
+			(parameterize ((current-directory dir))
+			  (let ((cmd (make-install-command e+d+v (> i 1)))
+				(name (car e+d+v)))
+			    (print "  " cmd)
+			    (keep-going 
+			     (name "installing")
+			     ($system cmd))
+			    (when (and *run-tests*
+				       (not isdep)
+				       (file-exists? "tests")
+				       (directory? "tests")
+				       (file-exists? "tests/run.scm") )
+			      (set! *running-test* #t)
+			      (current-directory "tests")
+			      (keep-going
+			       (name "testing")
+			       (command "~a -s run.scm ~a ~a" *csi* name (caddr e+d+v)))
+			      (set! *running-test* #f)))))))
+		 (if (and *target-extension* *host-extension*)
+		     (fluid-let ((*deploy* #f)
+				 (*prefix* #f))
+		       (setup eggdir))
+		     (setup eggdir))
+		 (when (and *target-extension* *host-extension*)
+		   (print "installing for target ...")
+		   (fluid-let ((*host-extension* #f))
+		     (setup tmpcopy)))))))
 	 (map (cut assoc <> *eggs+dirs+vers*) dag)
 	 (iota num num -1)))))
 
@@ -496,7 +594,8 @@ usage: chicken-install [OPTION | EXTENSION[:VERSION]] ...
   -r   -retrieve                only retrieve egg into current directory, don't install
   -n   -no-install              do not install, just build (implies `-keep')
   -p   -prefix PREFIX           change installation prefix to PREFIX
-       -host                    when cross-compiling, compile extension for host
+       -host                    when cross-compiling, compile extension only for host
+       -target                  when cross-compiling, compile extension only for target
        -test                    run included test-cases, if available
        -username USER           set username for transports that require this
        -password PASS           set password for transports that require this
@@ -505,38 +604,57 @@ usage: chicken-install [OPTION | EXTENSION[:VERSION]] ...
        -repository              print path used for egg installation
        -deploy                  build extensions for deployment
        -trunk                   build trunk instead of tagged version (only local)
+  -D   -feature FEATURE         features to pass to sub-invocations of `csc'
+       -debug                   enable full display of error message information
+       -keep-going              continue installation even if dependency fails
 EOF
 );|
     (exit code))
 
-  (define *short-options* '(#\h #\k #\l #\t #\s #\p #\r #\n #\v #\i #\u))
+  (define (setup-proxy uri)
+    (if (string? uri)
+        (cond ((string-match "(.+)\\:([0-9]+)" uri) =>
+               (lambda (m)
+                 (set! *proxy-host* (cadr m))
+                 (set! *proxy-port* (string->number (caddr m))))
+               (else
+                (set! *proxy-host* uri)
+                (set! *proxy-port* 80))))))
+  
+  (define *short-options* '(#\h #\k #\l #\t #\s #\p #\r #\n #\v #\i #\u #\D))
 
   (define (main args)
-    (let ((defaults (load-defaults))
-          (update #f)
+    (let ((update #f)
           (rx (regexp "([^:]+):(.+)")))
+      (setup-proxy (get-environment-variable "http_proxy"))
       (let loop ((args args) (eggs '()))
         (cond ((null? args)
-               (cond (update (update-db))
+               (cond ((and *deploy* (not *prefix*))
+		      (error 
+		       "`-deploy' only makes sense in combination with `-prefix DIRECTORY`"))
+		     (update (update-db))
                      (else
-                      (when (null? eggs)
-                        (let ((setups (glob "*.setup")))
-                          (cond ((pair? setups)
-                                 (set! *eggs+dirs+vers*
-				   (append
-				    (map
-				     (lambda (s) (cons (pathname-file s) (list "." "")))
-				     setups)
-				    *eggs+dirs+vers*)))
-                                (else
-                                 (print "no setup-scripts to process")
-                                 (exit 1))) ) )
-                      (unless defaults
-                        (unless *default-transport*
-                          (error "no default transport defined - please use `-transport' option"))
-                        (unless *default-location*
-                          (error "no default location defined - please use `-location' option")))
-                      (install (apply-mappings (reverse eggs))))))
+		      (let ((defaults (load-defaults)))
+			(when (null? eggs)
+			  (let ((setups (glob "*.setup")))
+			    (cond ((pair? setups)
+				   (set! *eggs+dirs+vers*
+				     (append
+				      (map
+				       (lambda (s) (cons (pathname-file s) (list "." "")))
+				       setups)
+				      *eggs+dirs+vers*)))
+				  (else
+				   (print "no setup-scripts to process")
+				   (exit 1))) ) )
+			(unless defaults
+			  (unless *default-transport*
+			    (error
+			     "no default transport defined - please use `-transport' option"))
+			  (unless *default-location*
+			    (error
+			     "no default location defined - please use `-location' option")))
+			(install (apply-mappings (reverse eggs)))))))
               (else
                (let ((arg (car args)))
                  (cond ((or (string=? arg "-help")
@@ -591,21 +709,30 @@ EOF
                         (exit 0))
 		       ((string=? "-proxy" arg)
                         (unless (pair? (cdr args)) (usage 1))
-			(cond ((string-match "(.+)\\:([0-9]+)" (cadr args)) =>
-			       (lambda (m)
-				 (set! *proxy-host* (cadr m))
-				 (set! *proxy-port* (string->number (caddr m)))))
-			      (else
-			       (set! *proxy-host* (cadr args))
-			       (set! *proxy-port* 80)))
+			(setup-proxy (cadr args))
+			(loop (cddr args) eggs))
+		       ((or (string=? "-D" arg) (string=? "-feature" arg))
+                        (unless (pair? (cdr args)) (usage 1))
+			(set! *csc-features* 
+			  (cons (string->symbol (cadr args)) *csc-features*))
+			(loop (cddr args) eggs))
+		       ((string=? "-no-feature" arg)
+                        (unless (pair? (cdr args)) (usage 1))
+			(set! *csc-nonfeatures* 
+			  (cons (string->symbol (cadr args)) *csc-nonfeatures*))
 			(loop (cddr args) eggs))
                        ((string=? "-test" arg)
                         (set! *run-tests* #t)
                         (loop (cdr args) eggs))
-                       ((or (string=? "-host" arg)
-			    (string=? "-host-extension" arg)) ; DEPRECATED
-                        (set! *host-extension* #t)
+                       ((string=? "-host" arg)
+                        (set! *target-extension* #f)
                         (loop (cdr args) eggs))
+                       ((string=? "-target" arg)
+                        (set! *host-extension* #f)
+                        (loop (cdr args) eggs))
+		       ((string=? "-debug" arg)
+			(set! *debug-setup* #t)
+			(loop (cdr args) eggs))
 		       ((string=? "-deploy" arg)
 			(set! *deploy* #t)
 			(loop (cdr args) eggs))
@@ -616,6 +743,9 @@ EOF
 		       ((string=? "-trunk" arg)
 			(set! *trunk* #t)
 			(loop (cdr args) eggs))
+		       ((string=? "-keep-going" arg)
+			(set! *keep-going* #t)
+			(loop (cdr args) eggs))
                        ((string=? "-password" arg)
                         (unless (pair? (cdr args)) (usage 1))
                         (set! *password* (cadr args))
