@@ -128,9 +128,10 @@
        (##sys#start-timer)
        (##sys#call-with-values 
 	(##core#lambda () ,@(cdr form))
-	(##core#lambda ,rvar
-		  (##sys#display-times (##sys#stop-timer))
-		  (##sys#apply ##sys#values ,rvar) ) ) ) ) ) ) )
+	(##core#lambda 
+	 ,rvar
+	 (##sys#display-times (##sys#stop-timer))
+	 (##sys#apply ##sys#values ,rvar) ) ) ) ) ) ) )
 
 (##sys#extend-macro-environment
  'declare '()
@@ -148,20 +149,25 @@
 (##sys#extend-macro-environment
  'assert '()
  (##sys#er-transformer
-  (lambda (form r c)
-    (##sys#check-syntax 'assert form '#(_ 1))
-    (let* ((exp (cadr form))
-	   (msg-and-args (cddr form))
-	   (msg (if (eq? '() msg-and-args)
-		    `(##core#immutable '"assertion failed")
-		    (car msg-and-args) ) ) )
-      `(##core#if (##core#check ,exp)
-		  (##core#undefined)
-		  (##sys#error 
-		   ,msg 
-		   ,@(if (fx> (length msg-and-args) 1)
-			 (cdr msg-and-args)
-			 `((##core#quote ,(##sys#strip-syntax exp))))))))))
+  (let ((string-append string-append))
+    (lambda (form r c)
+      (##sys#check-syntax 'assert form '#(_ 1))
+      (let* ((exp (cadr form))
+	     (ln (get-line-number form))
+	     (msg-and-args (cddr form))
+	     (msg  (if (null? msg-and-args)
+		       "assertion failed"
+		       (car msg-and-args)))
+	     (msg (if ln
+		      (string-append "(" ln ") " msg)
+		      msg)))
+	`(##core#if (##core#check ,exp)
+		    (##core#undefined)
+		    (##sys#error 
+		     ,msg 
+		     ,@(if (fx> (length msg-and-args) 1)
+			   (cdr msg-and-args)
+			   `((##core#quote ,(##sys#strip-syntax exp)))))))))))
 
 (##sys#extend-macro-environment
  'ensure
@@ -487,7 +493,7 @@
 			   `(##core#begin ,@(cdr clause)) )
 			  (else?
 			   (##sys#notice
-			    "non-`else' clause following `else' clause in `cond'"
+			    "non-`else' clause following `else' clause in `select'"
 			    (##sys#strip-syntax clause))
 			   (expand rclauses #t)
 			   '(##core#begin))
@@ -638,7 +644,7 @@
 	     (body-proc (r 'body))
 
 	     ;; A private var, bound to the value of the ARG-LIST expression.
-	     (rest-var (r '%rest))
+	     (rest-var (r '_%rest))
 
 	     (defaulter-names (map (lambda (var) (r (prefix-sym "def-" var)))
 				   vars))
