@@ -44,6 +44,7 @@
  canonicalize-begin-body
  canonicalize-expression
  check-and-open-input-file
+ check-for-unsafe-toplevel-procedure-calls
  check-signature
  chop-extension
  chop-separator
@@ -57,9 +58,7 @@
  compiler-source-file
  compiler-syntax-enabled
  compiler-syntax-statistics
- compiler-warning
  compute-database-statistics
- constant-declarations
  constant-table
  constant?
  constants-used
@@ -173,7 +172,6 @@
  read-info-hook
  initialize-analysis-database
  initialize-compiler
- inline-globally
  inline-lambda-bindings
  inline-locally
  inline-max-size
@@ -194,7 +192,7 @@
  load-type-database
  local-definitions
  location-pointer-map
-  make-argument-list
+ make-argument-list
  make-block-variable-literal
  make-random-name
  make-variable-list
@@ -204,6 +202,8 @@
  membership-unfold-limit
  no-argc-checks
  no-bound-checks
+ no-global-procedure-checks
+ enable-module-registration
  no-procedure-checks
  node->sexpr
  non-foldable-bindings
