@@ -61,25 +61,28 @@
   (let ((%let (r 'let))
 	(%if (r 'if))
 	(%loop (r 'loop))
+	(%proc (gensym))
 	(%begin (r 'begin))
 	(%and (r 'and))
 	(%pair? (r 'pair?))
 	(%lambda (r 'lambda))
 	(lsts (cddr x)))
     (if (and (memq 'for-each standard-bindings) ; we have to check this because the db (and thus 
-	     (> (length+ x) 2)			 ; intrinsic marks) isn't set up yet
-	     (or (and (pair? (cadr x))
-		      (c %lambda (caadr x)))
-		 (symbol? (cadr x))))
+	     (> (length+ x) 2))			 ; intrinsic marks) isn't set up yet
 	(let ((vars (map (lambda _ (gensym)) lsts)))
-	  `(,%let ,%loop ,(map list vars lsts)
-		  (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
-			(,%begin
-			 ((,%begin ,(cadr x))
-			  ,@(map (lambda (v) `(##sys#slot ,v 0)) vars))
-			 (##core#app 
-			  ,%loop
-			  ,@(map (lambda (v) `(##sys#slot ,v 1)) vars) ) ))))
+	  `(,%let ((,%proc ,(cadr x))
+		   ,@(map list vars lsts))
+		  ,@(map (lambda (var)
+			   `(##core#check (##sys#check-list ,var 'for-each)))
+			 vars)
+		  (,%let ,%loop ,(map list vars vars)
+			 (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
+			       (,%begin
+				(,%proc
+				 ,@(map (lambda (v) `(##sys#slot ,v 0)) vars))
+				(##core#app 
+				 ,%loop
+				 ,@(map (lambda (v) `(##sys#slot ,v 1)) vars) ) )))))
 	x)))
 
 (define-internal-compiler-syntax ((map ##sys#map #%map) x r c)
@@ -87,11 +90,12 @@
   (let ((%let (r 'let))
 	(%if (r 'if))
 	(%loop (r 'loop))
-	(%res (r 'res))
+	(%res (gensym))
 	(%cons (r 'cons))
 	(%set! (r 'set!))
-	(%result (r 'result))
-	(%node (r 'node))
+	(%result (gensym))
+	(%node (gensym))
+	(%proc (gensym))
 	(%quote (r 'quote))
 	(%begin (r 'begin))
 	(%lambda (r 'lambda))
@@ -99,27 +103,30 @@
 	(%pair? (r 'pair?))
 	(lsts (cddr x)))
     (if (and (memq 'map standard-bindings) ; s.a.
-	     (> (length+ x) 2)
-	     (or (and (pair? (cadr x))
-		      (c %lambda (caadr x)))
-		 (symbol? (cadr x))))
+	     (> (length+ x) 2))
 	(let ((vars (map (lambda _ (gensym)) lsts)))
 	  `(,%let ((,%result (,%quote ()))
-		   (,%node #f))
-		  (,%let ,%loop ,(map list vars lsts)
-		       (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
-			     (,%let ((,%res
-				      (,%cons
-				       ((,%begin ,(cadr x))
-					,@(map (lambda (v) `(##sys#slot ,v 0)) vars))
-				       (,%quote ()))))
-				    (,%if ,%node
-					  (##sys#setslot ,%node 1 ,%res)
-					  (,%set! ,%result ,%res))
-				    (,%set! ,%node ,%res)
-				    (,%loop
-				     ,@(map (lambda (v) `(##sys#slot ,v 1)) vars)))
-			     ,%result))))
+		   (,%node #f)
+		   (,%proc ,(cadr x))
+		   ,@(map list vars lsts))		   
+		  ,@(map (lambda (var)
+			   `(##core#check (##sys#check-list ,var 'map)))
+			 vars)
+		  (,%let ,%loop ,(map list vars vars)
+			 (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
+			       (,%let ((,%res
+					(,%cons
+					 (,%proc
+					  ,@(map (lambda (v) `(##sys#slot ,v 0)) vars))
+					 (,%quote ()))))
+				      (,%if ,%node
+					    (##sys#setslot ,%node 1 ,%res)
+					    (,%set! ,%result ,%res))
+				      (,%set! ,%node ,%res)
+				      (##core#app
+				       ,%loop
+				       ,@(map (lambda (v) `(##sys#slot ,v 1)) vars)))
+			       ,%result))))
 	x)))
 
 (define-internal-compiler-syntax ((o #%o) x r c) ()
@@ -177,12 +184,11 @@
 		(args (cdr args)))
 	    (define (fail ret? msg . args)
 	      (let ((ln (get-line x)))
-		(compiler-warning 
-		 'syntax
-		 "`~a', in format string ~s~a, ~?" 
-		 func fstr 
-		 (if ln (sprintf "(~a)" ln) "")
-		 msg args) ) 
+		(warning 
+		 (sprintf "`~a', in format string ~s~a, ~?" 
+		   func fstr 
+		   (if ln (sprintf "(~a)" ln) "")
+		   msg args) ))
 	      (when ret? (return #f)))
 	    (let ((code '())
 		  (index 0)
