@@ -50,7 +50,6 @@
 ; (c-options {<opt>})
 ; (compile-syntax)
 ; (disable-interrupts)
-; (disable-warning <class> ...)
 ; (emit-import-library {<module> | (<module> <filename>)})
 ; (export {<name>})
 ; (fixnum-arithmetic)
@@ -59,11 +58,11 @@
 ; (inline-limit <limit>)
 ; (keep-shadowed-macros)
 ; (lambda-lift)
-; (link-options {<opt>})
 ; (no-argc-checks)
 ; (no-bound-checks)
 ; (no-procedure-checks)
 ; (no-procedure-checks-for-usual-bindings)
+; (no-procedure-checks-for-toplevel-bindings)
 ; (post-process <string> ...)
 ; (profile <symbol> ...)
 ; (safe-globals)
@@ -90,6 +89,7 @@
 ;   ##compiler#profile -> BOOL
 ;   ##compiler#unused -> BOOL
 ;   ##compiler#foldable -> BOOL
+;   ##compiler#pure -> 'standard | 'extended | BOOL
 
 ; - Source language:
 ;
@@ -109,6 +109,7 @@
 ; (##core#lambda ({<variable>}+ [. <variable>]) <body>)
 ; (##core#set! <variable> <exp>)
 ; (##core#begin <exp> ...)
+; (##core#toplevel-begin <exp> ...)
 ; (##core#include <string>)
 ; (##core#loop-lambda <llist> <body>)
 ; (##core#undefined)
@@ -255,8 +256,7 @@
 
 
 (declare
- (unit compiler)
- (disable-warning var) )
+ (unit compiler))
 
 
 (include "compiler-namespace")
@@ -314,6 +314,7 @@
 (define no-bound-checks #f)
 (define no-argc-checks #f)
 (define no-procedure-checks #f)
+(define no-global-procedure-checks #f)
 (define source-filename #f)
 (define safe-globals-flag #f)
 (define explicit-use-flag #f)
@@ -324,13 +325,12 @@
 (define inline-max-size default-inline-max-size)
 (define emit-closure-info #t)
 (define undefine-shadowed-macros #t)
-(define constant-declarations '())
 (define profiled-procedures #f)
 (define import-libraries '())
 (define all-import-libraries #f)
+(define enable-module-registration #t)
 (define standalone-executable #t)
 (define local-definitions #f)
-(define inline-globally #f)
 (define inline-locally #f)
 (define inline-output-file #f)
 (define do-scrutinize #f)
@@ -508,21 +508,20 @@
     (cond ((symbol? x)
 	   (cond ((keyword? x) `(quote ,x))
 		 ((memq x unlikely-variables)
-		  (compiler-warning 
-		   'var
-		   "reference to variable `~s' possibly unintended" x) ))
+		  (warning 
+		   (sprintf "reference to variable `~s' possibly unintended" x) )))
 	   (resolve-variable x e se dest))
 	  ((not-pair? x)
 	   (if (constant? x)
 	       `(quote ,x)
-	       (syntax-error "illegal atomic form" x)))
+	       (##sys#syntax-error/context "illegal atomic form" x)))
 	  ((symbol? (car x))
 	   (let ([ln (get-line x)])
 	     (emit-syntax-trace-info x #f)
 	     (unless (proper-list? x)
 	       (if ln
-		   (syntax-error (sprintf "(~a) - malformed expression" ln) x)
-		   (syntax-error "malformed expression" x)))
+		   (##sys#syntax-error/context (sprintf "(~a) - malformed expression" ln) x)
+		   (##sys#syntax-error/context "malformed expression" x)))
 	     (set! ##sys#syntax-error-culprit x)
 	     (let* ((name0 (lookup (car x) se))
 		    (name (or (and (symbol? name0) (##sys#get name0 '##core#primitive)) name0))
@@ -601,8 +600,8 @@
 							    (##sys#canonicalize-extension-path 
 							     id 'require-extension)
 							    #f)) ) ) 
-					(compiler-warning 
-					 'ext "extension `~A' is currently not installed" id))
+					(warning 
+					 (sprintf "extension `~A' is currently not installed" id)))
 				      `(##core#begin ,exp ,(loop (cdr ids))) ) ) ) )
 			    e se dest) ) )
 
@@ -829,10 +828,15 @@
 						(cond ((or all-import-libraries
 							   (assq name import-libraries) ) =>
 						       (lambda (il)
-							 (emit-import-lib name il)
+							 (when enable-module-registration
+							   (emit-import-lib name il))
 							 (values 
 							  (reverse xs)
 							  '((##core#undefined)))))
+						      ((not enable-module-registration)
+						       (values 
+							(reverse xs)
+							'((##core#undefined))))
 						      (else
 						       (values
 							(reverse xs)
@@ -886,10 +890,9 @@
 				[ln (get-line x)]
 				[val (caddr x)] )
 			   (when (memq var unlikely-variables)
-			     (compiler-warning 
-			      'var
-			      "assignment to variable `~s' possibly unintended"
-			      var))
+			     (warning 
+			      (sprintf "assignment to variable `~s' possibly unintended"
+				var)))
 			   (cond ((assq var foreign-variables)
 				   => (lambda (fv)
 					(let ([type (second fv)]
@@ -918,17 +921,17 @@
 				    (when safe-globals-flag
 				      (mark-variable var '##compiler#always-bound-to-procedure)
 				      (mark-variable var '##compiler#always-bound)))
-				  (when (##sys#macro? var)
-				    (compiler-warning 
-				     'var "assigned global variable `~S' is a macro ~A"
-				     var
-				     (if ln (sprintf "(~a)" ln) "") )
-				    (when undefine-shadowed-macros (##sys#undefine-macro! var) ) )
+				  (cond ((##sys#macro? var)
+					 (warning 
+					  (sprintf "assigned global variable `~S' is syntax ~A"
+					    var
+					    (if ln (sprintf "(~a)" ln) "") ))
+					 (when undefine-shadowed-macros (##sys#undefine-macro! var) ) )
+					((and ##sys#notices-enabled
+					      (assq var (##sys#current-environment)))
+					 (##sys#notice "assignment to imported value binding" var)))
 				  (when (keyword? var)
-				    (compiler-warning 'syntax "assignment to keyword `~S'" var) )
-				  (when (pair? var) ; macro
-				    (syntax-error
-				     'set! "assignment to syntactic identifier" var))
+				    (warning (sprintf "assignment to keyword `~S'" var) ))
 				  `(set! ,var ,(walk val e se var0))))))
 
 			((##core#inline)
@@ -957,7 +960,7 @@
 			 (eval/meta (cadr x))
 			 '(##core#undefined) )
 
-			((##core#begin) 
+			((##core#begin ##core#toplevel-begin) 
 			 (if (pair? (cdr x))
 			     (canonicalize-begin-body
 			      (let fold ([xs (cdr x)])
@@ -1146,7 +1149,8 @@
 						    nonnull-c-string))
 						 `((##sys#make-c-string
 						    (##core#let
-						     () ,@(cddr lam)))))
+						     () ,@(cddr lam))
+                                                    ',name)))
 						((member 
 						  rtype
 						  '((const c-string*)
@@ -1169,7 +1173,7 @@
 						    ((r (##core#let () ,@(cddr lam))))
 						    (,(macro-alias 'and se)
 						     r 
-						     (##sys#make-c-string r)) ) ) )
+						     (##sys#make-c-string r ',name)) ) ) )
 						(else (cddr lam)) ) )
 					   rtype) ) )
 				      e se #f) ) ) ) )
@@ -1191,9 +1195,10 @@
 			       (walk `(##sys#make-locative ,sym 0 #f 'location) e se #f) ) ) )
 				 
 			(else
-			 (let* ([x2 (mapwalk x e se)]
-				[head2 (car x2)]
-				[old (##sys#hash-table-ref line-number-database-2 head2)] )
+			 (let* ((x2 (fluid-let ((##sys#syntax-context (cons name ##sys#syntax-context)))
+				      (mapwalk x e se)))
+				(head2 (car x2))
+				(old (##sys#hash-table-ref line-number-database-2 head2)) )
 			   (when ln
 			     (##sys#hash-table-set!
 			      line-number-database-2
@@ -1202,11 +1207,11 @@
 			   x2) ) ) ] ) ) ) )
 
 	  ((not (proper-list? x))
-	   (syntax-error "malformed expression" x) )
+	   (##sys#syntax-error/context "malformed expression" x) )
 
 	  ((constant? (car x))
 	   (emit-syntax-trace-info x #f)
-	   (compiler-warning 'syntax "literal in operator position: ~S" x) 
+	   (warning "literal in operator position" x) 
 	   (mapwalk x e se) )
 
 	  (else
@@ -1245,6 +1250,14 @@
   (define (strip x)			; raw symbol
     (##sys#strip-syntax x se))
   (define stripu ##sys#strip-syntax)
+  (define (globalize sym)
+    (if (symbol? sym)
+	(let loop ((se se))			; ignores syntax bindings
+	  (cond ((null? se) (##sys#alias-global-hook sym #f))
+		((and (eq? sym (caar se)) (symbol? (cdar se))) (cdar se))
+		(else (loop (cdr se)))))
+	sym))
+  (define (globalize-all syms) (map globalize syms))
   (call-with-current-continuation
    (lambda (return)
      (unless (pair? spec)
@@ -1266,7 +1279,7 @@
 	(let* ([u (stripu (cadr spec))]
 	       [un (string->c-identifier (stringify u))] )
 	  (when (and unit-name (not (string=? unit-name un)))
-	    (compiler-warning 'usage "unit was already given a name (new name is ignored)") )
+	    (warning "unit was already given a name (new name is ignored)") )
 	  (set! unit-name un) ) )
        ((standard-bindings)
 	(if (null? (cdr spec))
@@ -1296,9 +1309,6 @@
        ((no-procedure-checks) (set! no-procedure-checks #t))
        ((interrupts-enabled) (set! insert-timer-checks #t))
        ((disable-interrupts) (set! insert-timer-checks #f))
-       ((disable-warning)
-	(set! disabled-warnings
-	  (append (strip (cdr spec)) disabled-warnings)))
        ((always-bound) 
 	(for-each (cut mark-variable <> '##compiler#always-bound) (stripa (cdr spec))))
        ((safe-globals) (set! safe-globals-flag #t))
@@ -1309,8 +1319,10 @@
 	(for-each
 	 (cut mark-variable <> '##compiler#always-bound)
 	 (append default-standard-bindings default-extended-bindings)))
+       ((no-procedure-checks-for-toplevel-bindings)
+	(set! no-global-procedure-checks #t))
        ((bound-to-procedure)
-	(let ((vars (stripa (cdr spec))))
+	(let ((vars (globalize-all (cdr spec))))
 	  (for-each (cut mark-variable <> '##compiler#always-bound-to-procedure) vars)
 	  (for-each (cut mark-variable <> '##compiler#always-bound) vars)))
        ((foreign-declare)
@@ -1322,7 +1334,7 @@
        ((separate) (set! block-compilation #f))
        ((keep-shadowed-macros) (set! undefine-shadowed-macros #f))
        ((unused)
-	(for-each (cut mark-variable <> '##compiler#unused) (stripa (cdr spec))))
+	(for-each (cut mark-variable <> '##compiler#unused) (globalize-all (cdr spec))))
        ((not)
 	(check-decl spec 1)
 	(case (##sys#strip-syntax (second spec)) ; strip all
@@ -1343,7 +1355,7 @@
 	       (set! inline-locally #f)
 	       (for-each 
 		(cut mark-variable <> '##compiler#inline 'no)
-		(stripa (cddr spec)))) ]
+		(globalize-all (cddr spec)))) ]
 	  [(usual-integrations)      
 	   (cond [(null? (cddr spec))
 		  (set! standard-bindings '())
@@ -1354,28 +1366,27 @@
 		    (set! extended-bindings (lset-difference eq? default-extended-bindings syms)) ) ] ) ]
 	  ((inline-global)
 	   (set! enable-inline-files #t)
-	   (if (null? (cddr spec))
-	       (set! inline-globally #f)
-	       (for-each
-		(cut mark-variable <> '##compiler#inline-global 'no)
-		(stripa (cddr spec)))))
+	   (when (pair? (cddr spec))
+	     (for-each
+	      (cut mark-variable <> '##compiler#inline-global 'no)
+	      (globalize-all (cddr spec)))))
 	  [else
 	   (check-decl spec 1 1)
 	   (let ((id (strip (cadr spec))))
 	     (case id
 	       [(interrupts-enabled) (set! insert-timer-checks #f)]
 	       [(safe) (set! unsafe #t)]
-	       [else (compiler-warning 'syntax "illegal declaration specifier `~s'" id)]))]))
-       ((compile-syntax )
+	       [else (warning "unsupported declaration specifier" id)]))]))
+       ((compile-syntax)
 	(set! ##sys#enable-runtime-macros #t))
        ((block-global hide) 
-	(let ([syms (stripa (cdr spec))])
+	(let ([syms (globalize-all (cdr spec))])
 	  (if (null? syms)
 	      (set! block-compilation #t)
 	      (for-each hide-variable syms))))
        ((export)
 	(set! block-compilation #t)
-	(let ((syms (stripa (cdr spec))))
+	(let ((syms (globalize-all (cdr spec))))
 	  (for-each export-variable syms)))
        ((emit-external-prototypes-first)
 	(set! external-protos-first #t) )
@@ -1385,19 +1396,21 @@
 	    (set! inline-locally #t)
 	    (for-each
 	     (cut mark-variable <> '##compiler#inline 'yes)
-	     (stripa (cdr spec)))))
+	     (globalize-all (cdr spec)))))
        ((inline-limit)
 	(check-decl spec 1 1)
 	(let ([n (cadr spec)])
 	  (if (number? n)
 	      (set! inline-max-size n)
-	      (compiler-warning 
-	       'syntax
-	       "invalid argument to `inline-limit' declaration: ~s" spec) ) ) )
+	      (warning 
+	       "invalid argument to `inline-limit' declaration"
+	       spec) ) ) )
        ((constant)
 	(let ((syms (cdr spec)))
 	  (if (every symbol? syms)
-	      (set! constant-declarations (append syms constant-declarations))
+	      (for-each 
+	       (cut mark-variable <> '##compiler#pure #t) 
+	       (globalize-all syms))
 	      (quit "invalid arguments to `constant' declaration: ~S" spec)) ) )
        ((emit-import-library)
 	(set! import-libraries
@@ -1410,19 +1423,18 @@
 			      (symbol? (car il)) (string (cadr il)))
 			 (cons (car il) (cadr il))) 
 			(else
-			 (compiler-warning 
-			  'syntax
-			  "invalid import-library specification: ~s" il))))
+			 (warning 
+			  "invalid import-library specification" il))))
 		(strip (cdr spec))))))
        ((profile)
 	(set! emit-profile #t)
 	(cond ((null? (cdr spec))
 	       (set! profiled-procedures 'all) )
 	      (else
-	       (set! profiled-propcedures 'some)
+	       (set! profiled-procedures 'some)
 	       (for-each 
 		(cut mark-variable <> '##compiler#profile)
-		(stripa (cdr spec))))))
+		(globalize-all (cdr spec))))))
        ((local)
 	(cond ((null? (cdr spec))
 	       (set! local-definitions #t) )
@@ -1433,11 +1445,10 @@
        ((inline-global)
 	(set! enable-inline-files #t)
 	(set! inline-locally #t)
-	(if (null? (cdr spec))
-	    (set! inline-globally #t)
-	    (for-each
-	     (cut mark-variable <> '##compiler#inline-global 'yes)
-	     (stripa (cdr spec)))))
+	(when (pair? (cdr spec))
+	  (for-each
+	   (cut mark-variable <> '##compiler#inline-global 'yes)
+	   (globalize-all (cdr spec)))))
        ((type)
 	(for-each
 	 (lambda (spec)
@@ -1445,11 +1456,11 @@
 		  (##sys#put! (car spec) '##core#type (cadr spec))
 		  (##sys#put! (car spec) '##core#declared-type #t))
 		 (else
-		  (compiler-warning 'syntax "illegal `type' declaration item `~s'" spec))))
-	 (cdr spec)))
+		  (warning "illegal `type' declaration item" spec))))
+	 (globalize-all (cdr spec))))
        ((scrutinize)
 	(set! do-scrutinize #t))
-       (else (compiler-warning 'syntax "illegal declaration specifier `~s'" spec)) )
+       (else (warning "illegal declaration specifier" spec)) )
      '(##core#undefined) ) ) )
 
 
@@ -1676,8 +1687,8 @@
 ;;; Perform source-code analysis:
 
 (define (analyze-expression node)
-  (let ([db (make-vector analysis-database-size '())]
-	[explicitly-consed '()] )
+  (let ((db (make-vector analysis-database-size '()))
+	(explicitly-consed '()) )
 
     (define (grow n)
       (set! current-program-size (+ current-program-size n)) )
@@ -1714,7 +1725,7 @@
 	   (grow 1)
 	   (let ([fun (car subs)])
 	     (when (eq? '##core#variable (node-class fun))
-	       (let ([name (first (node-parameters fun))])
+	       (let ((name (first (node-parameters fun))))
 		 (collect! db name 'call-sites (cons here n))))
 	     (walk (first subs) env localenv here #t)
 	     (walkeach (cdr subs) env localenv here #f) ) )
@@ -1772,14 +1783,14 @@
 		  (set-car! (cdddr (node-parameters n)) (- current-program-size size0)) ) ) ) ) )
 	  
 	  ((set! ##core#set!) 
-	   (let* ([var (first params)]
-		  [val (car subs)] )
+	   (let* ((var (first params))
+		  (val (car subs)) )
 	     (when first-analysis 
 	       (case (variable-mark var '##compiler#intrinsic)
 		 ((standard)
-		  (compiler-warning 'redef "redefinition of standard binding `~S'" var) )
+		  (warning "redefinition of standard binding" var) )
 		 ((extended)
-		  (compiler-warning 'redef "redefinition of extended binding `~S'" var) ) )
+		  (warning "redefinition of extended binding" var) ) )
 	       (put! db var 'potential-value val) )
 	     (unless (memq var localenv)
 	       (grow 1)
@@ -1793,7 +1804,7 @@
 	     (walk (car subs) env localenv here #f) ) )
 
 	  ((##core#primitive ##core#inline)
-	   (let ([id (first params)])
+	   (let ((id (first params)))
 	     (when (and first-analysis here (symbol? id) (##sys#hash-table-ref real-name-table id))
 	       (set-real-name! id here) )
 	     (walkeach subs env localenv here #f) ) )
@@ -1906,11 +1917,12 @@
 		    (null? references)
 		    (not (variable-mark sym '##compiler#unused)))
 	   (when assigned-locally
-	     (compiler-warning 'var "local assignment to unused variable `~S' may be unintended" sym) )
+	     (##sys#notice 
+	      (sprintf "local assignment to unused variable `~S' may be unintended" sym) ) )
 	   (when (and (not (variable-visible? sym))
 		      (not (variable-mark sym '##compiler#constant)) )
 	     (##sys#notice 
-	      (sprintf "global variable `~S' is never used" sym) ) ) )
+	      (sprintf "global variable `~S' is only locally visible and never used" sym) ) ) )
 
  	 ;; Make 'boxed, if 'assigned & 'captured:
 	 (when (and assigned captured)
@@ -1996,14 +2008,16 @@
 			     (set! explicitly-consed (cons rest explicitly-consed))
 			     (put! db (first lparams) 'explicit-rest #t) ] ) ) ) ) ) ) ) )
 
-	 ;;  Make 'removable, if it has no references and is not assigned to, and if it has either a value that
-	 ;;    does not cause any side-effects or if it is 'undefined:
+	 ;; Make 'removable, if it has no references and is not assigned to, and if it 
+	 ;; has either a value that does not cause any side-effects or if it is 'undefined:
 	 (when (and (not assigned)
 		    (null? references)
 		    (or (and value
-			     (or (not (eq? '##core#variable (node-class value)))
-				 (not (get db (first (node-parameters value)) 'global)) )
-			     (not (expression-has-side-effects? value db)) )
+			     (if (eq? '##core#variable (node-class value))
+				 (let ((varname (first (node-parameters value))))
+				   (or (not (get db varname 'global))
+				       (not (variable-mark varname '##core#always-bound))))
+				 (not (expression-has-side-effects? value db)) ))
 			undefined) )
 	   (quick-put! plist 'removable #t) )
 
@@ -2055,9 +2069,46 @@
     ;; Set original program-size, if this is the first analysis-pass:
     (unless original-program-size
       (set! original-program-size current-program-size) )
+
+    ;; return database
     db) )
 
 
+;;; Collect unsafe global procedure calls that are assigned:
+
+(define (check-for-unsafe-toplevel-procedure-calls node db)
+  (let ((procs '()))
+
+    (define (walk n)
+      (let ((subs (node-subexpressions n))
+	    (params (node-parameters n)) 
+	    (class (node-class n)) )
+	(case class
+	  ((##core#call)
+	   (let ((fun (first subs)))
+	     (when (memq (node-class fun) '(##core#variable ##core#global-ref))
+	       (let ((name (first (node-parameters fun))))
+		 (when (and ##sys#notices-enabled
+			    (get db name 'global)
+			    (get db name 'assigned)
+			    (variable-visible? name)
+			    (or no-global-procedure-checks
+				(variable-mark name '##compiler#always-bound-to-procedure))
+			    (not unsafe))
+		   (set! procs (lset-adjoin eq? procs name))))))
+	   (for-each walk subs))
+	  (else (for-each walk subs)))))
+
+    (when ##sys#notices-enabled
+      (walk node)
+      (when (pair? procs)
+	(##sys#notice
+	 "the following non-intrinsic global procedures where declared to be unsafe but are externally visible:")
+	(newline (current-error-port))
+	(for-each (cute fprintf (current-error-port) "  ~S~%" <>) procs)
+	(flush-output (current-error-port))))))
+
+
 ;;; Convert closures to explicit data structures (effectively flattens function-binding structure):
 
 (define (perform-closure-conversion node db)
@@ -2515,10 +2566,6 @@
 			 (make-node '##core#setlocal (list i) (list (walk val e here boxes)) ) ) )
 		   (else
 		    (let* ([cval (node-class val)]
-			   [safe (not (or no-bound-checks
-					  unsafe
-					  (variable-mark var '##compiler#always-bound)
-					  (intrinsic? var)))]
 			   [blockvar (not (variable-visible? var))]
 			   [immf (or (and (eq? cval 'quote) (immediate? (first (node-parameters val))))
 				     (eq? '##core#undefined cval) ) ] )
@@ -2550,9 +2597,10 @@
 		   ((number? c)
 		    (cond ((eq? 'fixnum number-type)
 			   (cond ((and (integer? c) (not (big-fixnum? c)))
-				  (compiler-warning 
-				   'type 
-				   "coerced inexact literal number `~S' to fixnum ~S" c (inexact->exact c))
+				  (warning 
+				   (sprintf 
+				       "coerced inexact literal number `~S' to fixnum ~S" 
+				     c (inexact->exact c)))
 				  (immediate-literal (inexact->exact c)) )
 				 (else (quit "cannot coerce inexact literal `~S' to fixnum" c)) ) )
 			  (else (make-node '##core#literal (list (literal c)) '())) ) )
