@@ -39,6 +39,7 @@
 (define-foreign-variable TARGET_CFLAGS c-string "C_TARGET_CFLAGS")
 (define-foreign-variable INSTALL_CFLAGS c-string "C_INSTALL_CFLAGS")
 (define-foreign-variable TARGET_LDFLAGS c-string "C_TARGET_LDFLAGS")
+(define-foreign-variable TARGET_FEATURES c-string "C_TARGET_FEATURES")
 (define-foreign-variable INSTALL_LDFLAGS c-string "C_INSTALL_LDFLAGS")
 (define-foreign-variable INSTALL_MORE_LIBS c-string "C_INSTALL_MORE_LIBS")
 (define-foreign-variable INSTALL_MORE_STATIC_LIBS c-string "C_INSTALL_MORE_STATIC_LIBS")
@@ -125,6 +126,7 @@
 (define best-compilation-optimization-options default-compilation-optimization-options)
 (define default-linking-optimization-options (string-split (if host-mode INSTALL_LDFLAGS TARGET_LDFLAGS)))
 (define best-linking-optimization-options default-linking-optimization-options)
+(define extra-features (if host-mode '() (string-split TARGET_FEATURES)))
 
 (define-constant simple-options
   '(-explicit-use -no-trace -no-warnings -no-usual-integrations -optimize-leaf-routines -unsafe
@@ -135,15 +137,18 @@
     -analyze-only -keep-shadowed-macros -inline-global -ignore-repository
     -no-symbol-escape -no-parentheses-synonyms -r5rs-syntax
     -no-argc-checks -no-bound-checks -no-procedure-checks -no-compiler-syntax
-    -emit-all-import-libraries -setup-mode -unboxing -no-elevation
-    -no-procedure-checks-for-usual-bindings))
+    -emit-all-import-libraries -setup-mode -unboxing -no-elevation -no-module-registration
+    -no-procedure-checks-for-usual-bindings
+    -no-procedure-checks-for-toplevel-bindings))
 
 (define-constant complex-options
   '(-debug -output-file -heap-size -nursery -stack-size -compiler -unit -uses -keyword-style
     -optimize-level -include-path -database-size -extend -prelude -postlude -prologue -epilogue 
-    -inline-limit -profile-name -disable-warning -emit-inline-file -types
+    -inline-limit -profile-name
+    -disable-warning			; OBSOLETE
+    -emit-inline-file -types
     -feature -debug-level -heap-growth -heap-shrinkage -heap-initial-size -consult-inline-file
-    -emit-import-library -static-extension))
+    -emit-import-library -static-extension -no-feature))
 
 (define-constant shortcuts
   '((-h "-help")
@@ -156,7 +161,7 @@
     (-i "-case-insensitive")
     (|-K| "-keyword-style")
     (|-X| "-extend")
-    (|-N| "-no-usual-integrations")
+    (|-N| "-no-usual-integrations")	; DEPRECATED
     (|-J| "-emit-all-import-libraries")
     (-x "-explicit-use")
     (-u "-unsafe")
@@ -308,6 +313,7 @@ Usage: #{csc} FILENAME | OPTION ...
   Language options:
 
     -D  -DSYMBOL  -feature SYMBOL  register feature identifier
+    -no-feature SYMBOL             disable builtin feature identifier
     -c++                           compile via a C++ source file (.cpp) 
     -objc                          compile via Objective-C source file (.m)
 
@@ -324,6 +330,7 @@ Usage: #{csc} FILENAME | OPTION ...
     -j -emit-import-library MODULE write compile-time module information into
                                     separate file
     -J -emit-all-import-libraries  emit import-libraries for all defined modules
+    -no-module-registration        do not generate module registration code
     -no-compiler-syntax            disable expansion of compiler-macros
 
   Translation options:
@@ -336,7 +343,6 @@ Usage: #{csc} FILENAME | OPTION ...
   Debugging options:
 
     -w  -no-warnings               disable warnings
-    -disable-warning CLASS         disable specific class of warnings
     -d0 -d1 -d2 -debug-level NUMBER
                                    set level of available debugging information
     -no-trace                      disable rudimentary debugging information
@@ -350,10 +356,10 @@ Usage: #{csc} FILENAME | OPTION ...
 
   Optimization options:
 
-    -O -O1 -O2 -O3 -O4 -O5 -optimize-level NUMBER
+    -O -O0 -O1 -O2 -O3 -O4 -O5 -optimize-level NUMBER
                                    enable certain sets of optimization options
     -optimize-leaf-routines        enable leaf routine optimization
-    -N  -no-usual-integrations     standard procedures may be redefined
+    -no-usual-integrations         standard procedures may be redefined
     -u  -unsafe                    disable safety checks
     -local                         assume globals are only modified in current
                                     file
@@ -375,6 +381,9 @@ Usage: #{csc} FILENAME | OPTION ...
     -no-procedure-checks-for-usual-bindings
                                    disable procedure call checks only for usual
                                     bindings
+    -no-procedure-checks-for-toplevel-bindings
+                                   disable procedure call checks for toplevel
+                                    bindings
 
   Configuration options:
 
@@ -502,7 +511,7 @@ EOF
     (when osx
       (set! link-options (cons "-framework CoreFoundation" link-options))))
 
-  (let loop ([args args])
+  (let loop ((args args))
     (cond [(null? args)
 	   ;; Builtin search directory options do not override explict options
            (set! compile-options (append compile-options builtin-compile-options))
@@ -568,10 +577,12 @@ EOF
 	       [(-objc) 
 		(set! objc-mode #t) ]
 	       [(-static) 
-		(set! translate-options (cons* "-feature" "chicken-compile-static" translate-options))
+		(set! translate-options
+		  (cons* "-feature" "chicken-compile-static" translate-options))
 		(set! static #t) ]
 	       [(-static-libs) 
-		(set! translate-options (cons* "-feature" "chicken-compile-static" translate-options))
+		(set! translate-options
+		  (cons* "-feature" "chicken-compile-static" translate-options))
 		(set! static-libs #t) ]
 	       [(-cflags)
 		(set! inquiry-only #t) 
@@ -586,7 +597,7 @@ EOF
 	       [(-libs)
 		(set! inquiry-only #t)
 		(set! show-libs #t) ]
-	       [(-v)
+	       [(-v -verbose)
 		(when (and (number? verbose) (not msvc))
 		  (set! compile-options (cons* "-v" "-Q" compile-options))
 		  (set! link-options (cons (if msvc "-VERBOSE" "-v") link-options)) )
@@ -657,6 +668,7 @@ EOF
 		  (set! rest (cdr rest))
 		  (set! target-filename fn) ) ]
 	       [(|-O| |-O1|) (set! rest (cons* "-optimize-level" "1" rest))]
+	       [(|-O0|) (set! rest (cons* "-optimize-level" "0" rest))]
 	       [(|-O2|) (set! rest (cons* "-optimize-level" "2" rest))]
 	       [(|-O3|) (set! rest (cons* "-optimize-level" "3" rest))]
 	       [(|-O4|) (set! rest (cons* "-optimize-level" "4" rest))]
@@ -800,7 +812,14 @@ EOF
 		 (if to-stdout 
 		     '("-to-stdout")
 		     `("-output-file" ,(quotewrap fc)) )
-		 (map quote-option (append translate-options translation-optimization-options)) ) )
+		 (map quote-option
+		      (append 
+		       extra-features
+		       translate-options 
+		       (cond (cpp-mode '("-feature" "chicken-scheme-to-c++"))
+			     (objc-mode '("-feature" "chicken-scheme-to-objc"))
+			     (else '()))
+		       translation-optimization-options)) ) )
 	 " ") )
        (set! c-files (append (list fc) c-files))
        (set! generated-c-files (append (list fc) generated-c-files))))
@@ -923,13 +942,21 @@ EOF
 		  INSTALL_LIB_HOME
 		  TARGET_RUN_LIB_HOME))))
 
+(define (target-lib-path)
+  (let ((tdir TARGET_LIB_HOME))
+    (if (and (not (string=? tdir ""))
+	     cross-chicken
+	     (not host-mode))
+	tdir
+	(lib-path))))
+
 (define (copy-libraries targetdir)
   (let ((lib (make-pathname
-	      (lib-path) 
+	      (target-lib-path) 
 	      "libchicken"
 	      (cond (osx "dylib")
 		    (win "dll")
-		    (else (conc "so." BINARY_VERSION))))))
+		    (else "so")))))
     (copy-files lib targetdir)))
 
 (define (copy-files from to)
