@@ -26,10 +26,9 @@
 
 
 (declare
-  (uses chicken-syntax srfi-69 ports extras)
+  (uses chicken-syntax ports extras)
   (usual-integrations)
   (disable-interrupts)
-  (disable-warning var)
   (compile-syntax)
   (foreign-declare #<<EOF
 #if (defined(_MSC_VER) && defined(_WIN32)) || defined(HAVE_DIRECT_H)
@@ -48,14 +47,15 @@ EOF
   parse-option-string chop-separator lookup-script-file
   report describe dump hexdump bytevector-data get-config
   deldups tty-input?
-  history-list history-count history-add history-ref)
+  history-list history-count history-add history-ref history-clear history-show)
 
 (declare
   (always-bound
     ##sys#windows-platform)
   (hide parse-option-string bytevector-data member* canonicalize-args 
-	describer-table dirseparator? resolve-var
-	findall command-table) )
+	describer-table dirseparator? circular-list? improper-pairs?
+	show-frameinfo selected-frame select-frame copy-from-frame
+	findall command-table default-editor) )
 
 
 ;;; Parameters:
@@ -66,6 +66,16 @@ EOF
 (set! ##sys#features (cons #:csi ##sys#features))
 (set! ##sys#notices-enabled #t)
 
+(define editor-command (make-parameter #f))
+(define selected-frame #f)
+
+(define default-editor 
+  (or (get-environment-variable "EDITOR")
+      (get-environment-variable "VISUAL")
+      (if (get-environment-variable "EMACS")
+	  "emacsclient"
+	  "vi")))			; shudder
+
 
 ;;; Print all sorts of information:
 
@@ -86,6 +96,7 @@ usage: csi [FILENAME | OPTION ...]
     -p  -print EXPRESSION         evaluate and print result(s)
     -P  -pretty-print EXPRESSION  evaluate and print result(s) prettily
     -D  -feature SYMBOL           register feature identifier
+        -no-feature SYMBOL        disable built-in feature identifier
     -q  -quiet                    do not print banner
 
 EOF
@@ -222,6 +233,21 @@ EOF
 	(set! history-count (fx+ history-count 1))
 	x) ) ) )
 
+(define (history-clear)
+  (vector-fill! history-list (##sys#void)))
+
+(define history-show
+  (let ((newline newline))
+    (lambda ()
+      (do ((i 1 (fx+ i 1)))
+	  ((>= i history-count))
+	(printf "#~a: " i)
+	(##sys#with-print-length-limit
+	 80
+	 (lambda ()
+	   (##sys#print (vector-ref history-list i) #t ##sys#standard-output)))
+	(newline)))))
+
 (define (history-ref index)
   (let ([i (inexact->exact index)])
     (if (and (fx> i 0) (fx<= i history-count))
@@ -317,6 +343,27 @@ EOF
 			 (when ##sys#last-exception
 			   (history-add (list ##sys#last-exception))
 			   (describe ##sys#last-exception) ) )
+			((e)
+			 (let ((r (system
+				   (string-append 
+				    (or (editor-command) default-editor)
+				    " " (read-line)))))
+			   (if (not (zero? r))
+			       (printf "Editor returned with non-zero exit status ~a" r))))
+			((ch)
+			 (history-clear)
+			 (##sys#void))
+			((h)
+			 (history-show)
+			 (##sys#void))
+			((c)
+			 (show-frameinfo selected-frame)
+			 (##sys#void))
+			((f)
+			 (select-frame (read))
+			 (##sys#void))
+			((g)
+			 (copy-from-frame (read)))
 			((s)
 			 (let* ((str (read-line))
 				(r (system str)) )
@@ -335,8 +382,14 @@ EOF
  ,l FILENAME ...   Load one or more files
  ,ln FILENAME ...  Load one or more files and print result of each top-level expression
  ,r                Show system information
+ ,h                Show history of expression results
+ ,ch               Clear history of expression results
+ ,e FILENAME       Run external editor
  ,s TEXT ...       Execute shell-command
  ,exn              Describe last exception
+ ,c                Show call-chain of most recent error
+ ,f N              Select frame N
+ ,g NAME           Get variable NAME from current frame
  ,t EXP            Evaluate form and print elapsed time
  ,x EXP            Pretty print expanded expression EXP\n")
 			 (##sys#hash-table-for-each
@@ -355,9 +408,6 @@ EOF
 	       (history-add rs)
 	       (apply values rs) ) ) ) ) ) )
 
-(define (resolve-var str)
-  (##sys#strip-syntax (string->symbol str) (##sys#current-environment) #t))
-
 
 ;;; Parse options from string:
 
@@ -383,6 +433,7 @@ EOF
 	(sort sort)
 	(with-output-to-port with-output-to-port)
 	(current-output-port current-output-port) 
+	(argv argv)
 	(prefix
 	 (or (get-environment-variable "CHICKEN_PREFIX")
 	     (foreign-value "C_INSTALL_PREFIX" c-string) ) ))
@@ -414,7 +465,8 @@ EOF
                      Avg bucket length:\t~S~%  ~
                      Total symbol count:\t~S~%~
                    Memory:\theap size is ~S bytes~A with ~S bytes currently in use~%~  
-                     nursery size is ~S bytes, stack grows ~A~%"
+                     nursery size is ~S bytes, stack grows ~A~%~
+                   Command line:    \t~S~%"
 		    (machine-type)
 		    (if (##sys#fudge 3) "(64-bit)" "")
 		    (software-type)
@@ -430,7 +482,8 @@ EOF
 		    (if (##sys#fudge 17) " (fixed)" "")
 		    (vector-ref minfo 1)
 		    (vector-ref minfo 2)
-		    (if (= 1 (##sys#fudge 18)) "downward" "upward") )
+		    (if (= 1 (##sys#fudge 18)) "downward" "upward")
+		    (argv))
 	    (##sys#write-char-0 #\newline ##sys#standard-output)
 	    (when (##sys#fudge 14) (display "interrupts are enabled\n"))
 	    (when (##sys#fudge 15) (display "symbol gc is enabled\n")) 
@@ -449,6 +502,21 @@ EOF
     (f32vector "vector of 32-bit floats" f32vector-length f32vector-ref)
     (f64vector "vector of 64-bit floats" f64vector-length f64vector-ref) ) )
 
+(define (circular-list? x)
+  (let lp ((x x) (lag x))
+    (and (pair? x)
+	 (let ((x (cdr x)))
+	   (and (pair? x)
+		(let ((x   (cdr x))
+		      (lag (cdr lag)))
+		  (or (eq? x lag) (lp x lag))))))))
+
+(define (improper-pairs? x)
+  (let lp ((x x))
+    (if (not (pair? x)) #f
+       (or (eq? x (car x))
+           (lp (cdr x))))))
+
 (define-constant max-describe-lines 40)
 
 (define describer-table (make-vector 37 '()))
@@ -502,11 +570,12 @@ EOF
 	    [(number? x) (fprintf out "number ~S~%" x)]
 	    [(string? x) (descseq "string" ##sys#size string-ref 0)]
 	    [(vector? x) (descseq "vector" ##sys#size ##sys#slot 0)]
+	    ((keyword? x)
+	     (fprintf out "keyword symbol with name ~s~%" 
+	       (##sys#symbol->string x)))
 	    [(symbol? x)
 	     (unless (##sys#symbol-has-toplevel-binding? x)
 	       (display "unbound " out))
-	     (when (and (symbol? x) (fx= 0 (##sys#byte (##sys#slot x 1) 0)))
-	       (display "keyword " out) )
 	     (let ((q (##sys#qualified-symbol? x)))
 	       (fprintf out "~a~asymbol with name ~S~%"
 		 (if (##sys#interned-symbol? x) "" "uninterned ")
@@ -525,8 +594,19 @@ EOF
 		    (lambda ()
 		      (write (cadr plist) out) ) )
 		   (newline out) ) ) ) ]
+	    [(or (circular-list? x) (improper-pairs? x))
+	     (fprintf out "circular structure: ")
+	     (let loop-print ((x x)
+                              (cdr-refs (list x)))
+               (cond ((or (atom? x)
+                          (null? x)) (printf "eol~%"))
+                     ((memq (car x) cdr-refs)
+                      (fprintf out "(circle)~%" ))
+		     ((not (memq (car x) cdr-refs))
+		      (fprintf out "~S -> " (car x))
+		      (loop-print (cdr x) (cons (car x)  cdr-refs) ))))]
 	    [(list? x) (descseq "list" length list-ref 0)]
-	    [(pair? x) (fprintf out "pair with car ~S and cdr ~S~%" (car x) (cdr x))]
+	    [(pair? x) (fprintf out "pair with car ~S~%and cdr ~S~%" (car x) (cdr x))]
 	    [(procedure? x)
 	     (let ([len (##sys#size x)])
 	       (if (and (> len 3)
@@ -572,9 +652,16 @@ EOF
 	       (fprintf out "hash-table with ~S element~a~%  comparison procedure: ~A~%"
 			n (if (fx= n 1) "" "s")  (##sys#slot x 3)) )
 	     (fprintf out "  hash function: ~a~%" (##sys#slot x 4))
-	     (hash-table-walk		; blindly assumes it is bound
-	      x
-	      (lambda (k v) (fprintf out " ~S\t-> ~S~%" k v)) ) ]
+	     ;; this copies code out of srfi-69.scm, but we don't want to depend on it
+	     (let* ((vec (##sys#slot x 1))
+		    (len (##sys#size vec)) )
+	       (do ((i 0 (fx+ i 1)) )
+		   ((fx>= i len))
+		 (for-each
+		  (lambda (bucket)
+		    (fprintf out " ~S\t-> ~S~%"
+		      (##sys#slot bucket 0) (##sys#slot bucket 1)) )
+		  (##sys#slot vec i)) ) ) ]
 	    [(##sys#structure? x 'condition)
 	     (fprintf out "condition: ~s~%" (##sys#slot x 1))
 	     (for-each
@@ -644,12 +731,12 @@ EOF
 	(do ([j 0 (fx+ j 1)]
 	     [a a (fx+ a 1)] )
 	    ((or (fx>= j 16) (fx>= a len))
-	     (and-let* ([(fx>= a len)]
-			[o (fxmod len 16)]
-			[(not (fx= o 0))] )
-	       (do ([k (fx- 16 o) (fx- k 1)])
-		   ((fx= k 0))
-		 (display "   " out) ) ) )
+	     (when (fx>= a len)
+	       (let ((o (fxmod len 16)))
+		 (unless (fx= o 0)
+		   (do ((k (fx- 16 o) (fx- k 1)))
+		       ((fx= k 0))
+		     (display "   " out) ) ) ) ) ) 
 	  (write-char #\space out)
 	  (display (justify (ref bv a) 2 16 #\0) out) )
 	(write-char #\space out)
@@ -660,7 +747,111 @@ EOF
 	    (if (and (fx>= c 32) (fx< c 128))
 		(write-char (integer->char c) out)
 		(write-char #\. out) ) ) ) 
-	(##sys#write-char-0 #\newline out) ) ) ) )
+	(write-char #\newline out) ) ) ) )
+
+
+;;; Frame-info operations:
+
+(define show-frameinfo
+  (let ((write-char write-char)
+	(newline newline)
+	(display display))
+    (lambda (fn)
+      (define (prin1 x)
+	(##sys#with-print-length-limit
+	 100
+	 (lambda ()
+	   (##sys#print x #t ##sys#standard-output))))
+      (let* ((ct (or ##sys#repl-recent-call-chain '()))
+	     (len (length ct)))
+	(set! selected-frame 
+	  (or (and (memq fn ct) fn)
+	      (and (fx> len 0)
+		   (list-ref ct (fx- len 1)))))
+	(do ((ct ct (cdr ct))
+	     (i (fx- len 1) (fx- i 1)))
+	    ((null? ct))
+	  (let* ((info (car ct))
+		 (here (eq? selected-frame info))
+		 (form (##sys#slot info 1)) ; cooked1 (expr/form)
+		 (data (##sys#slot info 2)) ; cooked2 (cntr/frameinfo)
+		 (finfo (##sys#structure? data 'frameinfo))
+		 (cntr (if finfo (##sys#slot data 1) data))) ; cntr
+	    (printf "~a~a:~a\t~a\t  " 
+	      (if here #\* #\space)
+	      i
+	      (if (and finfo (pair? (##sys#slot data 2))) "[]" "  ") ; e
+	      (##sys#slot info 0))	; raw
+	    (when cntr (printf "[~a] " cntr))
+	    (when form (prin1 form))
+	    (newline)
+	    (when (and here finfo)
+	      (for-each
+	       (lambda (e v)
+		 (unless (null? e)
+		   (display "  ---\n")
+		   (do ((i 0 (fx+ i 1))
+			(be e (cdr be)))
+		       ((null? be))
+		     (printf "  ~s:\t  " (car be))
+		     (prin1 (##sys#slot v i))
+		     (newline))))
+	       (##sys#slot data 2)	   ; e
+	       (##sys#slot data 3)))))))))	   ; v
+	  
+(define select-frame
+  (let ((display display))
+    (lambda (n)
+      (cond ((or (not (number? n))
+		 (not ##sys#repl-recent-call-chain)
+		 (fx< n 0)
+		 (fx>= n (length ##sys#repl-recent-call-chain)))
+	     (display "no such frame\n"))
+	    (else
+	     (set! selected-frame
+	       (list-ref 
+		##sys#repl-recent-call-chain
+		(fx- (length ##sys#repl-recent-call-chain) (fx+ n 1))))
+	     (show-frameinfo selected-frame))))))
+
+(define copy-from-frame
+  (let ((display display)
+	(call/cc call/cc))
+    (lambda (name)
+      (let* ((ct (or ##sys#repl-recent-call-chain '()))
+	     (len (length ct))
+	     (name 
+	      (cond ((symbol? name) (##sys#slot name 1)) ; name
+		    ((string? name) name)
+		    (else 
+		     (display "string or symbol required for `,cf'\n")
+		     #f))))
+	(if name
+	    (call/cc
+	     (lambda (return)
+	       (define (fail msg)
+		 (display msg)
+		 (return (##sys#void)))
+	       (do ((ct ct (cdr ct))
+		    (i (fx- len 1) (fx- i 1)))
+		   ((null? ct) (fail "no environment in frame\n")) 
+		 (let* ((info (car ct))
+			(here (eq? selected-frame info))
+			(data (##sys#slot info 2)) ; cooked2 (cntr/frameinfo)
+			(finfo (##sys#structure? data 'frameinfo))
+			(cntr (if finfo (##sys#slot data 1) data))) ; cntr
+		   (when (and here finfo)
+		     (for-each
+		      (lambda (e v)
+			(do ((i 0 (fx+ i 1))
+			     (be e (cdr be)))
+			    ((null? be) (fail "no such variable\n"))
+			  (when (string=? name (##sys#slot (car be) 1)) ; name
+			    (history-add (list (##sys#slot v i)))
+			    (return (##sys#slot v i)))))
+		      (##sys#slot data 2)	; e
+		      (##sys#slot data 3))))))) ; v
+	    (##sys#void))))))
 
 
 ;;; Start interpreting:
@@ -695,7 +886,7 @@ EOF
   '(#\k #\s #\v #\h #\D #\e #\i #\R #\b #\n #\q #\w #\- #\I #\p #\P) )
 
 (define-constant long-options
-  '("-ss" "-sx" "-script" "-version" "-help" "--help" "-feature" "-eval"
+  '("-ss" "-sx" "-script" "-version" "-help" "--help" "-feature" "-no-feature" "-eval"
     "-case-insensitive" "-keyword-style" "-no-parentheses-synonyms" "-no-symbol-escape"
     "-r5rs-syntax" "-setup-mode"
     "-require-extension" "-batch" "-quiet" "-no-warnings" "-no-init" 
@@ -725,13 +916,14 @@ EOF
 	     (loop (cdr chars))))))
 
 (define-constant simple-options
-  '("--" "-b" "-batch" "-q" "-quiet" "-n" "-no-init" "-w" "-no-warnings" "-i" "-case-insensitive"
+  '("--" "-b" "-batch" "-q" "-quiet" "-n" "-no-init" "-w" "-no-warnings" 
+    "-i" "-case-insensitive"
     "-no-parentheses-synonyms" "-no-symbol-escape" "-r5rs-syntax" "-setup-mode"
     ; Not "simple" but processed early
     "-ss" "-sx" "-s" "-script") )
 
 (define-constant complex-options
-  '("-D" "-feature" "-I" "-include-path" "-K" "-keyword-style") )
+  '("-D" "-feature" "-I" "-include-path" "-K" "-keyword-style" "-no-feature") )
 
 (define (run)
   (let* ([extraopts (parse-option-string (or (get-environment-variable "CSI_OPTIONS") ""))]
@@ -781,8 +973,6 @@ EOF
 	  (do ([x (read in) (read in)])
 	      ((eof-object? x))
 	    (rec (receive (eval x))) ) ) )
-      (when quietflag
-	(set! ##sys#eval-debug-level 0))
       (when (member* '("-h" "-help" "--help") args)
 	(print-usage)
 	(exit 0) )
@@ -806,6 +996,7 @@ EOF
 	(case-sensitive #f) )
       (for-each register-feature! (collect-options "-feature"))
       (for-each register-feature! (collect-options "-D"))
+      (for-each unregister-feature! (collect-options "-no-feature"))
       (set! ##sys#include-pathnames 
 	(deldups
 	 (append (map chop-separator (collect-options "-include-path"))
@@ -843,7 +1034,7 @@ EOF
 	     (repl)
 	     (##sys#write-char-0 #\newline ##sys#standard-output) ) )
 	(let* ((arg (car args)))
-	  (cond ((member arg simple-options) )
+	  (cond ((member arg simple-options))
 		((member arg complex-options)
 		 (set! args (cdr args)) )
 		((or (string=? "-R" arg) (string=? "-require-extension" arg))
