@@ -48,7 +48,7 @@
    identity
    intersperse
    join
-   left-section
+   left-section				; DEPRECATED
    list->queue
    list-of?
    make-queue
@@ -56,7 +56,7 @@
    merge!
    never?
    none?
-   noop
+   noop					;DEPRECATED
    o
    project
    queue->list
@@ -70,7 +70,7 @@
    queue?
    rassoc
    reverse-string-append
-   right-section
+   right-section			; DEPRECATED
    shuffle
    sort
    sort!
