@@ -27,47 +27,12 @@
 
 (declare
  (unit data-structures)
- (usual-integrations)
- (disable-warning redef)
  (foreign-declare #<<EOF
 #define C_mem_compare(to, from, n)   C_fix(C_memcmp(C_c_string(to), C_c_string(from), C_unfix(n)))
 EOF
 ) )
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (no-bound-checks)
-    (no-procedure-checks-for-usual-bindings)
-    (bound-to-procedure
-      ##sys#check-char ##sys#check-exact ##sys#check-port ##sys#check-string
-      ##sys#substring ##sys#for-each ##sys#map ##sys#setslot
-      ##sys#allocate-vector ##sys#check-pair ##sys#error-not-a-proper-list
-      ##sys#member ##sys#assoc ##sys#error ##sys#signal-hook ##sys#read-string!
-      ##sys#check-symbol ##sys#check-vector 
-      ##sys#check-number
-      ##sys#flonum-fraction ##sys#make-port ##sys#fetch-and-check-port-arg
-      ##sys#print ##sys#check-structure ##sys#make-structure make-parameter
-      ##sys#flush-output ##sys#write-char-0 ##sys#number->string
-      ##sys#fragments->string ##sys#symbol->qualified-string
-      ##sys#number? ##sys#procedure->string
-      ##sys#pointer->string ##sys#user-print-hook ##sys#peek-char-0
-      ##sys#read-char-0 ##sys#write-char ##sys#string-append ##sys#gcd ##sys#lcm
-      ##sys#fudge ##sys#check-list ##sys#user-read-hook ##sys#check-closure ##sys#check-inexact
-      input-port? make-vector list->vector sort! merge! open-output-string floor
-      get-output-string current-output-port display write port? list->string
-      make-string string pretty-print-width newline char-name read random
-      open-input-string make-string call-with-input-file read-line reverse ) ) ] )
-
-(private data-structures
-  fprintf0 generic-write )
-
-(declare
-  (hide
-    fprintf0 generic-write ) )
-
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'data-structures)
 
@@ -77,7 +42,7 @@ EOF
 
 (define (identity x) x)
 
-(define (project n)
+(define (project n)			; DEPRECATED
   (lambda args (list-ref args n)) )
 
 (define (conjoin . preds)
@@ -136,7 +101,7 @@ EOF
 	    [(pred (##sys#slot lst 0)) (loop (##sys#slot lst 1))]
 	    [else #f] ) ) ) )
 
-(define (noop . _) (void))
+(define (noop . _) (void))		;DEPRECATED
 
 (define (each . procs)
   (cond ((null? procs) (lambda _ (void)))
@@ -160,12 +125,12 @@ EOF
 
 (define (never? . _) #f)
 
-(define (left-section proc . args)
+(define (left-section proc . args)	;DEPRECATED
   (##sys#check-closure proc 'left-section)
   (lambda xs
     (##sys#apply proc (##sys#append args xs)) ) )
 
-(define right-section
+(define right-section			;DEPRECATED
   (let ([##sys#reverse reverse])
     (lambda (proc . args)
       (##sys#check-closure proc 'right-section)
@@ -259,7 +224,6 @@ EOF
 	      [else (loop (##sys#slot blst 1) (##sys#slot lst 1))] ) ) ) ) )
 
 (define shuffle
-  ;; this should really shadow SORT! and RANDOM...
   (lambda (l random)
     (let ((len (length l)))
       (map cdr
@@ -340,10 +304,8 @@ EOF
 ;;; Anything->string conversion:
 
 (define ->string 
-  (let ([open-output-string open-output-string]
-	[display display]
-	[string string]
-	[get-output-string get-output-string] )
+  (let ([display display]
+	[string string])
     (lambda (x)
       (cond [(string? x) x]
 	    [(symbol? x) (symbol->string x)]
@@ -835,7 +797,7 @@ EOF
 	(and (fx> len 0)
 	     (let loop ([ps 0]
 			[pe len] )
-	       (let ([p (fx+ ps (##core#inline "C_fixnum_divide" (fx- pe ps) 2))])
+	       (let ([p (fx+ ps (##core#inline "C_fixnum_shift_right" (fx- pe ps) 1))])
 		 (let* ([x (##sys#slot vec p)]
 			[r (proc x)] )
 		   (cond [(fx= r 0) p]
