@@ -28,21 +28,12 @@
 (declare
   (unit eval)
   (uses expand)
-  (disable-warning var)
-  (hide ##sys#split-at-separator
-	##sys#r4rs-environment ##sys#r5rs-environment 
+  (hide ##sys#r4rs-environment ##sys#r5rs-environment 
 	##sys#interaction-environment pds pdss pxss d) 
   (not inline ##sys#repl-eval-hook ##sys#repl-read-hook ##sys#repl-print-hook 
        ##sys#read-prompt-hook ##sys#alias-global-hook ##sys#user-read-hook
        ##sys#syntax-error-hook))
 
-(define (d arg1 . more)
-  (if (null? more)
-      (pp arg1)
-      (apply print arg1 more)))
-
-(define-syntax d (syntax-rules () ((_ . _) (void))))
-
 #>
 #ifndef C_INSTALL_EGG_HOME
 # define C_INSTALL_EGG_HOME    "."
@@ -57,48 +48,7 @@
 #endif
 <#
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    ;***(no-bound-checks)
-    (no-procedure-checks-for-usual-bindings)
-    (bound-to-procedure 
-     ##sys#check-char ##sys#check-exact ##sys#check-port ##sys#check-string ##sys#load-library
-     ##sys#load-library-0
-     ##sys#for-each ##sys#map ##sys#setslot ##sys#allocate-vector ##sys#check-pair ##sys#error-not-a-proper-list
-     ##sys#check-symbol ##sys#check-vector 
-     ##sys#check-number ##sys#copy-env-table
-     ##sys#flonum-fraction ##sys#make-port ##sys#fetch-and-check-port-arg ##sys#print ##sys#check-structure 
-     ##sys#make-structure ##sys#feature?
-     ##sys#error-handler ##sys#hash-symbol ##sys#check-syntax
-     ##sys#hash-table-ref ##sys#hash-table-set! ##sys#canonicalize-body ##sys#decompose-lambda-list
-     ##sys#make-c-string ##sys#resolve-include-filename
-     ##sys#load ##sys#error ##sys#warn ##sys#hash-table-location ##sys#expand-home-path
-     ##sys#make-flonum ##sys#make-pointer ##sys#null-pointer ##sys#address->pointer 
-     ##sys#pointer->address ##sys#compile-to-closure ##sys#make-string ##sys#make-lambda-info
-     ##sys#number? ##sys#symbol->qualified-string ##sys#decorate-lambda ##sys#string-append
-     ##sys#ensure-heap-reserve ##sys#syntax-error-hook ##sys#read-prompt-hook
-     ##sys#repl-eval-hook ##sys#append ##sys#eval-decorator
-     open-output-string get-output-string make-parameter software-type software-version machine-type
-     build-platform ##sys#string->symbol list->vector get-environment-variable
-     extension-information syntax-error ->string chicken-home ##sys#expand-curried-define
-     vector->list store-string open-input-string eval ##sys#gc
-     with-exception-handler print-error-message read-char read ##sys#read-error
-     ##sys#reset-handler call-with-current-continuation ##sys#peek-char-0 ##sys#read-char-0
-     ##sys#clear-trace-buffer ##sys#write-char-0 print-call-chain ##sys#with-print-length-limit
-     repl-prompt ##sys#flush-output ##sys#extended-lambda-list? keyword? get-line-number
-     symbol->string string-append display ##sys#repository-path ##sys#file-info make-vector
-     ##sys#make-vector string-copy vector->list ##sys#do-the-right-thing ##sys#->feature-id
-     ##sys#extension-information ##sys#symbol->string ##sys#canonicalize-extension-path
-     file-exists? ##sys#load-extension ##sys#find-extension ##sys#substring reverse
-     dynamic-load-libraries ##sys#string->c-identifier load-verbose ##sys#load ##sys#get-keyword
-     port? ##sys#file-info ##sys#signal-hook ##sys#dload open-input-file close-input-port
-     read write newline ##sys#eval-handler ##sys#set-dlopen-flags! cadadr ##sys#lookup-runtime-requirements
-     map string->keyword ##sys#abort
-     ##sys#expand-0) ) ] )
-
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (define-foreign-variable install-egg-home c-string "C_INSTALL_EGG_HOME")
 (define-foreign-variable installation-home c-string "C_INSTALL_SHARE_HOME")
@@ -106,7 +56,8 @@
 
 (define ##sys#core-library-modules
   '(extras lolevel utils files tcp regex posix srfi-1 srfi-4 srfi-13 
-	   srfi-14 srfi-18 srfi-69 data-structures ports chicken-syntax))
+	   srfi-14 srfi-18 srfi-69 data-structures ports chicken-syntax
+	   chicken-ffi-syntax))
 
 (define ##sys#explicit-library-modules '())
 
@@ -226,14 +177,12 @@
      p) ) )
 
 (define ##sys#unbound-in-eval #f)
-(define ##sys#eval-debug-level 1)
 (define ##sys#unsafe-eval #f)
+(define ##sys#eval-debug-level (make-parameter 1))
 
 (define ##sys#compile-to-closure
   (let ([write write]
 	[reverse reverse]
-	[open-output-string open-output-string]
-	[get-output-string get-output-string] 
 	[with-input-from-file with-input-from-file]
 	[unbound (##sys#slot '##sys#arbitrary-unbound-symbol 0)]
 	[display display] )
@@ -263,13 +212,21 @@
 		((eq? x (##sys#slot lst 0)) i)
 		(else (loop (##sys#slot lst 1) (fx+ i 1))) ) ) )
 
-      (define (emit-trace-info tf info cntr) 
+      (define (emit-trace-info tf info cntr e v) 
 	(when tf
-	  (##core#inline "C_emit_eval_trace_info" info cntr ##sys#current-thread) ) )
-
+	  (##core#inline 
+	   "C_emit_eval_trace_info" 
+	   info
+	   (##sys#make-structure 'frameinfo cntr e v)
+	   ##sys#current-thread) ) )
+      
       (define (emit-syntax-trace-info tf info cntr) 
 	(when tf
-	  (##core#inline "C_emit_syntax_trace_info" info cntr ##sys#current-thread) ) )
+	  (##core#inline
+	   "C_emit_syntax_trace_info"
+	   info
+	   cntr
+	   ##sys#current-thread) ) )
 	
       (define (decorate p ll h cntr)
 	(##sys#eval-decorator p ll h cntr) )
@@ -300,7 +257,8 @@
 				       (or (##sys#get j '##core#primitive) j))))
 			  (if ##sys#eval-environment
 			      (let ([loc (##sys#hash-table-location ##sys#eval-environment var #t)])
-				(unless loc (##sys#syntax-error-hook "reference to undefined identifier" var))
+				(unless loc
+				  (##sys#syntax-error-hook "reference to undefined identifier" var))
 				(if ##sys#unsafe-eval 
 				    (lambda v (##sys#slot loc 1))
 				    (lambda v 
@@ -333,7 +291,8 @@
 		   (eof-object? x)
 		   (string? x) )
 	       (lambda v x) ]
-	      [(not (pair? x)) (##sys#syntax-error-hook "illegal non-atomic object" x)]
+	      [(not (pair? x)) 
+	       (##sys#syntax-error/context "illegal non-atomic object" x)]
 	      [(symbol? (##sys#slot x 0))
 	       (emit-syntax-trace-info tf x cntr)
 	       (let ((x2 (##sys#expand x se #f)))
@@ -362,7 +321,7 @@
 			    (lambda v c)))
 
 			 [(##core#global-ref)
-			  (let ([var (cadr x)])
+			  (let ([var (cadr x)]) ;XXX broken - should alias (see above)
 			    (if ##sys#eval-environment
 				(let ([loc (##sys#hash-table-location ##sys#eval-environment var #t)])
 				  (lambda v (##sys#slot loc 1)) )
@@ -384,7 +343,7 @@
 					  (compile '(##core#undefined) e #f tf cntr se) ) ] )
 			    (lambda (v) (if (##core#app test v) (##core#app cns v) (##core#app alt v))) ) ]
 
-			 [(##core#begin)
+			 [(##core#begin ##core#toplevel-begin)
 			  (let* ((body (##sys#slot x 1))
 				 (len (length body)) )
 			    (case len
@@ -404,13 +363,18 @@
 			    (receive (i j) (lookup var e se)
 			      (let ((val (compile (caddr x) e var tf cntr se)))
 				(cond [(not i)
+				       (when ##sys#notices-enabled
+					 (and-let* ((a (assq var (##sys#current-environment)))
+						    ((symbol? (cdr a))))
+					   (##sys#notice "assignment to imported value binding" var)))
 				       (let ((var (##sys#alias-global-hook j #t)))
 					 (if ##sys#eval-environment
 					     (let ([loc (##sys#hash-table-location
 							 ##sys#eval-environment 
 							 var
 							 ##sys#environment-is-mutable) ] )
-					       (unless loc (##sys#error "assignment of undefined identifier" var))
+					       (unless loc
+						 (##sys#error "assignment to undefined identifier" var))
 					       (if (##sys#slot loc 2)
 						   (lambda (v) (##sys#setslot loc 1 (##core#app val v)))
 						   (lambda v (##sys#error "assignment to immutable variable" var)) ) )
@@ -744,7 +708,9 @@
 			 [(##core#app)
 			  (compile-call (cdr x) e tf cntr se) ]
 
-			 [else (compile-call x e tf cntr se)] ) ) ) ) ]
+			 (else
+			  (fluid-let ((##sys#syntax-context (cons head ##sys#syntax-context)))
+			    (compile-call x e tf cntr se)))))))]
 	      
 	      [else
 	       (emit-syntax-trace-info tf x cntr)
@@ -776,38 +742,38 @@
 	       [argc (checked-length args)]
 	       [info x] )
 	  (case argc
-	    [(#f) (##sys#syntax-error-hook "malformed expression" x)]
+	    [(#f) (##sys#syntax-error/context "malformed expression" x)]
 	    [(0) (lambda (v)
-		   (emit-trace-info tf info cntr)
-		   ((fn v)))]
+		   (emit-trace-info tf info cntr e v)
+		   ((##core#app fn v)))]
 	    [(1) (let ([a1 (compile (##sys#slot args 0) e #f tf cntr se)])
 		   (lambda (v)
-		     (emit-trace-info tf info cntr)
+		     (emit-trace-info tf info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v))) ) ]
 	    [(2) (let* ([a1 (compile (##sys#slot args 0) e #f tf cntr se)]
 			[a2 (compile (##core#inline "C_u_i_list_ref" args 1) e #f tf cntr se)] )
 		   (lambda (v)
-		     (emit-trace-info tf info cntr)
+		     (emit-trace-info tf info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v) (##core#app a2 v))) ) ]
 	    [(3) (let* ([a1 (compile (##sys#slot args 0) e #f tf cntr se)]
 			[a2 (compile (##core#inline "C_u_i_list_ref" args 1) e #f tf cntr se)]
 			[a3 (compile (##core#inline "C_u_i_list_ref" args 2) e #f tf cntr se)] )
 		   (lambda (v)
-		     (emit-trace-info tf info cntr)
+		     (emit-trace-info tf info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v) (##core#app a2 v) (##core#app a3 v))) ) ]
 	    [(4) (let* ([a1 (compile (##sys#slot args 0) e #f tf cntr se)]
 			[a2 (compile (##core#inline "C_u_i_list_ref" args 1) e #f tf cntr se)]
 			[a3 (compile (##core#inline "C_u_i_list_ref" args 2) e #f tf cntr se)] 
 			[a4 (compile (##core#inline "C_u_i_list_ref" args 3) e #f tf cntr se)] )
 		   (lambda (v)
-		     (emit-trace-info tf info cntr)
+		     (emit-trace-info tf info cntr e v)
 		     ((##core#app fn v) (##core#app a1 v) (##core#app a2 v) (##core#app a3 v) (##core#app a4 v))) ) ]
 	    [else (let ([as (##sys#map (lambda (a) (compile a e #f tf cntr se)) args)])
 		    (lambda (v)
-		      (emit-trace-info tf info cntr)
+		      (emit-trace-info tf info cntr e v)
 		      (apply (##core#app fn v) (##sys#map (lambda (a) (##core#app a v)) as))) ) ] ) ) )
 
-      (compile exp env #f (fx> ##sys#eval-debug-level 0) cntr se) ) ) )
+      (compile exp env #f (fx> (##sys#eval-debug-level) 0) cntr se) ) ) )
 
 (define ##sys#eval-handler 
   (make-parameter
@@ -882,12 +848,10 @@
       [write write]
       [display display]
       [newline newline]
-      (flush-output flush-output)
       [eval eval]
       [open-input-file open-input-file]
       [close-input-port close-input-port]
       [string-append string-append] 
-      [load-verbose load-verbose]
       [topentry (##sys#make-c-string "C_toplevel")] )
   (define (has-sep? str)
     (let loop ([i (fx- (##sys#size str) 1)])
@@ -929,9 +893,13 @@
 	       (display " ...\n") 
 	       (flush-output)] )
 	(or (and fname
-		 (or (##sys#dload (##sys#make-c-string fname) topentry #t) 
+		 (or (##sys#dload (##sys#make-c-string fname 'load) topentry #t) 
 		     (and (not (has-sep? fname))
-			  (##sys#dload (##sys#make-c-string (##sys#string-append "./" fname)) topentry #t) ) ) )
+			  (##sys#dload 
+			   (##sys#make-c-string
+			    (##sys#string-append "./" fname) 
+			    'load) 
+			   topentry #t) ) ) )
 	    (call-with-current-continuation
 	     (lambda (abrt)
 	       (fluid-let ((##sys#read-error-with-line-number #t)
@@ -947,7 +915,12 @@
 		    (lambda ()
 		      (let ((c1 (peek-char in)))
 			(when (char=? c1 (integer->char 127))
-			  (##sys#error 'load "unable to load compiled module" fname _dlerror) ) )
+			  (##sys#error 
+			   'load 
+			   (##sys#string-append 
+			    "unable to load compiled module - " 
+			    (or _dlerror "unknown reason"))
+			   fname)))
 		      (let ((x1 (read in)))
 			(do ((x x1 (read in)))
 			    ((eof-object? x))
@@ -1012,9 +985,7 @@
        x) ) ) )
 
 (define ##sys#load-library-0
-  (let ([load-verbose load-verbose]
-	[string-append string-append]
-	[dynamic-load-libraries dynamic-load-libraries]
+  (let ([string-append string-append]
 	[display display] )
     (lambda (uname lib)
       (let ([id (##sys#->feature-id uname)])
@@ -1029,14 +1000,14 @@
 		    (string-append 
 		     "C_"
 		     (##sys#string->c-identifier (##sys#slot uname 1)) 
-		     "_toplevel") ) ] )
+		     "_toplevel") 'load-library) ] )
 	      (when (load-verbose)
 		(display "; loading library ")
 		(display uname)
 		(display " ...\n") )
 	      (let loop ([libs libs])
 		(cond [(null? libs) #f]
-		      [(##sys#dload (##sys#make-c-string (##sys#slot libs 0)) top #f)
+		      [(##sys#dload (##sys#make-c-string (##sys#slot libs 0) 'load-library) top #f)
 		       (unless (memq id ##sys#features) (set! ##sys#features (cons id ##sys#features)))
 		       #t]
 		      [else (loop (##sys#slot libs 1))] ) ) ) ) ) ) ) )
@@ -1049,22 +1020,8 @@
 
 (define load-library ##sys#load-library)
 
-(define ##sys#split-at-separator
-  (let ([reverse reverse] )
-    (lambda (str sep)
-      (let ([len (##sys#size str)])
-	(let loop ([items '()] [i 0] [j 0])
-	  (cond [(fx>= i len)
-		 (reverse (cons (##sys#substring str j len) items)) ]
-		[(char=? (##core#inline "C_subchar" str i) sep)
-		 (let ([i2 (fx+ i 1)])
-		   (loop (cons (##sys#substring str j i) items) i2 i2) ) ]
-		[else (loop items (fx+ i 1) j)] ) ) ) ) ) )
-
 (define ##sys#include-forms-from-file
-  (let ((load-verbose load-verbose)
-	(print print)
-	(with-input-from-file with-input-from-file)
+  (let ((with-input-from-file with-input-from-file)
 	(read read)
 	(reverse reverse))
     (lambda (fname)
@@ -1130,8 +1087,7 @@
 (define ##sys#setup-mode #f)
 
 (define ##sys#find-extension
-  (let ((file-exists? file-exists?)
-	(string-append string-append) )
+  (let ((string-append string-append) )
     (lambda (p inc?)
       (let ((rp (##sys#repository-path)))
 	(define (check path)
@@ -1200,7 +1156,6 @@
 
 (define ##sys#extension-information
   (let ([with-input-from-file with-input-from-file]
-	[file-exists? file-exists?]
 	[string-append string-append]
 	[read read] )
     (lambda (id loc)
@@ -1244,6 +1199,11 @@
 	  ,@(if (and imp? (or (not builtin?) (##sys#current-module)))
 		`((import ,id))		;XXX make hygienic
 		'())))
+      (define (srfi-id n)
+	(if (fixnum? n)
+	    (##sys#intern-symbol
+	     (##sys#string-append "srfi-" (##sys#number->string n)))
+	    (##sys#syntax-error 'require-extension "invalid SRFI number" n)))
       (define (doit id impid)
 	(cond ((or (memq id builtin-features)
 		   (if comp?
@@ -1286,10 +1246,12 @@
 			      ,@(if s `((##core#require-for-syntax ',id)) '())
 			      ,@(if (or nr (and (not rr) s))
 				    '()
-				    `((##sys#require
-				       ,@(map (lambda (id) `',id)
-					      (cond (rr (cdr rr))
-						    (else (list id)) ) ) ) ) ) )
+				    (begin
+				      (add-req id #f)
+				      `((##sys#require
+					 ,@(map (lambda (id) `',id)
+						(cond (rr (cdr rr))
+						      (else (list id)) ) ) ) ) ) ) )
 			    impid #f)
 			   #t) ) )
 		       (else
@@ -1306,21 +1268,19 @@
 		       (exp
 			`(##core#begin
 			  ,@(map (lambda (n)
-				   (unless (fixnum? n)
-				     (##sys#syntax-error 'require-extension "invalid SRFI number" n))
-				   (let ((rid (string->symbol (string-append "srfi-" (number->string n)))))
+				   (let ((rid (srfi-id n)))
 				     (let-values (((exp f2) (doit rid rid)))
 				       (set! f (or f f2))
 				       exp)))
 				 (cdr id)))))
 		  (values exp f)))
 	       ((rename except only prefix)
-		(doit
-		 (let follow ((id2 id))
-		   (if (and (pair? id2) (pair? (cdr id2)))
-		       (follow (cadr id2))
-		       id2))
-		 id))
+		(let follow ((id2 id))
+		  (if (and (pair? id2) (pair? (cdr id2)))
+		      (if (and (eq? 'srfi (car id2)) (null? (cddr id2))) ; only allow one number
+			  (doit (srfi-id (cadr id2)) id)
+			  (follow (cadr id2)))
+		      (doit id2 id))))
 	       (else (##sys#error "invalid extension specifier" id) ) ) )
 	    ((symbol? id)
 	     (doit id id))
@@ -1495,34 +1455,6 @@
 		  (else (loop (##sys#slot paths 1))) ) ) ) ) ) )
 
 
-;;; Print timing information (support for "time" macro):
-
-(define ##sys#display-times
-  (let* ((display display)
-	 (spaces 
-	  (lambda (n)
-	    (do ((i n (fx- i 1)))
-		((fx<= i 0))
-	      (display #\space) ) ) )
-	 (display-rj 
-	  (lambda (x w)
-	    (let* ((xs (if (zero? x) "0" (number->string x)))
-		   (xslen (##core#inline "C_block_size" xs)) )
-	      (spaces (fx- w xslen))
-	      (display xs) ) ) ) )
-    (lambda (info)
-      (display-rj (##sys#slot info 0) 8)
-      (display " seconds elapsed\n") 
-      (display-rj (##sys#slot info 1) 8)
-      (display " seconds in (major) GC\n")
-      (display-rj (##sys#slot info 2) 8)
-      (display " mutations\n")
-      (display-rj (##sys#slot info 3) 8)
-      (display " minor GCs\n")
-      (display-rj (##sys#slot info 4) 8)
-      (display " major GCs\n") ) ) )
-
-
 ;;; SRFI-0 support code:
 
 (set! ##sys#features
@@ -1535,6 +1467,7 @@
 (define ##sys#repl-eval-hook #f)
 (define ##sys#repl-print-length-limit #f)
 (define ##sys#repl-read-hook #f)
+(define ##sys#repl-recent-call-chain #f)
 
 (define (##sys#repl-print-hook x port)
   (##sys#with-print-length-limit ##sys#repl-print-length-limit (cut ##sys#print x #t port))
@@ -1554,10 +1487,7 @@
   (let ((eval eval)
 	(read read)
 	(call-with-current-continuation call-with-current-continuation)
-	(print-call-chain print-call-chain)
-	(flush-output flush-output)
-	(load-verbose load-verbose)
-	(reset reset) )
+	(string-append string-append))
     (lambda ()
 
       (define (write-err xs)
@@ -1567,7 +1497,11 @@
 	(cond ((null? xs)
 	       (##sys#print "; no values\n" #f ##sys#standard-output))
 	      ((not (eq? (##core#undefined) (car xs)))
-	       (for-each (cut ##sys#repl-print-hook <> ##sys#standard-output) xs) ) ) )
+	       (for-each (cut ##sys#repl-print-hook <> ##sys#standard-output) xs)
+	       (when (pair? (cdr xs))
+		 (##sys#print 
+		  (string-append "; " (##sys#number->string (length xs)) " values\n")
+		  #f ##sys#standard-output)))))
 
       (let ((stdin ##sys#standard-input)
 	    (stdout ##sys#standard-output)
@@ -1605,7 +1539,8 @@
 		  (begin
 		    (##sys#write-char-0 #\newline ##sys#standard-error)
 		    (write-err args) ) )
-	      (print-call-chain ##sys#standard-error)
+	      (set! ##sys#repl-recent-call-chain
+		(print-call-chain ##sys#standard-error))
 	      (flush-output ##sys#standard-error) ) ) )
 	 (lambda ()
 	   (let loop ()
@@ -1784,13 +1719,3 @@
 
 (define-external (CHICKEN_get_error_message ((c-pointer "char") buf) (int bufsize)) void
   (store-string (or last-error "No error") bufsize buf) )
-
-
-;;; Create lambda-info object
-
-(define (##sys#make-lambda-info str)
-  (let* ((sz (##sys#size str))
-	 (info (##sys#make-string sz)) )
-    (##core#inline "C_copy_memory" info str sz)
-    (##core#inline "C_string_to_lambdainfo" info)
-    info) )
