@@ -35,31 +35,14 @@
   (not inline ##sys#syntax-error-hook ##sys#compiler-syntax-hook
        ##sys#alias-global-hook ##sys#toplevel-definition-hook))
 
+(include "common-declarations.scm")
 
 (set! ##sys#features
   (append '(#:hygienic-macros #:syntax-rules) ##sys#features))
 
-(define (d arg1 . more)
-  (when (##sys#fudge 13)
-    (if (null? more)
-	(pp arg1)
-	(apply print arg1 more))) )
-
-(define dd d)
-(define dm d)
-(define dx d)
-
-(cond-expand
- ((not debugbuild)
-  (declare 
-    (no-bound-checks)
-    (no-procedure-checks)))
- (else))
-
-(begin
-  (define-syntax dd (syntax-rules () ((_ . _) (void))))
-  (define-syntax dm (syntax-rules () ((_ . _) (void))))
-  (define-syntax dx (syntax-rules () ((_ . _) (void)))) )
+(define-alias dd d)
+(define-alias dm d)
+(define-alias dx d)
 
 (define-inline (getp sym prop)
   (##core#inline "C_i_getprop" sym prop #f))
@@ -342,8 +325,7 @@
 	   [else (loop (cdr llist))] ) ) ) )
 
 (define ##sys#expand-extended-lambda-list
-  (let ([reverse reverse]
-	[gensym gensym] )
+  (let ([reverse reverse])
     (lambda (llist0 body errh se)
       (define (err msg) (errh msg llist0))
       (define (->keyword s) (string->keyword (##sys#slot s 1)))
@@ -370,7 +352,7 @@
 				,(map (lambda (k)
 					(let ([s (car k)])
 					  `(,s (##sys#get-keyword
-						',(->keyword s) ,rvar
+						',(->keyword (##sys#strip-syntax s)) ,rvar
 						,@(if (pair? (cdr k)) 
 						      `((,%lambda () ,@(cdr k)))
 						      '())))))
@@ -437,11 +419,22 @@
 			    [else (err "invalid lambda list syntax")] ) ] ) ) ] ) ) ) ) ) )
 
 
+;;; Error message for redefinition of currently used defining form
+;
+; (i.e.`"(define define ...)")
+
+(define (##sys#defjam-error form)
+  (##sys#syntax-error-hook
+   "redefinition of currently used defining form" ; help me find something better
+   form))
+
+
 ;;; Expansion of bodies (and internal definitions)
+;
+; This code is disgustingly complex.
 
 (define ##sys#canonicalize-body
-  (let ([reverse reverse]
-	[map map] )
+  (let ([reverse reverse])
     (lambda (body #!optional (se (##sys#current-environment)) cs?)
       (define (fini vars vals mvars mvals body)
 	(if (and (null? vars) (null? mvars))
@@ -495,11 +488,15 @@
 		  (let ((def (car body)))
 		    (loop 
 		     (cdr body) 
-		     (cons (if (pair? (cadr def))
-			       `(##core#define-syntax
-				 ,(caadr def)
-				 (##core#lambda ,(cdadr def) ,@(cddr def)))
-			       def)
+		     (cons (cond ((pair? (cadr def))
+				  `(define-syntax ; (the first element is actually ignored)
+				     ,(caadr def)
+				     (##core#lambda ,(cdadr def) ,@(cddr def))))
+				 ;; insufficient, if introduced by different expansions, but
+				 ;; better than nothing:
+				 ((eq? (car def) (cadr def))
+				  (##sys#defjam-error def))
+				 (else def))
 			   defs) 
 		     #f)))
 		 (else (loop body defs #t))))))		       
@@ -520,6 +517,8 @@
 			 (let ([head (cadr x)])
 			   (cond [(not (pair? head))
 				  (##sys#check-syntax 'define x '(_ variable . #(_ 0)) #f se)
+				  (when (eq? (car x) head) ; see above
+				    (##sys#defjam-error x))
 				  (loop rest (cons head vars)
 					(cons (if (pair? (cddr x))
 						  (caddr x)
@@ -543,6 +542,7 @@
 		       (##sys#check-syntax 'define-syntax x '(_ _ . #(_ 1)) se)
 		       (fini/syntax vars vals mvars mvals body) )
 		      [(eq? 'define-values (or (lookup head se) head))
+		       ;;XXX check for any of the variables being `define-values' (?)
 		       (##sys#check-syntax 'define-values x '(_ #(_ 0) _) #f se)
 		       (loop rest vars vals (cons (cadr x) mvars) (cons (caddr x) mvals)) ]
 		      [(eq? '##core#begin head)
@@ -594,11 +594,61 @@
 
 (define ##sys#line-number-database #f)
 (define ##sys#syntax-error-culprit #f)
+(define ##sys#syntax-context '())
 
 (define (##sys#syntax-error-hook . args)
   (apply ##sys#signal-hook #:syntax-error
 	 (##sys#strip-syntax args)))
 
+(define ##sys#syntax-error/context
+  (lambda (msg arg)
+    (define (syntax-imports sym)
+      (let loop ((defs (or (##sys#get (##sys#strip-syntax sym) '##core#db) '())))
+	(cond ((null? defs) '())
+	      ((eq? 'syntax (caar defs))
+	       (cons (cadar defs) (loop (cdr defs))))
+	      (else (loop (cdr defs))))))		     
+    (if (null? ##sys#syntax-context)
+	(##sys#syntax-error-hook msg arg)
+	(let ((out (open-output-string)))
+	  (define (outstr str)
+	    (##sys#print str #f out))
+	  (let loop ((cx ##sys#syntax-context))
+	    (cond ((null? cx)		; no unimported syntax found
+		   (outstr msg)
+		   (outstr ": ")
+		   (##sys#print arg #t out)
+		   (outstr "\ninside expression `(")
+		   (##sys#print (##sys#strip-syntax (car ##sys#syntax-context)) #t out)
+		   (outstr " ...)'"))
+		  (else 
+		   (let* ((sym (##sys#strip-syntax (car cx)))
+			  (us (syntax-imports sym)))
+		     (cond ((pair? us)
+			    (outstr msg)
+			    (outstr ": ")
+			    (##sys#print arg #t out)
+			    (outstr "\n\n  Perhaps you intended to use the syntax `(")
+			    (##sys#print sym #t out)
+			    (outstr " ...)' without importing it first.\n")
+			    (if (fx= 1 (length us))
+				(outstr
+				 (string-append
+				  "  Suggesting: `(import "
+				  (symbol->string (car us))
+				  ")'"))
+				(outstr
+				 (string-append
+				  "  Suggesting one of:\n"
+				  (let loop ((lst us))
+				    (if (null? lst)
+					""
+					(string-append
+					 "\n      (import " (symbol->string (car lst)) ")'"
+					 (loop (cdr lst)))))))))
+			   (else (loop (cdr cx))))))))
+	  (##sys#syntax-error-hook (get-output-string out))))))
+
 (define syntax-error ##sys#syntax-error-hook)
 
 (define (##sys#syntax-rules-mismatch input)
@@ -619,8 +669,6 @@
 
 (define ##sys#check-syntax
   (let ([string-append string-append]
-	[keyword? keyword?]
-	[get-line-number get-line-number]
 	[symbol->string symbol->string] )
     (lambda (id exp pat #!optional culprit (se (##sys#current-environment)))
 
@@ -784,7 +832,8 @@
   (let ((%only (r 'only))
 	(%rename (r 'rename))
 	(%except (r 'except))
-	(%prefix (r 'prefix)))
+	(%prefix (r 'prefix))
+	(%srfi (r 'srfi)))
     (define (resolve sym)
       (or (lookup sym '()) sym))	;*** empty se?
     (define (tostr x)
@@ -802,9 +851,11 @@
 		     #t)))
 	    (cond (il (parameterize ((##sys#current-module #f)
 				     (##sys#current-environment '())
-				     (##sys#current-meta-environment (##sys#current-meta-environment))
+				     (##sys#current-meta-environment 
+				      (##sys#current-meta-environment))
 				     (##sys#macro-environment (##sys#meta-macro-environment)))
-			(##sys#load il #f #f))
+			(fluid-let ((##sys#notices-enabled #f)) ; to avoid re-import warnings
+			  (##sys#load il #f #f)))
 		      (set! mod (##sys#find-module mname)))
 		  (else
 		   (syntax-error
@@ -817,12 +868,16 @@
       (cond ((symbol? spec) (import-name spec))
 	    ((or (not (list? spec)) (< (length spec) 2))
 	     (syntax-error loc "invalid import specification" spec))
+	    ((and (c %srfi (car spec)) (fixnum? (cadr spec)) (null? (cddr spec))) ; only one number
+	     (import-name 
+	      (##sys#intern-symbol
+	       (##sys#string-append "srfi-" (##sys#number->string (cadr spec))))))
 	    (else
 	     (let* ((s (car spec))
 		    (imp (import-spec (cadr spec)))
 		    (impv (car imp))
 		    (imps (cdr imp)))
-	       (cond ((c %only (car spec))
+	       (cond ((c %only s)
 		      (##sys#check-syntax loc spec '(_ _ . #(symbol 0)))
 		      (let ((ids (map resolve (cddr spec))))
 			(let loop ((ids ids) (v '()) (s '()))
@@ -834,7 +889,7 @@
 				 (lambda (a) 
 				   (loop (cdr ids) v (cons a s))))
 				(else (loop (cdr ids) v s))))))
-		     ((c %except (car spec))
+		     ((c %except s)
 		      (##sys#check-syntax loc spec '(_ _ . #(symbol 0)))
 		      (let ((ids (map resolve (cddr spec))))
 			(let loop ((impv impv) (v '()))
@@ -845,7 +900,7 @@
 					 (else (loop (cdr imps) (cons (car imps) s))))))
 				((memq (caar impv) ids) (loop (cdr impv) v))
 				(else (loop (cdr impv) (cons (car impv) v)))))))
-		     ((c %rename (car spec))
+		     ((c %rename s)
 		      (##sys#check-syntax loc spec '(_ _ . #((symbol symbol) 0)))
 		      (let loop ((impv impv) (imps imps) (v '()) (s '()) (ids (cddr spec)))
 			(cond ((null? impv) 
@@ -871,7 +926,7 @@
 			      (else (loop (cdr impv) imps
 					  (cons (car impv) v)
 					  s ids)))))
-		     ((c %prefix (car spec))
+		     ((c %prefix s)
 		      (##sys#check-syntax loc spec '(_ _ _))
 		      (let ((pref (tostr (caddr spec))))
 			(define (ren imp)
@@ -925,7 +980,8 @@
 	     (set-module-export-list! 
 	      cm
 	      (append 
-	       (module-export-list cm) 
+	       (let ((xl (module-export-list cm) ))
+		 (if (eq? #t xl) '() xl))
 	       (map car vsv)
 	       (map car vss)))
 	     (when (pair? prims)
@@ -1017,6 +1073,8 @@
 	(cond ((not (pair? head))
 	       (##sys#check-syntax 'define form '(_ symbol . #(_ 0 1)))
 	       (##sys#register-export head (##sys#current-module))
+	       (when (c (r 'define) head)
+		 (##sys#defjam-error x))
 	       `(##core#set! 
 		 ,head 
 		 ,(if (pair? body) (car body) '(##core#undefined))) )
@@ -1038,10 +1096,16 @@
 	     (##sys#check-syntax 'define-syntax head 'symbol)
 	     (##sys#check-syntax 'define-syntax body '#(_ 1))
 	     (##sys#register-export head (##sys#current-module))
+	     (when (c (r 'define-syntax) head)
+	       (##sys#defjam-error form))
 	     `(##core#define-syntax ,head ,(car body)))
 	    (else
 	     (##sys#check-syntax 'define-syntax head '(_ . lambda-list))
 	     (##sys#check-syntax 'define-syntax body '#(_ 1))
+	     (when (eq? (car form) (car head))
+	       (##sys#syntax-error-hook
+		"redefinition of `define-syntax' not allowed in syntax-definition"
+		form))
 	     `(##core#define-syntax 
 	       ,(car head)
 	       (##core#lambda ,(cdr head) ,@body))))))))
@@ -1141,7 +1205,7 @@
 		     (expand rclauses #t)
 		     `(##core#begin ,@(cdr clause)))
 		    (else?
-		     (##sys#notice
+		     (##sys#warn
 		      "non-`else' clause following `else' clause in `cond'"
 		      (##sys#strip-syntax clause))
 		     (expand rclauses #t)
@@ -1430,8 +1494,10 @@
 	 exps)
 	(set-module-export-list! 
 	 mod
-	 (append (module-export-list mod) 
-		 (map ##sys#strip-syntax exps))))
+	 (let ((xl (module-export-list mod)))
+	   (if (eq? xl #t) 
+	       #t
+	       (append xl (map ##sys#strip-syntax exps))))))
       '(##core#undefined)))))
 
 
@@ -1799,7 +1865,7 @@
 				       (set! missing #t)
 				       (##sys#warn 
 					(string-append 
-					 "exported identifier for module `" 
+					 "exported identifier of module `" 
 					 (symbol->string name)
 					 "' has not been defined")
 					id)
@@ -1824,12 +1890,12 @@
 		   (if (null? lst)
 		       ""
 		       (string-append
-			"Warning:     `(import " (symbol->string (cadar lst)) ")'\n"
+			"Warning:     (import " (symbol->string (cadar lst)) ")\n"
 			(loop (cdr lst)))))))))))
      (module-undefined-list mod))
     (when missing
       (##sys#error "module unresolved" name))
-    (let* ((exports 
+    (let* ((iexports 
 	    (map (lambda (exp)
 		   (cond ((symbol? (cdr exp)) exp)
 			 ((assq (car exp) (##sys#macro-environment)))
@@ -1838,11 +1904,11 @@
 	   (new-se (merge-se 
 		    (##sys#macro-environment) 
 		    (##sys#current-environment) 
-		    exports)))
-      (##sys#mark-imported-symbols exports)
+		    iexports vexports sexports sdlist)))
+      (##sys#mark-imported-symbols iexports)
       (for-each
        (lambda (m)
-	 (let ((se (merge-se (cadr m) new-se)))
+	 (let ((se (merge-se (cadr m) new-se))) ;XXX needed?
 	   (dm `(FIXUP: ,(car m) ,@(map-se se)))
 	   (set-car! (cdr m) se)))
        sdlist)
@@ -1850,7 +1916,7 @@
 	    ,(module-name mod) 
 	    (DLIST: ,@dlist)
 	    (SDLIST: ,@(map-se sdlist))
-	    (IEXPORTS: ,@(map-se exports))
+	    (IEXPORTS: ,@(map-se iexports))
 	    (VEXPORTS: ,@(map-se vexports))
 	    (SEXPORTS: ,@(map-se sexports))))
       (set-module-vexports! mod vexports)
