@@ -32,7 +32,9 @@
    make-pathname
    directory-null?
    make-absolute-pathname
+   create-temporary-directory
    create-temporary-file
+   decompose-directory
    decompose-pathname
    absolute-pathname?
    pathname-directory
