@@ -37,38 +37,34 @@
 (declare
   (unit files)
   (uses regex data-structures)
-  (usual-integrations)
-  (fixnum)
   (hide chop-pds absolute-pathname-root root-origin root-directory split-directory)
-  (disable-interrupts) )
+  (disable-interrupts) 
+  (foreign-declare #<<EOF
+#include <unistd.h>
+#include <errno.h>
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (always-bound
-      ##sys#windows-platform)
-    (bound-to-procedure
-      string-match regexp
-      ##sys#string-append ##sys#substring  string-append
-      get-environment-variable
-      file-exists? delete-file
-      call-with-output-file read-string)
-    (no-procedure-checks-for-usual-bindings)
-    (no-bound-checks))] )
+#ifndef _WIN32
+# include <sys/stat.h>
+# define C_mkdir(str)       C_fix(mkdir(C_c_string(str), S_IRWXU | S_IRWXG | S_IRWXO))
+#else
+# define C_mkdir(str)	    C_fix(mkdir(C_c_string(str)))
+#endif
+EOF
+))
 
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'files)
 
 
+(define-foreign-variable strerror c-string "strerror(errno)")
+
+
 ;;; Like `delete-file', but does nothing if the file doesn't exist:
 
 (define delete-file*
-  (let ([file-exists? file-exists?]
-	[delete-file delete-file] )
-    (lambda (file)
-      (and (file-exists? file) (delete-file file)) ) ) )
+  (lambda (file)
+    (and (file-exists? file) (delete-file file)) ) )
 
 ;;; file-copy and file-move : they do what you'd think.
 (define (file-copy origfile newfile #!optional (clobber #f) (blocksize 1024))
@@ -97,9 +93,9 @@
                                          "could not open newfile for write - "
                                          newfile)))))
            (s   (make-string blocksize)))
-        (let loop ((d   (read-string! blocksize s i))
-                   (l   0))
-            (if (= 0 d)
+        (let loop ((d (read-string! blocksize s i))
+                   (l 0))
+            (if (fx= 0 d)
                 (begin
                     (close-input-port i)
                     (close-output-port o)
@@ -112,55 +108,55 @@
                             (##sys#error (string-append
                                              "error writing file starting at "
                                              (number->string l)))))
-                    (loop (read-string! blocksize s i) (+ d l)))))))
+                    (loop (read-string! blocksize s i) (fx+ d l)))))))
 
 (define (file-move origfile newfile #!optional (clobber #f) (blocksize 1024))
-    (##sys#check-string origfile 'file-move)
-    (##sys#check-string newfile 'file-move)
-    (##sys#check-number blocksize 'file-move)
-    (or (and (integer? blocksize) (> blocksize 0))
-        (##sys#error (string-append
-                         "invalid blocksize given: not a positive integer - "
-                         (number->string blocksize))))
-    (or (file-exists? origfile)
-        (##sys#error (string-append "origfile does not exist - " origfile)))
-    (and (file-exists? newfile)
-         (or clobber
-             (##sys#error (string-append
-                              "newfile exists but clobber is false - "
-                              newfile))))
-    (let* ((i   (condition-case (open-input-file origfile)
-                    (val ()
-                        (##sys#error (string-append
-                                         "could not open origfile for read - "
-                                         origfile)))))
-           (o   (condition-case (open-output-file newfile)
-                    (val ()
-                        (##sys#error (string-append
-                                         "could not open newfile for write - "
-                                         newfile)))))
-           (s   (make-string blocksize)))
-        (let loop ((d   (read-string! blocksize s i))
-                   (l   0))
-            (if (= 0 d)
-                (begin
-                    (close-input-port i)
-                    (close-output-port o)
-                    (condition-case (delete-file origfile)
-                        (val ()
-                            (##sys#error (string-append
-                                             "could not remove origfile - "
-                                             origfile))))
-                    l)
-                (begin
-                    (condition-case (write-string s d o)
-                        (val ()
-                            (close-input-port i)
-                            (close-output-port o)
-                            (##sys#error (string-append
-                                             "error writing file starting at "
-                                             (number->string l)))))
-                    (loop (read-string! blocksize s i) (+ d l)))))))
+  (##sys#check-string origfile 'file-move)
+  (##sys#check-string newfile 'file-move)
+  (##sys#check-number blocksize 'file-move)
+  (or (and (integer? blocksize) (> blocksize 0))
+      (##sys#error (string-append
+		    "invalid blocksize given: not a positive integer - "
+		    (number->string blocksize))))
+  (or (file-exists? origfile)
+      (##sys#error (string-append "origfile does not exist - " origfile)))
+  (and (file-exists? newfile)
+       (or clobber
+	   (##sys#error (string-append
+			 "newfile exists but clobber is false - "
+			 newfile))))
+  (let* ((i   (condition-case (open-input-file origfile)
+		(val ()
+		     (##sys#error (string-append
+				   "could not open origfile for read - "
+				   origfile)))))
+	 (o   (condition-case (open-output-file newfile)
+		(val ()
+		     (##sys#error (string-append
+				   "could not open newfile for write - "
+				   newfile)))))
+	 (s   (make-string blocksize)))
+    (let loop ((d (read-string! blocksize s i))
+	       (l 0))
+      (if (fx= 0 d)
+	  (begin
+	    (close-input-port i)
+	    (close-output-port o)
+	    (condition-case (delete-file origfile)
+	      (val ()
+		   (##sys#error (string-append
+				 "could not remove origfile - "
+				 origfile))))
+	    l)
+	  (begin
+	    (condition-case (write-string s d o)
+	      (val ()
+		   (close-input-port i)
+		   (close-output-port o)
+		   (##sys#error (string-append
+				 "error writing file starting at "
+				 (number->string l)))))
+	    (loop (read-string! blocksize s i) (fx+ d l)))))))
 
 ;;; Pathname operations:
 
@@ -173,16 +169,15 @@
 (define root-origin)
 (define root-directory)
 
-(let ((string-match string-match))
-  (if ##sys#windows-platform
-      (let ((rx (regexp "([A-Za-z]:)?([\\/\\\\]).*")))
-        (set! absolute-pathname-root (lambda (pn) (string-match rx pn)))
-        (set! root-origin (lambda (rt) (and rt (cadr rt))))
-        (set! root-directory (lambda (rt) (and rt (caddr rt)))) )
-      (let ((rx (regexp "([\\/\\\\]).*")))
-        (set! absolute-pathname-root (lambda (pn) (string-match rx pn)))
-        (set! root-origin (lambda (rt) #f))
-        (set! root-directory (lambda (rt) (and rt (cadr rt)))) ) ) )
+(if ##sys#windows-platform
+    (let ((rx (regexp "([A-Za-z]:)?([\\/\\\\]).*")))
+      (set! absolute-pathname-root (lambda (pn) (string-match rx pn)))
+      (set! root-origin (lambda (rt) (and rt (cadr rt))))
+      (set! root-directory (lambda (rt) (and rt (caddr rt)))) )
+    (let ((rx (regexp "([\\/\\\\]).*")))
+      (set! absolute-pathname-root (lambda (pn) (string-match rx pn)))
+      (set! root-origin (lambda (rt) #f))
+      (set! root-directory (lambda (rt) (and rt (cadr rt)))) ) )
 
 (define (absolute-pathname? pn)
   (##sys#check-string pn 'absolute-pathname?)
@@ -203,8 +198,8 @@
 
 (define make-pathname)
 (define make-absolute-pathname)
+
 (let ([string-append string-append]
-      [absolute-pathname? absolute-pathname?]
       [def-pds "/"] )
 
   (define (conc-dirs dirs pds)
@@ -248,126 +243,140 @@
        ext) ) )
 
   (set! make-pathname
-    (lambda (dirs file #!optional ext pds) ; The 'pds' argument is DEPRECATED
-      (_make-pathname 'make-pathname (canonicalize-dirs dirs pds) file ext pds)))
+    (lambda (dirs file #!optional ext)
+      (_make-pathname 'make-pathname (canonicalize-dirs dirs def-pds) file ext def-pds)))
 
   (set! make-absolute-pathname
-    (lambda (dirs file #!optional ext pds) ; The 'pds' argument is DEPRECATED
+    (lambda (dirs file #!optional ext)
       (_make-pathname
        'make-absolute-pathname
-       (let ([dir (canonicalize-dirs dirs pds)])
+       (let ([dir (canonicalize-dirs dirs def-pds)])
 	 (if (absolute-pathname? dir)
 	     dir
-	     (##sys#string-append (or pds def-pds) dir)) )
-       file ext pds) ) ) )
+	     (##sys#string-append def-pds dir)) )
+       file ext def-pds) ) ) )
 
 (define decompose-pathname
-  (let ((string-match string-match))
-    (let* ([patt1 "^(.*[\\/\\\\])?([^\\/\\\\]+)(\\.([^\\/\\\\.]+))$"]
-	   [patt2 "^(.*[\\/\\\\])?((\\.)?[^\\/\\\\]+)$"]
-	   [rx1 (regexp patt1)]
-	   [rx2 (regexp patt2)]
-	   [strip-pds
-	     (lambda (dir)
-	       (and dir
-		    (if (member dir '("/" "\\"))
-		        dir
-		        (chop-pds dir #f) ) ) )] )
-      (lambda (pn)
-        (##sys#check-string pn 'decompose-pathname)
-        (if (fx= 0 (##sys#size pn))
-	    (values #f #f #f)
-	    (let ([ms (string-match rx1 pn)])
-	      (if ms
-		  (values (strip-pds (cadr ms)) (caddr ms) (car (cddddr ms)))
-		  (let ([ms (string-match rx2 pn)])
-		    (if ms
-		        (values (strip-pds (cadr ms)) (caddr ms) #f)
-		        (values (strip-pds pn) #f #f) ) ) ) ) ) ) ) ) )
-
-(define pathname-directory)
-(define pathname-file)
-(define pathname-extension)
-(define pathname-strip-directory)
-(define pathname-strip-extension)
-(define pathname-replace-directory)
-(define pathname-replace-file)
-(define pathname-replace-extension)
-(let ([decompose-pathname decompose-pathname])
-
-  (set! pathname-directory
+  (let* ([patt1 "^(.*[\\/\\\\])?([^\\/\\\\]+)(\\.([^\\/\\\\.]+))$"]
+	 [patt2 "^(.*[\\/\\\\])?((\\.)?[^\\/\\\\]+)$"]
+	 [rx1 (regexp patt1)]
+	 [rx2 (regexp patt2)]
+	 [strip-pds
+	  (lambda (dir)
+	    (and dir
+		 (if (member dir '("/" "\\"))
+		     dir
+		     (chop-pds dir #f) ) ) )] )
     (lambda (pn)
-      (let-values ([(dir file ext) (decompose-pathname pn)])
-	dir) ) )
+      (##sys#check-string pn 'decompose-pathname)
+      (if (fx= 0 (##sys#size pn))
+	  (values #f #f #f)
+	  (let ([ms (string-match rx1 pn)])
+	    (if ms
+		(values (strip-pds (cadr ms)) (caddr ms) (car (cddddr ms)))
+		(let ([ms (string-match rx2 pn)])
+		  (if ms
+		      (values (strip-pds (cadr ms)) (caddr ms) #f)
+		      (values (strip-pds pn) #f #f) ) ) ) ) ) ) ) )
 
-  (set! pathname-file
-    (lambda (pn)
-      (let-values ([(dir file ext) (decompose-pathname pn)])
-	file) ) )
+(define pathname-directory
+  (lambda (pn)
+    (let-values ([(dir file ext) (decompose-pathname pn)])
+      dir) ) )
 
-  (set! pathname-extension
-    (lambda (pn)
-      (let-values ([(dir file ext) (decompose-pathname pn)])
-	ext) ) )
+(define pathname-file
+  (lambda (pn)
+    (let-values ([(dir file ext) (decompose-pathname pn)])
+      file) ) )
 
-  (set! pathname-strip-directory
-    (lambda (pn)
-      (let-values ([(dir file ext) (decompose-pathname pn)])
-	(make-pathname #f file ext) ) ) )
+(define pathname-extension
+  (lambda (pn)
+    (let-values ([(dir file ext) (decompose-pathname pn)])
+      ext) ) )
 
-  (set! pathname-strip-extension
-    (lambda (pn)
-      (let-values ([(dir file ext) (decompose-pathname pn)])
-	(make-pathname dir file) ) ) )
+(define pathname-strip-directory
+  (lambda (pn)
+    (let-values ([(dir file ext) (decompose-pathname pn)])
+      (make-pathname #f file ext) ) ) )
 
-  (set! pathname-replace-directory
-    (lambda (pn dir)
-      (let-values ([(_ file ext) (decompose-pathname pn)])
-	(make-pathname dir file ext) ) ) )
+(define pathname-strip-extension
+  (lambda (pn)
+    (let-values ([(dir file ext) (decompose-pathname pn)])
+      (make-pathname dir file) ) ) )
 
-  (set! pathname-replace-file
-    (lambda (pn file)
-      (let-values ([(dir _ ext) (decompose-pathname pn)])
-	(make-pathname dir file ext) ) ) )
+(define pathname-replace-directory
+  (lambda (pn dir)
+    (let-values ([(_ file ext) (decompose-pathname pn)])
+      (make-pathname dir file ext) ) ) )
 
-  (set! pathname-replace-extension
-    (lambda (pn ext)
-      (let-values ([(dir file _) (decompose-pathname pn)])
-	(make-pathname dir file ext) ) ) ) )
+(define pathname-replace-file
+  (lambda (pn file)
+    (let-values ([(dir _ ext) (decompose-pathname pn)])
+      (make-pathname dir file ext) ) ) )
 
-(define create-temporary-file
-  (let ([get-environment-variable get-environment-variable]
-	[make-pathname make-pathname]
-	[file-exists? file-exists?]
-	[call-with-output-file call-with-output-file] )
-    (lambda ext
-      (let ((dir (or (get-environment-variable "TMPDIR") 
-		     (get-environment-variable "TEMP")
-		     (get-environment-variable "TMP")
-		     (file-exists? "/tmp")))
-	    (ext (if (pair? ext) (car ext) "tmp")))
-	(##sys#check-string ext 'create-temporary-file)
-	(let loop ()
-	  (let* ([n (##sys#fudge 16)]
-		 [pn (make-pathname dir (##sys#string-append "t" (number->string n 16)) ext)] )
-	    (if (file-exists? pn)
-		(loop)
-		(call-with-output-file pn (lambda (p) pn)) ) ) ) ) ) ) )
+(define pathname-replace-extension
+  (lambda (pn ext)
+    (let-values ([(dir file _) (decompose-pathname pn)])
+      (make-pathname dir file ext) ) ) )
+
+(define create-temporary-file)
+(define create-temporary-directory)
+
+(let ((call-with-output-file call-with-output-file)
+      (temp #f)
+      (temp-prefix "temp"))
+  (define (tempdir)
+    (or temp
+	(let ((tmp 
+	       (or (get-environment-variable "TMPDIR") 
+		   (get-environment-variable "TEMP")
+		   (get-environment-variable "TMP")
+		   "/tmp")))
+	  (set! temp tmp)
+	  tmp)))
+  (set! create-temporary-file
+    (lambda (#!optional (ext "tmp"))
+      (##sys#check-string ext 'create-temporary-file)
+      (let loop ()
+	(let* ((n (##core#inline "C_random_fixnum" #x10000))
+	       (pn (make-pathname 
+		    (tempdir)
+		    (##sys#string-append 
+		     temp-prefix
+		     (number->string n 16)) ext)) )
+	  (if (file-exists? pn)
+	      (loop)
+	      (call-with-output-file pn (lambda (p) pn)) ) ) ) ) )
+  (set! create-temporary-directory
+    (lambda ()
+      (let loop ()
+	(let* ((n (##core#inline "C_random_fixnum" #x10000))
+	       (pn (make-pathname 
+		    (tempdir)
+		    (string-append
+		     temp-prefix
+		     (number->string n 16)))))
+	  (if (directory-exists? pn) 
+	      (loop)
+	      (let ((r (##core#inline "C_mkdir" (##sys#make-c-string pn 'create-temporary-directory))))
+		(if (eq? r 0)
+		    pn
+		    (##sys#signal-hook 
+		     #:file-error 'create-temporary-directory
+		     (##sys#string-append "cannot create temporary directory - " strerror)
+		     pn) ))))))))
 
 
 ;;; normalize pathname for a particular platform
 
 (define normalize-pathname
-  (let ((open-output-string open-output-string)
-	(get-output-string get-output-string)
-	(get-environment-variable get-environment-variable)
-	(reverse reverse)
+  (let ((reverse reverse)
 	(display display)
 	(bldplt (if (memq (build-platform) '(msvc mingw32)) 'windows 'unix)) )
     (define (addpart part parts)
-      (cond ((string=? "." part)        parts )
-            ((string=? ".." part)       (if (null? parts) '("..") (cdr parts)) )
-            (else                       (cons part parts) ) ) )
+      (cond ((string=? "." part) parts)
+            ((string=? ".." part) (if (null? parts) '("..") (cdr parts)))
+            (else (cons part parts) ) ) )
     (lambda (path #!optional (platform bldplt))
       (let ((sep (if (eq? platform 'windows) #\\ #\/)))
 	(##sys#check-string path 'normalize-pathname)
@@ -419,10 +428,9 @@
 ;; does arg check
 
 (define split-directory
-  (let ((string-split string-split) )
-    (lambda (loc dir keep?)
-      (##sys#check-string dir loc)
-      (string-split dir "/\\" keep?) ) ) )
+  (lambda (loc dir keep?)
+    (##sys#check-string dir loc)
+    (string-split dir "/\\" keep?) ) )
 
 ;; Directory string or list only contains path-separators
 ;; and/or current-directory (".") names.
