@@ -90,7 +90,8 @@
 	     (scan val e)
 	     (let ((p (alist-ref var previous)))
 	       (when (and p (not (memq var unsafe)))
-		 (##sys#notice
+		 ;; disabled for the moment - this doesn't really look like it's helpful
+		 #;(##sys#notice
 		  (sprintf "dropping assignment of unused value to global variable `~s'"
 		    var))
 		 (copy-node!
@@ -275,7 +276,7 @@
 				  (or (test var 'value)
 				      (test var 'local-value)))]
 		       [args (cdr subs)] )
-		  (cond [(test var 'contractable)
+		  (cond ((test var 'contractable)
 			 (let* ([lparams (node-parameters lval)]
 				[llist (third lparams)] )
 			   (check-signature var args llist)
@@ -284,43 +285,52 @@
 			   (for-each (cut put! db <> 'inline-target #t) fids)
 			   (walk
 			    (inline-lambda-bindings llist args (first (node-subexpressions lval)) #f db)
-			    fids) ) ]
-			[(memq var constant-declarations)
-			 (or (and-let* ((k (car args))
-					((eq? '##core#variable (node-class k)))
-					(kvar (first (node-parameters k)))
-					(lval (and (not (test kvar 'unknown)) (test kvar 'value))) 
-					((eq? '##core#lambda (node-class lval)))
-					(llist (third (node-parameters lval)))
-					((or (test (car llist) 'unused)
-					     (and (not (test (car llist) 'references))
-						  (not (test (car llist) 'assigned)))))
-					((not (any (cut expression-has-side-effects? <> db) (cdr args) ))))
-			       (debugging 'x "removed call to constant procedure with unused result" var)
-			       (make-node
-				'##core#call '(#t)
-				(list k (make-node '##core#undefined '() '())) ) ) 
-			     (walk-generic n class params subs fids)) ]
-			[(and lval
+			    fids) ) )
+			((variable-mark var '##compiler#pure) =>
+			 (lambda (pb)
+			   (or (and-let* ((k (car args))
+					  ((or (eq? #t pb) 
+					       (let ((im (variable-mark var '##compiler#intrinsic)))
+						 (or (eq? im 'internal) (eq? im pb)))))
+					  ((eq? '##core#variable (node-class k)))
+					  (kvar (first (node-parameters k)))
+					  (lval (and (not (test kvar 'unknown)) (test kvar 'value))) 
+					  ((eq? '##core#lambda (node-class lval)))
+					  (llist (third (node-parameters lval)))
+					  ((or (test (car llist) 'unused)
+					       (and (not (test (car llist) 'references))
+						    (not (test (car llist) 'assigned)))))
+					  ((not (any (cut expression-has-side-effects? <> db) (cdr args) ))))
+				 (let ((info (and (pair? (cdr params)) (second params))))
+				   (debugging 
+				    'o
+				    "removed call to pure procedure with unused result"
+				    (or (source-info->string info) var)))
+				 (make-node
+				  '##core#call '(#t)
+				  (list k (make-node '##core#undefined '() '())) ) ) 
+			       (walk-generic n class params subs fids)) ) )
+			((and lval
 			      (eq? '##core#lambda (node-class lval)))
 			 (let* ([lparams (node-parameters lval)]
 				[llist (third lparams)] )
 			   (decompose-lambda-list
 			    llist
 			    (lambda (vars argc rest)
-			      (let ([ifid (first lparams)])
-				(cond [(and inline-locally 
+			      (let ((ifid (first lparams))
+				    (external (node? (variable-mark var '##compiler#inline-global))))
+				(cond ((and inline-locally 
 					    (test var 'inlinable)
 					    (not (test ifid 'inline-target)) ; inlinable procedure has changed
 					    (case (variable-mark var '##compiler#inline) 
 					      ((yes) #t)
 					      ((no) #f)
 					      (else 
-					       (< (fourth lparams) inline-max-size) ) ))
+					       (or external (< (fourth lparams) inline-max-size)))))
 				       (debugging 
 					'i
-					(if (node? (variable-mark var '##compiler#inline-global))
-					    "global inlining" 
+					(if external
+					    "global inlining" 	
 					    "inlining")
 					var ifid (fourth lparams))
 				       (for-each (cut put! db <> 'inline-target #t) fids)
@@ -329,8 +339,8 @@
 				       (touch)
 				       (walk
 					(inline-lambda-bindings llist args (first (node-subexpressions lval)) #t db)
-					fids) ]
-				      [(test ifid 'has-unused-parameters)
+					fids) )
+				      ((test ifid 'has-unused-parameters)
 				       (if (< (length args) argc) ; Expression was already optimized (should this happen?)
 					   (walk-generic n class params subs fids)
 					   (let loop ((vars vars) (argc argc) (args args) (used '()))
@@ -355,8 +365,8 @@
 						   [else (loop (cdr vars)
 							       (sub1 argc)
 							       (cdr args)
-							       (cons (car args) used) ) ] ) ) ) ]
-				      [(and (test ifid 'explicit-rest)
+							       (cons (car args) used) ) ] ) ) ) )
+				      ((and (test ifid 'explicit-rest)
 					    (not (memq n rest-consers)) ) ; make sure we haven't inlined rest-list already
 				       (let ([n (llist-length llist)])
 					 (if (< (length args) n)
@@ -379,9 +389,9 @@
 									      (list "C_a_i_list" (* 3 (length rargs)))
 									      rargs) ) ) ) ) ) ) ] )
 						   (set! rest-consers (cons n2 rest-consers))
-						   n2) ) ) ) ) ]
-				      [else (walk-generic n class params subs fids)] ) ) ) ) ) ]
-			[else (walk-generic n class params subs fids)] ) ) ]
+						   n2) ) ) ) ) )
+				      (else (walk-generic n class params subs fids)) ) ) ) ) ) )
+			(else (walk-generic n class params subs fids)) ) ) ]
 	       [(##core#lambda)
 		(if (first params)
 		    (walk-generic n class params subs fids)
@@ -846,12 +856,15 @@
 	      cont
 	      (make-node '##core#inline (list (second classargs)) callargs) ) ) ) ) )
 
-    ;; (<op>) -> <var>
+    ;; (<op> ...) -> <var>
     ((3) ; classargs = (<var>)
      (and inline-substitutions-enabled
-	  (null? callargs)
 	  (intrinsic? name)
-	  (make-node '##core#call '(#t) (list cont (varnode (first classargs)))) ) )
+	  (fold-right
+	   (lambda (val body)
+	     (make-node 'let (list (gensym "t")) (list val body)) )
+	   (make-node '##core#call '(#t) (list cont (varnode (first classargs))))
+	   callargs)))
 
     ;; (<op> a b) -> (<primitiveop> a (quote <i>) b)
     ((4) ; classargs = (<primitiveop> <i>)
@@ -1021,15 +1034,18 @@
 		(else #f) ) ) )
 
     ;; (<alloc-op> ...) -> (##core#inline_allocate (<aiop> <words>) ...)
-    ((16) ; classargs = (<argc> <aiop> <safe> <words>)
+    ((16) ; classargs = (<argc> <aiop> <safe> <words> [<counted>])
      ;; - <argc> may be #f, saying that any number of arguments is allowed,
      ;; - <words> may be a list of one element (the number of words), meaning that
      ;;   the words are to be multiplied with the number of arguments.
      ;; - <words> may also be #t, meaning that the number of words is the same as the
      ;;   number of arguments plus 1.
-     (let ([argc (first classargs)]
-	   [rargc (length callargs)]
-	   [w (fourth classargs)] )
+     ;; - if <counted> is given and true and <argc> is between 1-8, append "<count>"
+     ;;   to the name of the inline routine.
+     (let ((argc (first classargs))
+	   (rargc (length callargs))
+	   (w (fourth classargs))
+	   (counted (and (pair? (cddddr classargs)) (fifth classargs))))
        (and inline-substitutions-enabled
 	    (or (not argc) (= rargc argc))
 	    (intrinsic? name)
@@ -1039,7 +1055,9 @@
 	     (list cont 
 		   (make-node
 		    '##core#inline_allocate
-		    (list (second classargs) 
+		    (list (if (and counted (positive? rargc) (<= rargc 8))
+			      (conc (second classargs) rargc)
+			      (second classargs) )
 			  (cond [(eq? #t w) (add1 rargc)]
 				[(pair? w) (* rargc (car w))]
 				[else w] ) )
