@@ -10,12 +10,10 @@
 ;
 ;   Redistributions of source code must retain the above copyright
 ;   notice, this list of conditions and the following disclaimer.
-
 ;   Redistributions in binary form must reproduce the above copyright
 ;   notice, this list of conditions and the following disclaimer in
 ;   the documentation and/or other materials provided with the
 ;   distribution.
-
 ;   Neither the name of the author nor the names of its contributors
 ;   may be used to endorse or promote products derived from this
 ;   software without specific prior written permission.
@@ -36,37 +34,10 @@
 
 (declare
  (unit ports)
-; (uses data-structures)
- (usual-integrations)
- (disable-warning redef) )
+ (hide read-and-write write-buf read-buf read-and-write-buf))
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (no-bound-checks)
-    (no-procedure-checks-for-usual-bindings)
-    (bound-to-procedure
-      ##sys#check-char ##sys#check-exact ##sys#check-port ##sys#check-string
-      ##sys#substring ##sys#for-each ##sys#map ##sys#setslot
-      ##sys#allocate-vector ##sys#check-pair ##sys#error-not-a-proper-list
-      ##sys#member ##sys#assoc ##sys#error ##sys#signal-hook ##sys#read-string!
-      ##sys#check-symbol ##sys#check-vector ##sys#check-number
-      ##sys#flonum-fraction ##sys#make-port ##sys#fetch-and-check-port-arg
-      ##sys#print ##sys#check-structure ##sys#make-structure make-parameter
-      ##sys#flush-output ##sys#write-char-0 ##sys#number->string
-      ##sys#fragments->string ##sys#symbol->qualified-string
-      ##sys#number? ##sys#procedure->string
-      ##sys#pointer->string ##sys#user-print-hook ##sys#peek-char-0
-      ##sys#read-char-0 ##sys#write-char ##sys#string-append ##sys#gcd ##sys#lcm
-      ##sys#fudge ##sys#check-list ##sys#user-read-hook ##sys#check-closure ##sys#check-inexact
-      input-port? make-vector list->vector  open-output-string floor 
-      get-output-string current-output-port display write port? list->string
-      call-with-input-string with-input-from-string
-      make-string string newline char-name read 
-      open-input-string call-with-input-file reverse ) ) ] )
 
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'ports)
 
@@ -90,18 +61,73 @@
 	      (loop (cons (fn x) xs))))))))
 
 (define (port-fold fn acc thunk)
-  (let loop ([acc acc])
-    (let ([x (thunk)])
+  (let loop ((acc acc))
+    (let ((x (thunk)))
       (if (eof-object? x)
           acc
           (loop (fn x acc))) ) ) )
 
+(define-constant +buf-size+ 1024)
+
+(define (read-buf port writer)
+  (let ((buf (make-string +buf-size+)))
+    (let loop ()
+      (let ((n (read-string! +buf-size+ buf port)))
+	(unless (eq? n 0)
+	  (writer buf n)
+	  (loop))))))
+
+(define (write-buf buf n port writer)
+  (do ((i 0 (fx+ i 1)))
+      ((fx>= i n))
+    (writer (integer->char (##sys#byte buf i)) port)))
+
+(define (read-and-write reader writer)
+  (let loop ()
+    (let ((x (reader)))
+      (unless (eof-object? x)
+	(writer x)
+	(loop)))))
+
+(define (read-and-write-buf src dest reader)
+  (let ((buf (make-string +buf-size+)))
+    (let loop ((n 0))
+      (when (fx>= n +buf-size+)
+	(write-string buf +buf-size+ dest)
+	(set! n 0))
+      (let ((c (reader src)))
+	(cond ((eof-object? c)
+	       (when (fx>= n 0)
+		 (write-string buf n dest)))
+	      (else
+	       (##sys#setbyte buf n (char->integer c))
+	       (loop (fx+ n 1))))))))
+
+(define copy-port 
+  (let ((read-char read-char)
+	(write-char write-char))
+    (lambda (src dest #!optional (read read-char) (write write-char))
+      ;; does not check port args intentionally
+      (cond ((eq? read read-char)
+	     (read-buf
+	      src
+	      (if (eq? write write-char)
+		  (lambda (buf n) (write-string buf n dest))
+		  (lambda (buf n) (write-buf buf n dest write)))))
+	    ((eq? write write-char)
+	     (read-and-write-buf src dest read))
+	    (else
+	     (read-and-write 
+	      (lambda () (read src))
+	      (lambda (x) (write x dest))))))))
+
+
 ;;;; funky-ports
 
 (define (make-broadcast-port . ports)
   (make-output-port
    (lambda (s) (for-each (cut write-string s #f <>) ports))
-   noop
+   void
    (lambda () (for-each flush-output ports)) ) )
 
 (define (make-concatenated-port p1 . ports)
@@ -119,7 +145,7 @@
      (lambda ()
        (and (not (null? ports))
 	    (char-ready? (car ports))))
-     noop
+     void
      (lambda ()
        (let loop ()
 	 (if (null? ports)
@@ -160,32 +186,26 @@
 ;;; Extended string-port operations:
   
 (define call-with-input-string 
-  (let ([open-input-string open-input-string])
-    (lambda (str proc)
-      (let ((in (open-input-string str)))
-	(proc in) ) ) ) )
+  (lambda (str proc)
+    (let ((in (open-input-string str)))
+      (proc in) ) ) )
 
 (define call-with-output-string
-  (let ((open-output-string open-output-string)
-	(get-output-string get-output-string) )
-    (lambda (proc)
-      (let ((out (open-output-string)))
-	(proc out)
-	(get-output-string out) ) ) ) )
+  (lambda (proc)
+    (let ((out (open-output-string)))
+      (proc out)
+      (get-output-string out) ) ) )
 
 (define with-input-from-string
-  (let ((open-input-string open-input-string))
-    (lambda (str thunk)
-      (fluid-let ([##sys#standard-input (open-input-string str)])
-	(thunk) ) ) ) )
+  (lambda (str thunk)
+    (fluid-let ([##sys#standard-input (open-input-string str)])
+      (thunk) ) ) )
 
 (define with-output-to-string
-  (let ([open-output-string open-output-string]
-	[get-output-string get-output-string] )
-    (lambda (thunk)
-      (fluid-let ([##sys#standard-output (open-output-string)])
-	(thunk) 
-	(get-output-string ##sys#standard-output) ) ) ) )
+  (lambda (thunk)
+    (fluid-let ([##sys#standard-output (open-output-string)])
+      (thunk) 
+      (get-output-string ##sys#standard-output) ) ) )
 
 
 ;;; Custom ports:
