@@ -65,22 +65,13 @@
   (unit posix)
   (uses scheduler regex extras utils files ports)
   (disable-interrupts)
-  (usual-integrations)
-  (hide ##sys#stat posix-error
-	$quote-args-list $exec-setup $exec-teardown
-	check-time-vector)
+  (hide $quote-args-list $exec-setup $exec-teardown)
   (not inline ##sys#interrupt-hook ##sys#user-interrupt-hook)
   (foreign-declare #<<EOF
 #ifndef WIN32_LEAN_AND_MEAN
 # define WIN32_LEAN_AND_MEAN
 #endif
 
-/*
-MinGW should have winsock2.h and ws2tcpip.h as well.
-The CMake build will set HAVE_WINSOCK2_H and HAVE_WS2TCPIP_H.
-However, the _MSC_VER test is still needed for vcbuild.bat.
-./configure doesn't test for these.  It should, for MinGW.
-*/
 #if (_MSC_VER > 1300) || (defined(HAVE_WINSOCK2_H) && defined(HAVE_WS2TCPIP_H))
 # include <winsock2.h>
 # include <ws2tcpip.h>
@@ -93,16 +84,9 @@ However, the _MSC_VER test is still needed for vcbuild.bat.
 #include <io.h>
 #include <stdio.h>
 #include <process.h>
-
-static int C_not_implemented(void);
-int C_not_implemented() { return -1; }
-
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <direct.h>
 #include <utime.h>
-
 #include <time.h>
 
 #define ARG_MAX		256
@@ -117,7 +101,6 @@ static C_TLS struct group *C_group;
 static C_TLS int C_pipefds[ 2 ];
 static C_TLS time_t C_secs;
 static C_TLS struct tm C_tm;
-static C_TLS struct stat C_statbuf;
 
 /* pipe handles */
 static C_TLS HANDLE C_rd0, C_wr0, C_wr0_, C_rd1, C_wr1, C_rd1_;
@@ -233,8 +216,6 @@ readdir(DIR * dir)
 #define C_readdir(h,e)		C_set_block_item(e, 0, (C_word) readdir((DIR *)C_block_item(h, 0)))
 #define C_foundfile(e,b)	(strcpy(C_c_string(b), ((struct dirent *) C_block_item(e, 0))->d_name), C_fix(strlen(((struct dirent *) C_block_item(e, 0))->d_name)))
 
-#define C_curdir(buf)	    (getcwd(C_c_string(buf), 256) ? C_fix(strlen(C_c_string(buf))) : C_SCHEME_FALSE)
-
 #define open_binary_input_pipe(a, n, name)   C_mpointer(a, _popen(C_c_string(name), "r"))
 #define open_text_input_pipe(a, n, name)     open_binary_input_pipe(a, n, name)
 #define open_binary_output_pipe(a, n, name)  C_mpointer(a, _popen(C_c_string(name), "w"))
@@ -257,8 +238,7 @@ readdir(DIR * dir)
 #define C_getenventry(i)   environ[ i ]
 
 #define C_putenv(s)	    C_fix(putenv((char *)C_data_pointer(s)))
-#define C_stat(fn)	    C_fix(stat((char *)C_data_pointer(fn), &C_statbuf))
-#define C_fstat(f)	    C_fix(fstat(C_unfix(f), &C_statbuf))
+#define C_lstat(fn)	    C_stat(fn)
 
 static C_word C_fcall
 C_setenv(C_word x, C_word y)
@@ -923,29 +903,10 @@ static int set_file_mtime(char *filename, C_word tm)
 EOF
 ) )
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (no-bound-checks)
-    (no-procedure-checks-for-usual-bindings)
-    (bound-to-procedure
-     ##sys#make-port ##sys#file-info ##sys#update-errno ##sys#fudge ##sys#make-c-string ##sys#check-port
-     ##sys#error ##sys#signal-hook ##sys#peek-unsigned-integer ##sys#process
-     ##sys#peek-fixnum ##sys#make-structure ##sys#check-structure ##sys#enable-interrupts) ) ] )
 
-(include "unsafe-declarations.scm")
+;;; common code
 
-(register-feature! 'posix)
-
-(define posix-error
-  (let ([strerror (foreign-lambda c-string "strerror" int)]
-	[string-append string-append] )
-    (lambda (type loc msg . args)
-      (let ([rn (##sys#update-errno)])
-	(apply ##sys#signal-hook type loc (string-append msg " - " (strerror rn)) args) ) ) ) )
-
-(define ##sys#posix-error posix-error)
+(include "posix-common.scm")
 
 
 ;;; Lo-level I/O:
@@ -1011,7 +972,7 @@ EOF
 	(##sys#check-string filename 'file-open)
 	(##sys#check-exact flags 'file-open)
 	(##sys#check-exact mode 'file-open)
-	(let ([fd (##core#inline "C_open" (##sys#make-c-string (##sys#expand-home-path filename)) flags mode)])
+	(let ([fd (##core#inline "C_open" (##sys#make-c-string (##sys#expand-home-path filename) 'file-open) flags mode)])
 	  (when (eq? -1 fd)
 	    (##sys#update-errno)
 	    (##sys#signal-hook #:file-error 'file-open "cannot open file" filename flags mode) )
@@ -1055,7 +1016,7 @@ EOF
   (let ([string-length string-length])
     (lambda (template)
       (##sys#check-string template 'file-mkstemp)
-      (let* ([buf (##sys#make-c-string template)]
+      (let* ([buf (##sys#make-c-string template 'file-mkstemp)]
 	     [fd (##core#inline "C_mkstemp" buf)]
 	     [path-length (string-length buf)])
 	(when (eq? -1 fd)
@@ -1074,71 +1035,19 @@ EOF
 (define seek/end _seek_end)
 (define seek/cur _seek_cur)
 
-(define-foreign-variable _stat_st_ino unsigned-int "C_statbuf.st_ino")
-(define-foreign-variable _stat_st_nlink unsigned-int "C_statbuf.st_nlink")
-(define-foreign-variable _stat_st_gid unsigned-int "C_statbuf.st_gid")
-(define-foreign-variable _stat_st_size unsigned-int "C_statbuf.st_size")
-(define-foreign-variable _stat_st_mtime double "C_statbuf.st_mtime")
-(define-foreign-variable _stat_st_atime double "C_statbuf.st_atime")
-(define-foreign-variable _stat_st_ctime double "C_statbuf.st_ctime")
-(define-foreign-variable _stat_st_uid unsigned-int "C_statbuf.st_uid")
-(define-foreign-variable _stat_st_mode unsigned-int "C_statbuf.st_mode")
-
-(define (##sys#stat file)
-  (let ([r (cond [(fixnum? file) (##core#inline "C_fstat" file)]
-		 [(string? file) (##core#inline "C_stat" (##sys#make-c-string (##sys#expand-home-path file)))]
-		 [else (##sys#signal-hook #:type-error "bad argument type - not a fixnum or string" file)] ) ] )
-    (when (fx< r 0)
-      (##sys#update-errno)
-      (##sys#signal-hook #:file-error "cannot access file" file) ) ) )
-
-(define (file-stat f #!optional link)
-  (##sys#stat f)
-  (vector _stat_st_ino _stat_st_mode _stat_st_nlink
-	  _stat_st_uid _stat_st_gid _stat_st_size
-	  _stat_st_atime _stat_st_ctime _stat_st_mtime
-	  0 0 0 0) )
-
-(define (file-size f) (##sys#stat f) _stat_st_size)
-
-(define file-modification-time
-  (getter-with-setter 
-   (lambda (f)
-     (##sys#stat f) _stat_st_mtime)
-   (lambda (f t)
-     (##sys#check-string f 'set-file-modification-time)
-     (##sys#check-number t 'set-file-modification-time)
-     (let ((r ((foreign-lambda int "set_file_mtime" c-string scheme-object)
-	       (##sys#expand-home-path f) 
-	       t)))
-       (when (fx< r 0)
-	 (posix-error 
-	  #:file-error 'set-file-modification-time
-	  "cannot set file modification-time" f t))))))
-
-(define (file-access-time f) (##sys#stat f) _stat_st_atime)
-(define (file-change-time f) (##sys#stat f) _stat_st_ctime)
-(define (file-owner f) (##sys#stat f) _stat_st_uid)
-(define (file-permissions f) (##sys#stat f) _stat_st_mode)
-
-(define (regular-file? fname)
-  (##sys#check-string fname 'regular-file?)
-  (let ((info (##sys#file-info (##sys#expand-home-path fname))))
-    (and info (fx= 0 (##sys#slot info 4))) ) )
-
 (define (symbolic-link? fname)
   (##sys#check-string fname 'symbolic-link?)
   #f)
 
 (let ((stat-type
-         (lambda (name)
-             (lambda (fname)
-                 (##sys#check-string fname name)
-                 #f))))
-    (set! character-device? (stat-type 'character-device?))
-    (set! block-device? (stat-type 'block-device?))
-    (set! fifo? (stat-type 'fifo?))
-    (set! socket? (stat-type 'socket?)))
+       (lambda (name)
+	 (lambda (fname)
+	   (##sys#check-string fname name)
+	   #f))))
+  (set! character-device? (stat-type 'character-device?))
+  (set! block-device? (stat-type 'block-device?))
+  (set! fifo? (stat-type 'fifo?))
+  (set! socket? (stat-type 'socket?)))
 
 (define set-file-position!
    (lambda (port pos . whence)
@@ -1170,16 +1079,17 @@ EOF
        (when (< pos 0)
 	 (posix-error #:file-error 'file-position "cannot retrieve file position of port" port) )
        pos) )
-   set-file-position!) )		; doesn't accept WHENCE
+   set-file-position!
+   "(file-position port)") )		; doesn't accept WHENCE
 
 
 ;;; Directory stuff:
 
 (define-inline (create-directory-helper name)
-    (unless (fx= 0 (##core#inline "C_mkdir" (##sys#make-c-string name)))
-            (##sys#update-errno)
-            (##sys#signal-hook #:file-error 'create-directory
-                               "cannot create directory" name)))
+  (unless (fx= 0 (##core#inline "C_mkdir" (##sys#make-c-string name 'create-directory)))
+    (##sys#update-errno)
+    (##sys#signal-hook #:file-error 'create-directory
+		       "cannot create directory" name)))
 
 (define-inline (create-directory-check name)
     (if (file-exists? name)
@@ -1213,146 +1123,13 @@ EOF
 (define change-directory
   (lambda (name)
     (##sys#check-string name 'change-directory)
-    (let ((sname (##sys#make-c-string (##sys#expand-home-path name))))
+    (let ((sname (##sys#make-c-string (##sys#expand-home-path name) 'change-directory)))
       (unless (fx= 0 (##core#inline "C_chdir" sname))
 	(##sys#update-errno)
 	(##sys#signal-hook
 	 #:file-error 'change-directory "cannot change current directory" name) )
       name)))
 
-(define delete-directory
-  (lambda (name)
-    (##sys#check-string name 'delete-directory)
-    (let ((sname (##sys#make-c-string (##sys#expand-home-path name))))
-      (unless (fx= 0 (##core#inline "C_rmdir" sname))
-	(##sys#update-errno)
-	(##sys#signal-hook #:file-error 'delete-directory "cannot delete directory" name) )
-      name)))
-
-(define directory
-  (let ([string-append string-append]
-	[make-string make-string]
-	[string string])
-    (lambda (#!optional (spec (current-directory)) show-dotfiles?)
-      (##sys#check-string spec 'directory)
-      (let ([buffer (make-string 256)]
-	    [handle (##sys#make-pointer)]
-	    [entry (##sys#make-pointer)] )
-	(##core#inline "C_opendir" (##sys#make-c-string (##sys#expand-home-path spec)) handle)
-	(if (##sys#null-pointer? handle)
-	    (begin
-	      (##sys#update-errno)
-	      (##sys#signal-hook #:file-error 'directory "cannot open directory" spec) )
-	    (let loop ()
-	      (##core#inline "C_readdir" handle entry)
-	      (if (##sys#null-pointer? entry)
-		  (begin
-		    (##core#inline "C_closedir" handle)
-		    '() )
-		  (let* ([flen (##core#inline "C_foundfile" entry buffer)]
-			 [file (##sys#substring buffer 0 flen)]
-			 [char1 (string-ref file 0)]
-			 [char2 (and (> flen 1) (string-ref file 1))] )
-		    (if (and (eq? char1 #\.)
-			     (or (not char2)
-				 (and (eq? char2 #\.) (eq? flen 2))
-				 (not show-dotfiles?) ) )
-			(loop)
-			(cons file (loop)) ) ) ) ) ) ) ) ) )
-
-(define (directory? fname)
-  (##sys#check-string fname 'directory?)
-  (let ((info (##sys#file-info
-		(##sys#platform-fixup-pathname (##sys#expand-home-path fname)))))
-    (and info (fx= 1 (##sys#slot info 4))) ) )
-
-(define current-directory
-  (let ([make-string make-string])
-    (lambda (#!optional dir)
-      (if dir
-	  (change-directory dir)
-	  (let* ([buffer (make-string 256)]
-		 [len (##core#inline "C_curdir" buffer)] )
-	    (##sys#update-errno)
-	    (if len
-		(##sys#substring buffer 0 len)
-		(##sys#signal-hook #:file-error 'current-directory "cannot retrieve current directory") ) ) ) ) ) )
-
-
-(define canonical-path                                  ;;DEPRECATED
-    (let ((null?      null?)
-          (char=?     char=?)
-          (string=?   string=?)
-          (alpha?     char-alphabetic?)
-          (sref       string-ref)
-          (ssplit     (cut string-split <> "/\\"))
-          (sappend    string-append)
-          (isperse    (cut string-intersperse <> "\\"))
-          (sep?       (lambda (c) (or (char=? #\/ c) (char=? #\\ c))))
-          (user       current-user-name)
-          (cwd        (let ((cw   current-directory))
-                          (lambda ()
-                              (condition-case (cw)
-                                  (var ()    "c:\\"))))))
-        (lambda (path)
-            (##sys#check-string path 'canonical-path)
-            (let ((p   (cond ((fx= 0 (##sys#size path))
-                                 (sappend (cwd) "\\"))
-                             ((and (fx< (##sys#size path) 3)
-                                   (sep? (sref path 0)))
-                                 (sappend
-                                     (##sys#substring (cwd) 0 2)
-                                     path))
-                             ((fx= 1 (##sys#size path))
-                                 (sappend (cwd) "\\" path))
-                             ((and (char=? #\~ (sref path 0))
-                                   (sep? (sref path 1)))
-                                 (sappend
-                                     (##sys#substring (cwd) 0 3)
-                                     "Documents and Settings\\"
-                                     (user)
-                                     (##sys#substring path 1
-                                         (##sys#size path))))
-                             ((fx= 2 (##sys#size path))
-                                 (sappend (cwd) "\\" path))
-                             ((and (alpha? (sref path 0))
-                                   (char=? #\: (sref path 1))
-                                   (sep? (sref path 2)))
-                                 path)
-                             ((and (char=? #\/ (sref path 0))
-                                   (alpha? (sref path 1))
-                                   (char=? #\: (sref path 2)))
-                                 (sappend
-                                     (##sys#substring path 1 3)
-                                     "\\"
-                                     (##sys#substring path 3
-                                         (##sys#size path))))
-                             ((sep? (sref path 0))
-                                 (sappend
-                                     (##sys#substring (cwd) 0 2)
-                                     path))
-                             (else
-                                 (sappend (cwd) "\\" path)))))
-                (let loop ((l   (ssplit (##sys#substring p 3 (##sys#size p))))
-                           (r   '()))
-                    (if (null? l)
-                        (if (null? r)
-                            (##sys#substring p 0 3)
-                            (if (sep? (sref p (- (##sys#size p) 1)))
-                                (sappend
-                                    (##sys#substring p 0 3)
-                                    (isperse (reverse (cons "" r))))
-                                (sappend
-                                    (##sys#substring p 0 3)
-                                    (isperse (reverse r)))))
-                        (loop
-                            (cdr l)
-                            (if (string=? ".." (car l))
-                                (cdr r)
-                                (if (string=? "." (car l))
-                                    r
-                                    (cons (car l) r))))))))))
-                           
 
 ;;; Pipes:
 
@@ -1373,8 +1150,8 @@ EOF
 	(check
 	 cmd #t
 	 (case m
-	   ((###text) (##core#inline_allocate ("open_text_input_pipe" 2) (##sys#make-c-string cmd)))
-	   ((###binary) (##core#inline_allocate ("open_binary_input_pipe" 2) (##sys#make-c-string cmd)))
+	   ((###text) (##core#inline_allocate ("open_text_input_pipe" 2) (##sys#make-c-string cmd 'open-input-pipe)))
+	   ((###binary) (##core#inline_allocate ("open_binary_input_pipe" 2) (##sys#make-c-string cmd 'open-input-pipe)))
 	   (else (badmode m)) ) ) ) ) )
   (set! open-output-pipe
     (lambda (cmd . m)
@@ -1383,8 +1160,8 @@ EOF
 	(check
 	 cmd #f
 	 (case m
-	   ((###text) (##core#inline_allocate ("open_text_output_pipe" 2) (##sys#make-c-string cmd)))
-	   ((###binary) (##core#inline_allocate ("open_binary_output_pipe" 2) (##sys#make-c-string cmd)))
+	   ((###text) (##core#inline_allocate ("open_text_output_pipe" 2) (##sys#make-c-string cmd 'open-output-pipe)))
+	   ((###binary) (##core#inline_allocate ("open_binary_output_pipe" 2) (##sys#make-c-string cmd 'open-output-pipe)))
 	   (else (badmode m)) ) ) ) ) )
   (set! close-input-pipe
     (lambda (port)
@@ -1599,7 +1376,7 @@ EOF
   (lambda (fname m)
     (##sys#check-string fname 'change-file-mode)
     (##sys#check-exact m 'change-file-mode)
-    (when (fx< (##core#inline "C_chmod" (##sys#make-c-string (##sys#expand-home-path fname)) m) 0)
+    (when (fx< (##core#inline "C_chmod" (##sys#make-c-string (##sys#expand-home-path fname) 'change-file-mode) m) 0)
       (##sys#update-errno)
       (##sys#signal-hook #:file-error 'change-file-mode "cannot change file mode" fname m) ) ) )
 
@@ -1610,7 +1387,7 @@ EOF
 (let ()
   (define (check filename acc loc)
     (##sys#check-string filename loc)
-    (let ([r (fx= 0 (##core#inline "C_test_access" (##sys#make-c-string (##sys#expand-home-path filename)) acc))])
+    (let ([r (fx= 0 (##core#inline "C_test_access" (##sys#make-c-string (##sys#expand-home-path filename) loc) acc))])
       (unless r (##sys#update-errno))
       r) )
   (set! file-read-access? (lambda (filename) (check filename _r_ok 'file-read-access?)))
@@ -1630,7 +1407,7 @@ EOF
 (define fileno/stderr _stderr_fileno)
 
 (let ()
-  (define (mode inp m)
+  (define (mode inp m loc)
     (##sys#make-c-string
      (cond [(pair? m)
 	    (let ([m (car m)])
@@ -1638,7 +1415,8 @@ EOF
 		[(###append) (if (not inp) "a" (##sys#error "invalid mode for input file" m))]
 		[else (##sys#error "invalid mode argument" m)] ) ) ]
 	   [inp "r"]
-	   [else "w"] ) ) )
+	   [else "w"] )
+     loc) )
   (define (check fd inp r)
     (##sys#update-errno)
     (if (##sys#null-pointer? r)
@@ -1649,11 +1427,11 @@ EOF
   (set! open-input-file*
     (lambda (fd . m)
       (##sys#check-exact fd 'open-input-file*)
-      (check fd #t (##core#inline_allocate ("C_fdopen" 2) fd (mode #t m))) ) )
+      (check fd #t (##core#inline_allocate ("C_fdopen" 2) fd (mode #t m 'open-input-file*))) ) )
   (set! open-output-file*
     (lambda (fd . m)
       (##sys#check-exact fd 'open-output-file*)
-      (check fd #f (##core#inline_allocate ("C_fdopen" 2) fd (mode #f m)) ) ) ) )
+      (check fd #f (##core#inline_allocate ("C_fdopen" 2) fd (mode #f m 'open-output-file*)) ) ) ) )
 
 (define port->fileno
   (lambda (port)
@@ -1686,12 +1464,12 @@ EOF
   (lambda (var val)
     (##sys#check-string var 'setenv)
     (##sys#check-string val 'setenv)
-    (##core#inline "C_setenv" (##sys#make-c-string var) (##sys#make-c-string val))
+    (##core#inline "C_setenv" (##sys#make-c-string var 'setenv) (##sys#make-c-string val 'setenv))
     (##core#undefined) ) )
 
 (define (unsetenv var)
   (##sys#check-string var 'unsetenv)
-  (##core#inline "C_putenv" (##sys#make-c-string var))
+  (##core#inline "C_putenv" (##sys#make-c-string var 'unsetenv))
   (##core#undefined) )
 
 (define get-environment-variables
@@ -1707,7 +1485,6 @@ EOF
 		    (scan (fx+ j 1)) ) )
 	      '() ) ) ) ) ) )
 
-(define current-environment get-environment-variables) ; DEPRECATED
 
 ;;; Time related things:
 
@@ -1740,7 +1517,7 @@ EOF
       (if fmt
           (begin
             (##sys#check-string fmt 'time->string)
-            (or (strftime tm (##sys#make-c-string fmt))
+            (or (strftime tm (##sys#make-c-string fmt 'time->string))
                 (##sys#error 'time->string "time formatting overflows buffer" tm)) )
           (let ([str (asctime tm)])
             (if str
@@ -1757,7 +1534,7 @@ EOF
 (define local-timezone-abbreviation
   (foreign-lambda* c-string ()
    "char *z = (_daylight ? _tzname[1] : _tzname[0]);\n"
-   "return(z);") )
+   "C_return(z);") )
 
 ;;; Other things:
 
@@ -1797,30 +1574,6 @@ EOF
 		   0)
 	  (##sys#error 'set-buffering-mode! "cannot set buffering mode" port mode size) ) ) ) )
 
-;;; Filename globbing:
-
-(define glob
-  (let ([regexp regexp]
-	[string-match string-match]
-	[glob->regexp glob->regexp]
-	[directory directory]
-	[make-pathname make-pathname]
-	[decompose-pathname decompose-pathname] )
-    (lambda paths
-      (let conc-loop ([paths paths])
-	(if (null? paths)
-	    '()
-	    (let ([path (car paths)])
-	      (let-values ([(dir fil ext) (decompose-pathname path)])
-		(let* ([patt (glob->regexp (make-pathname #f (or fil "*") ext))]
-		       [rx (regexp patt)])
-		  (let loop ([fns (directory (or dir ".") #t)])
-		    (cond [(null? fns) (conc-loop (cdr paths))]
-			  [(string-match rx (car fns))
-			   => (lambda (m) (cons (make-pathname dir (car m)) (loop (cdr fns)))) ]
-			  [else (loop (cdr fns))] ) ) ) ) ) ) ) ) ) )
-
-
 ;;; Process handling:
 
 (define-foreign-variable _p_overlay int "P_OVERLAY")
@@ -1869,7 +1622,6 @@ EOF
 (define $exec-setup
   (let ([setarg (foreign-lambda void "C_set_exec_arg" int scheme-pointer int)]
 	[setenv (foreign-lambda void "C_set_exec_env" int scheme-pointer int)]
-	[pathname-strip-directory pathname-strip-directory]
 	[build-exec-argvec
 	  (lambda (loc lst argvec-setter idx)
 	    (if lst
@@ -1889,7 +1641,7 @@ EOF
       (build-exec-argvec loc (and arglst ($quote-args-list arglst exactf)) setarg 1)
       (build-exec-argvec loc envlst setenv 0)
       (##core#inline "C_flushall")
-      (##sys#make-c-string (##sys#expand-home-path filename)) ) ) )
+      (##sys#make-c-string (##sys#expand-home-path filename) loc) ) ) )
 
 (define ($exec-teardown loc msg filename res)
   (##sys#update-errno)
@@ -1925,13 +1677,11 @@ EOF
   (list "/c" cmdlin) )
 
 (define process-run
-  (let ([process-spawn process-spawn]
-	[get-environment-variable get-environment-variable] )
-    (lambda (f . args)
-      (let ([args (if (pair? args) (car args) #f)])
-	(if args
-	    (process-spawn spawn/nowait f args)
-	    (process-spawn spawn/nowait (##sys#shell-command) (##sys#shell-command-arguments f)) ) ) ) ) )
+  (lambda (f . args)
+    (let ([args (if (pair? args) (car args) #f)])
+      (if args
+	  (process-spawn spawn/nowait f args)
+	  (process-spawn spawn/nowait (##sys#shell-command) (##sys#shell-command-arguments f)) ) ) ) )
 
 ;;; Run subprocess connected with pipes:
 (define-foreign-variable _rdbuf char "C_rdbuf")
@@ -2059,47 +1809,6 @@ EOF
 	(##sys#error 'current-user-name "cannot retrieve current user-name") ) ) )
 
 
-;;; Find matching files:
-
-(define find-files
-  (let ([glob glob]
-	[string-match string-match]
-	[make-pathname make-pathname]
-	[pathname-file pathname-file]
-	[directory? directory?] )
-    (lambda (dir pred . action-id-limit)
-      (let-optionals
-	  action-id-limit
-	  ([action (lambda (x y) (cons x y))] ; we want cons inlined
-	   [id '()]
-	   [limit #f] )
-	(##sys#check-string dir 'find-files)
-	(let* ([depth 0]
-	       [lproc
-		(cond [(not limit) (lambda _ #t)]
-		      [(fixnum? limit) (lambda _ (fx< depth limit))]
-		      [else limit] ) ]
-	       [pproc
-		(if (or (string? pred) (regexp? pred))
-		    (lambda (x) (string-match pred x))
-		    pred) ] )
-	  (let loop ([fs (glob (make-pathname dir "*"))]
-		     [r id] )
-	    (if (null? fs)
-		r
-		(let ([f (##sys#slot fs 0)]
-		      [rest (##sys#slot fs 1)] )
-		  (cond [(directory? f)
-			 (cond [(member (pathname-file f) '("." "..")) (loop rest r)]
-			       [(lproc f)
-				(loop rest
-				      (fluid-let ([depth (fx+ depth 1)])
-					(loop (glob (make-pathname f "*"))
-					      (if (pproc f) (action f r) r)) ) ) ]
-			       [else (loop rest (if (pproc f) (action f r) r))] ) ]
-			[(pproc f) (loop rest (action f r))]
-			[else (loop rest r)] ) ) ) ) ) ) ) ) )
-
 ;;; unimplemented stuff:
 
 (define-syntax define-unimplemented
