@@ -25,14 +25,10 @@
 ; POSSIBILITY OF SUCH DAMAGE.
 
 
-(cond-expand
- [chicken-compile-shared]
- [else (declare (unit regex))] )
+(declare (unit regex))
 
 (declare
-  (usual-integrations)
   (disable-interrupts)
-;  (disable-warning var)
   (fixnum)
   (export
     regexp? regexp
@@ -53,14 +49,7 @@
     irregex-nfa irregex-flags irregex-submatches irregex-lengths irregex-names
     ))
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (no-bound-checks)
-    (no-procedure-checks) ) ] )
-
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'regex 'irregex)
 
@@ -71,6 +60,44 @@
 
 (define-record regexp x)
 
+(define-syntax (build-cache x r c)
+  ;; (build-cache N ARG FAIL) 
+  (let* ((n (cadr x))
+	 (n2 (* n 2))
+	 (arg (caddr x))
+	 (fail (cadddr x))
+	 (%cache (r 'cache))
+	 (%index (r 'index))
+	 (%arg (r 'arg))
+	 (%let (r 'let))
+	 (%let* (r 'let*))
+	 (%if (r 'if))
+	 (%fx+ (r 'fx+))
+	 (%fxmod (r 'fxmod))
+	 (%equal? (r 'equal?))
+	 (%quote (r 'quote))
+	 (%tmp (r 'tmp))
+	 (%begin (r 'begin))
+	 (cache (make-vector (add1 n2) #f)))
+    (vector-set! cache n2 0)		; last slot: current index
+    `(,%let* ((,%cache (,%quote ,cache))
+	      (,%arg ,arg))
+	     ,(let fold ((i 0))
+		(if (>= i n)
+		    ;; this should be thread-safe: a context-switch can only
+		    ;; happen before this code and in the call to FAIL.
+		    `(,%let ((,%tmp ,fail)
+			     (,%index (##sys#slot ,%cache ,n2)))
+			    (##sys#setslot ,%cache ,%index ,%arg)
+			    (##sys#setslot ,%cache (,%fx+ ,%index 1) ,%tmp)
+			    (##sys#setislot 
+			     ,%cache ,n2
+			     (##core#inline "C_u_fixnum_modulo" (,%fx+ ,%index 2) ,n2))
+			    ,%tmp)
+		    `(,%if (,%equal? (##sys#slot ,%cache ,(* i 2)) ,%arg)
+			   (##sys#slot ,%cache ,(add1 (* i 2)))
+			   ,(fold (add1 i))))))))
+
 (define (regexp pat #!optional caseless extended utf8)
   (if (regexp? pat)
       pat
@@ -87,7 +114,10 @@
 (define (unregexp x)
   (cond ((regexp? x) (regexp-x x))
 	((irregex? x) x)
-	(else (irregex x))))
+	(else
+	 (build-cache
+	  5 x
+	  (irregex x)))))
 
 
 ;;; Basic `regexp' operations
@@ -264,7 +294,8 @@
 			  (rest (cdr cs)) )
 		      (cond ((char=? c #\*) 
 			     (if dir
-				 `((or (: (~ ("./\\")) (* (~ ("/\\"))))
+				 `((or (: (~ ("./\\"))
+					  (* (~ ("/\\"))))
 				       (* (~ ("./\\"))))
 				   ,@(loop rest #f))
 				 `((* (~ ("/\\"))) ,@(loop rest #f))))
