@@ -26,18 +26,16 @@
 
 
 (declare
-  (fixnum)
   (unit scheduler)
   (disable-interrupts)
-  (usual-integrations)
-  (disable-warning var)
   (hide ##sys#ready-queue-head ##sys#ready-queue-tail ##sys#timeout-list
 	##sys#update-thread-state-buffer ##sys#restore-thread-state-buffer
 	##sys#remove-from-ready-queue ##sys#unblock-threads-for-i/o ##sys#force-primordial
 	##sys#fdset-input-set ##sys#fdset-output-set ##sys#fdset-clear
 	##sys#fdset-select-timeout ##sys#fdset-restore
-	##sys#clear-i/o-state-for-thread!) 
+	##sys#clear-i/o-state-for-thread! ##sys#abandon-mutexes) 
   (not inline ##sys#interrupt-hook)
+  (unsafe)
   (foreign-declare #<<EOF
 #ifdef HAVE_ERRNO_H
 # include <errno.h>
@@ -81,17 +79,12 @@ static fd_set C_fdset_input, C_fdset_output, C_fdset_input_2, C_fdset_output_2;
 EOF
 ) )
 
-(cond-expand
- [paranoia]
- [else
-  (declare (unsafe)) ] )
-
+(include "common-declarations.scm")
 
 (define-syntax dbg
   (syntax-rules ()
     ((_ . _) #f))) 
 
-
 (define (##sys#schedule)
   (define (switch thread)
     (dbg "switching to " thread)
@@ -112,17 +105,17 @@ EOF
     (let loop1 ()
       ;; Unblock threads waiting for timeout:
       (unless (null? ##sys#timeout-list)
-	(let ([now (##sys#fudge 16)])
+	(let ((now (##core#inline_allocate ("C_a_i_current_milliseconds" 4) #f)))
 	  (dbg "timeout (" now ") list: " ##sys#timeout-list)
-	  (let loop ([lst ##sys#timeout-list])
+	  (let loop ((lst ##sys#timeout-list))
 	    (if (null? lst)
 		(set! ##sys#timeout-list '())
-		(let* ([tmo1 (caar lst)]
-		       [tto (cdar lst)]
-		       [tmo2 (##sys#slot tto 4)] )
+		(let* ([tmo1 (caar lst)] ; timeout of thread on list
+		       [tto (cdar lst)]	 ; thread on list
+		       [tmo2 (##sys#slot tto 4)] ) ; timeout value stored in thread
 		  (dbg "  " tto " -> " tmo2)
-		  (if (eq? tmo1 tmo2)
-		      (if (>= now tmo1)
+		  (if (equal? tmo1 tmo2)  ;XXX why do we check this?
+		      (if (fp>= now tmo1) ; timeout reached?
 			  (begin
 			    (##sys#setislot tto 13 #t) ; mark as being unblocked by timeout
 			    (##sys#clear-i/o-state-for-thread! tto)
@@ -137,10 +130,15 @@ EOF
 			    (when (and (null? ##sys#ready-queue-head)
 				       (null? ##sys#fd-list) 
 				       (pair? ##sys#timeout-list))
-			      (let ([tmo1 (caar ##sys#timeout-list)])
+			      (let ((tmo1 (caar ##sys#timeout-list)))
 				(set! eintr
-				  (and (not (##core#inline "C_msleep" (fxmax 0 (- tmo1 now))))
-				       (foreign-value "C_signal_interrupted_p" bool) ) ) ) ) ) )
+				  (and (not (##core#inline 
+					     "C_msleep" 
+					     (fxmax 
+					      0
+					      (##sys#inexact->exact (fp- tmo1 now)))))
+				       (foreign-value
+					"C_signal_interrupted_p" bool) ) ) ) ) ) )
 		      (loop (cdr lst)) ) ) ) ) ) )
       ;; Unblock threads blocked by I/O:
       (if eintr
@@ -228,14 +226,14 @@ EOF
   (dbg t " blocks for " tm)
   ;; This should really use a balanced tree:
   (let loop ([tl ##sys#timeout-list] [prev #f])
-    (if (or (null? tl) (< tm (caar tl)))
+    (if (or (null? tl) (fp< tm (caar tl)))
 	(if prev
 	    (set-cdr! prev (cons (cons tm t) tl))
 	    (set! ##sys#timeout-list (cons (cons tm t) tl)) )
 	(loop (cdr tl) tl) ) ) 
   (##sys#setslot t 3 'blocked)
   (##sys#setislot t 13 #f)
-  (##sys#setislot t 4 tm) )
+  (##sys#setslot t 4 tm) )
 
 (define (##sys#thread-block-for-termination! t t2)
   (dbg t " blocks for " t2)
@@ -246,15 +244,40 @@ EOF
       (##sys#setislot t 13 #f)
       (##sys#setslot t 11 t2) ) ) )
 
+(define (##sys#abandon-mutexes thread)
+  (let ((ms (##sys#slot thread 8)))
+    (unless (null? ms)
+      (##sys#for-each
+       (lambda (m)
+	 (##sys#setislot m 2 #f)
+	 (##sys#setislot m 4 #t) 
+	 (##sys#setislot m 5 #f)
+	 (let ((wts (##sys#slot m 3)))
+	   (unless (null? wts)
+	     (for-each
+	      (lambda (t2)
+		(dbg "  unblocking: " t2)
+		(##sys#thread-basic-unblock! t2) )
+	      wts) ) )
+	 (##sys#setislot m 3 '()) )
+       ms) ) ) )
+
 (define (##sys#thread-kill! t s)
   (dbg "killing: " t " -> " s ", recipients: " (##sys#slot t 12))
   (##sys#abandon-mutexes t)
+  (let ((blocked (##sys#slot t 11)))
+    (cond
+     ((##sys#structure? blocked 'condition-variable)
+      (##sys#setslot blocked 2 (##sys#delq thread (##sys#slot blocked 2))))
+     ((##sys#structure? blocked 'thread)
+      (##sys#setslot blocked 12 (##sys#delq thread (##sys#slot blocked 12))))) )
+  (##sys#remove-from-timeout-list t)
+  (##sys#clear-i/o-state-for-thread! t)
   (##sys#setslot t 3 s)
   (##sys#setislot t 4 #f)
   (##sys#setislot t 11 #f)
   (##sys#setislot t 8 '())
-  (##sys#remove-from-timeout-list t)
-  (let ([rs (##sys#slot t 12)])
+  (let ((rs (##sys#slot t 12)))
     (unless (null? rs)
       (for-each
        (lambda (t2)
@@ -278,7 +301,8 @@ EOF
 	[get-output-string get-output-string] )
     (lambda (arg)
       (let ([ct ##sys#current-thread])
-	(dbg "exception: " ct " -> " (if (##sys#structure? arg 'condition) (##sys#slot arg 2) arg))
+	(dbg "exception: " ct " -> " 
+	     (if (##sys#structure? arg 'condition) (##sys#slot arg 2) arg))
 	(cond [(foreign-value "C_abort_on_thread_exceptions" bool)
 	       (let* ([pt ##sys#primordial-thread]
 		      [ptx (##sys#slot pt 1)] )
@@ -292,7 +316,7 @@ EOF
 	       (let ([o (open-output-string)])
 		 (display "Warning (" o)
 		 (display ct o)
-		 (display "): " o)
+		 (display ")" o)
 		 (print-error-message arg ##sys#standard-error (get-output-string o))
 		 (print-call-chain ##sys#standard-error 0 ct) ) ] )
 	(##sys#setslot ct 7 arg)
@@ -305,13 +329,13 @@ EOF
 (define ##sys#fd-list '())
 
 (define ##sys#fdset-select-timeout
-  (foreign-lambda* int ([bool to] [unsigned-long tm])
+  (foreign-lambda* int ([bool to] [double tm])
     "struct timeval timeout;"
     "timeout.tv_sec = tm / 1000;"
-    "timeout.tv_usec = (tm % 1000) * 1000;"
+    "timeout.tv_usec = fmod(tm, 1000) * 1000;"
     "C_fdset_input_2 = C_fdset_input;"
     "C_fdset_output_2 = C_fdset_output;"
-    "return(select(FD_SETSIZE, &C_fdset_input, &C_fdset_output, NULL, to ? &timeout : NULL));") )
+    "C_return(select(FD_SETSIZE, &C_fdset_input, &C_fdset_output, NULL, to ? &timeout : NULL));") )
 
 (define ##sys#fdset-restore
   (foreign-lambda* void ()
@@ -361,10 +385,10 @@ EOF
 	 [n (##sys#fdset-select-timeout	; we use FD_SETSIZE, but really should use max fd
 	     (or rq? to?)
 	     (if (and to? (not rq?))	; no thread was unblocked by timeout, so wait
-		 (let* ([tmo1 (caar ##sys#timeout-list)]
-			[now (##sys#fudge 16)])
-		   (fxmax 0 (- tmo1 now)) )
-		 0) ) ] )		; otherwise immediate timeout.
+		 (let* ((tmo1 (caar ##sys#timeout-list))
+			(now (##core#inline_allocate ("C_a_i_current_milliseconds" 4) #f)))
+		   (fpmax 0.0 (fp- tmo1 now)) )
+		 0.0) ) ] )		; otherwise immediate timeout.
     (dbg n " fds ready")
     (cond [(eq? -1 n) 
 	   (##sys#force-primordial)]
@@ -466,7 +490,8 @@ EOF
 ;;; Unblock thread cleanly:
 
 (define (##sys#thread-unblock! t)
-  (when (eq? 'blocked (##sys#slot t 3))
+  (when (or (eq? 'blocked (##sys#slot t 3))
+	    (eq? 'sleeping (##sys#slot r 3)))
     (##sys#remove-from-timeout-list t)
     (set! ##sys#fd-list 
       (let loop ([fdl ##sys#fd-list])
@@ -477,5 +502,4 @@ EOF
 	       (cons (##sys#slot a 0)
 		     (##sys#delq t (##sys#slot a 1)) )
 	       (loop (##sys#slot fdl 1)) ) ) ) ) )
-    (##sys#setislot t 12 '())
     (##sys#thread-basic-unblock! t) ) )
