@@ -45,7 +45,7 @@
 ;   SPEC = * | (VAL1 ...)
 ;   VAL = (or VAL1 ...)
 ;       | (struct NAME)
-;       | (procedure (VAL1 ... [#!optional VALOPT1 ...] [#!rest [VAL | values]]) . RESULTS)
+;       | (procedure [NAME] (VAL1 ... [#!optional VALOPT1 ...] [#!rest [VAL | values]]) . RESULTS)
 ;       | BASIC
 ;       | deprecated
 ;   BASIC = * | string | symbol | char | number | boolean | list | pair | 
@@ -89,8 +89,15 @@
 		   ((eq? a 'deprecated)
 		    (report
 		     loc
-		     (sprintf "use of deprecated toplevel identifier `~a'" id) )
+		     (sprintf "use of deprecated library procedure `~a'" id) )
 		    '*)
+		   ((and (pair? a) (eq? (car a) 'deprecated))
+		    (report 
+		     loc
+		     (sprintf 
+			 "use of deprecated library procedure `~a' - consider using `~a' instead"
+		       id (cadr a)))
+		     '*)
 		   (else (list a)))))
 	  (else '*)))
   (define (variable-result id e loc)
@@ -386,10 +393,8 @@
 			   what n (multiples n)))
 		 (first tv))))))
   (define (report loc desc)
-    (compiler-warning
-     'scrutiny
-     "~a~a" 
-     (location-name loc) desc))
+    (warning
+     (conc (location-name loc) desc)))
   (define (location-name loc)
     (define (lname loc1)
       (if loc1
@@ -471,7 +476,8 @@
 		      (every procedure-type? (cdr t)))))))
   (define (procedure-argument-types t n)
     (cond ((or (memq t '(* procedure)) 
-	       (not-pair? t) )
+	       (not-pair? t)
+	       (eq? 'deprecated (car t)))
 	   (values (make-list n '*) #f))
 	  ((eq? 'procedure (car t))
 	   (let* ((vf #f)
@@ -619,9 +625,9 @@
 	      (old (##sys#get name '##core#type))
 	      (new (cadr e)))
 	 (when (and old (not (equal? old new)))
-	   (compiler-warning 
-	    'scrutiny
-	    "type-definition `~a' for toplevel binding `~a' conflicts with previously loaded type `~a'"
-	    name new old))
+	   (##sys#notice
+	    (sprintf
+		"type-definition `~a' for toplevel binding `~a' conflicts with previously loaded type `~a'"
+		name new old)))
 	 (##sys#put! name '##core#type new)))
      (read-file dbfile))))
