@@ -36,8 +36,7 @@
 
 (module setup-api
 
-    (move-file 
-     (run execute)
+    ((run execute)
      compile
      standard-extension
      make make/proc
@@ -46,24 +45,26 @@
      setup-verbose-mode setup-install-mode deployment-mode
      installation-prefix
      destination-prefix
-     chicken-prefix 			;XXX remove at some stage from exports
+     chicken-prefix
      find-library find-header 
      program-path remove-file* 
-     patch yes-or-no? abort-setup
+     patch abort-setup
      setup-root-directory create-directory/parents
-     test-compile try-compile copy-file run-verbose
-     required-chicken-version required-extension-version cross-chicken
+     test-compile try-compile run-verbose
+     extra-features extra-nonfeatures
+     copy-file move-file
+     required-chicken-version required-extension-version
      sudo-install keep-intermediates
      version>=?
      extension-name-and-version
      extension-name
      extension-version
-     create-temporary-directory
      remove-directory
      remove-extension
      read-info
      register-program find-program
-     shellpath)
+     shellpath
+     setup-error-handling)
   
   (import scheme chicken foreign
 	  regex utils posix ports extras data-structures
@@ -102,17 +103,29 @@
 (define (shellpath str)
   (qs (normalize-pathname str)))
 
-(define (cross-chicken) (##sys#fudge 39)) ; DEPRECATED
-
 (define *csc-options* '())
 (define *base-directory* (current-directory))
 
-(define setup-root-directory      (make-parameter *base-directory*))
-(define setup-verbose-mode        (make-parameter #f))
-(define setup-install-mode        (make-parameter #t))
-(define deployment-mode           (make-parameter #f))
-(define program-path              (make-parameter *chicken-bin-path*))
-(define keep-intermediates (make-parameter #f))
+(define setup-root-directory (make-parameter *base-directory*))
+(define setup-verbose-mode   (make-parameter #f))
+(define setup-install-mode   (make-parameter #t))
+(define deployment-mode      (make-parameter #f))
+(define program-path         (make-parameter *chicken-bin-path*))
+(define keep-intermediates   (make-parameter #f))
+
+(define extra-features
+  (let ((xfs '()))
+    (lambda (#!optional fs)
+      (cond (fs (apply register-feature! fs)
+		(set! xfs fs))
+	    (else xfs)))))
+
+(define extra-nonfeatures
+  (let ((xfs '()))
+    (lambda (#!optional fs)
+      (cond (fs (apply unregister-feature! fs)
+		(set! xfs fs))
+	    (else xfs)))))
 
 ; Setup shell commands
 
@@ -169,21 +182,11 @@
 
 (define abort-setup (make-parameter (cut exit 1)))
 
-(define (yes-or-no? str #!key default (abort (abort-setup)))
-  (let loop ()
-    (printf "~%~A (yes/no/abort) " str)
-    (when default (printf "[~A] " default))
-    (flush-output)
-    (let ((ln (read-line)))
-      (cond ((eof-object? ln) (set! ln "abort"))
-	    ((and default (string=? "" ln)) (set! ln default)) )
-      (cond ((string-ci=? "yes" ln) #t)
-	    ((string-ci=? "no" ln) #f)
-	    ((string-ci=? "abort" ln) (abort))
-	    (else
-	     (printf "~%Please enter \"yes\", \"no\" or \"abort\".~%")
-	     (loop) ) ) ) ) )
-  
+(define-syntax ignore-errors
+  (syntax-rules ()
+    ((_ body ...)
+     (handle-exceptions ex #f body ...))))
+
 (define (patch which rx subst)
   (when (setup-verbose-mode) (printf "patching ~A ...~%" which))
   (if (list? which)
@@ -239,7 +242,14 @@
 	(if (keep-intermediates) "-k" "")
 	(if (host-extension) "-host" "")
 	(if (deployment-mode) "-deployed" "")
-	*csc-options*) 
+	(append
+	 (map (lambda (f)
+		(string-append "-feature " (symbol->string f)))
+	      (extra-features))
+	 (map (lambda (f)
+		(string-append "-no-feature " (symbol->string f)))
+	      (extra-nonfeatures))
+	 *csc-options*) )
        " ")
       (find-program prg)))
 
@@ -314,7 +324,7 @@
 (define (make:check-argv argv)
   (or (string? argv)
       (every string? argv)
-      (error "argument is not a string or string list" argv)))
+      (error "argument-list to `make' is not a string or string list" argv)))
 
 (define (make:make/proc/helper spec argv)
   (when (vector? argv) (set! argv (vector->list argv)))
@@ -341,7 +351,11 @@
 				 (any (lambda (dep)
 					  (let ((dep2 (fixmaketarget dep)))
 					    (unless (file-exists? dep2)
-					      (error (sprintf "dependancy ~a was not made~%" dep2)))
+					      ;;XXX internal error?
+					      (error
+					       (sprintf
+						   "(make) dependency ~a was not made~%"
+						 dep2)))
 					    (and (> (file-modification-time dep2) date)
 						 dep2)) )
 					deps))))
@@ -371,7 +385,7 @@
 				    (signal exn) )
 				((car l))))))))
 		    (unless date
-		      (error (sprintf "don't know how to make ~a" s2))))))))
+		      (error (sprintf "(make) don't know how to make ~a" s2))))))))
     (cond
      ((string? argv) (make-file argv ""))
      ((null? argv) (make-file (caar spec) ""))
@@ -512,24 +526,39 @@
 	   ((equal? "a" ext) (if *windows-shell* "lib" "a"))
 	   (else ext)))))
 
+(define (what-version version)
+  (or version
+      (let ((n+v (extension-name-and-version)))
+	(if (and n+v (pair? n+v) (not (equal? "" (cadr n+v))))
+	    (cadr n+v)
+	    "unknown"))))
+
+(define (supply-version info version)
+  (if (assq 'version info)
+      info
+      (cons `(version ,(what-version version)) info)))
+
 
 ;;; Convenience function
 
 (define (standard-extension name version #!key (static #t) (info '()))
   (let* ((sname (->string name))
 	 (fname (make-pathname #f sname "scm"))
-	 (iname (make-pathname #f sname "import.scm")))
-    (compile -s -O3 -d1 ,fname -j ,name)
+	 (iname (make-pathname #f sname "import.scm"))
+	 (ilname (make-pathname #f sname "inline")))
+    (compile -dynamic -optimize-level 3 -debug-level 1 ,fname -emit-import-library ,name)
     (when static
-      (compile -c -O3 -d1 ,fname -j ,name -unit ,name))
-    (compile -s -O3 -d0 ,iname)
+      (compile -c -optimize-level 3 -debug-level 1 ,fname -emit-import-library
+	       ,name -unit ,name))
+    (compile -dynamic -optimize-level 3 -debug-level 0 ,iname)
     (install-extension
      name
-     (list (pathname-replace-extension fname "so")
-	   (pathname-replace-extension iname "so")
-	   (make-pathname #f sname "setup"))
-     `((version ,version)
-       ,@info
+     `(,(pathname-replace-extension fname "so")
+       ,(pathname-replace-extension iname "so")
+       ,@(if (file-exists? ilname)
+	     (list ilname)
+	     '()))
+     `(,@(supply-version info version)
        ,@(if static `((static ,(make-pathname #f fname "o"))) '())))))
 
 
@@ -554,9 +583,9 @@
 					(equal? (cadr static) from) 
 					(equal? (pathname-extension to) "a"))
 			       (run (,*ranlib-command* ,(shellpath to)) ) ))
-			   (make-dest-pathname rpath f)))
+			   to))
 		       files) ) )
-      (write-info id dests info) ) ) )
+      (write-info id dests (supply-version info #f)) ) ) )
 
 (define (install-program id files #!optional (info '()))
   (define (exify f)
@@ -582,7 +611,7 @@
 				   (run (,*chmod-command* a+r ,(shellpath to))))
 			   to) )
 		       files) ) )
-      (write-info id dests info) ) ) )
+      (write-info id dests (supply-version info #f)) ) ) )
 
 (define (install-script id files #!optional (info '()))
   (when (setup-install-mode)
@@ -599,7 +628,7 @@
 			files) ) )
       (unless *windows-shell*
 	(run (,*chmod-command* a+rx ,(string-intersperse pfiles " "))) )
-      (write-info id pfiles info) ) ) )
+      (write-info id pfiles (supply-version info #f)) ) ) )
 
 
 ;;; More helper stuff
@@ -644,12 +673,12 @@
 			   (if compile-only
 			       "" 
 			       (conc "-L" *target-lib-home* " " ldflags " " *target-libs*) )
-			   " >/dev/null "
+			   (if *windows* " >nul: " " >/dev/null ")
 			   (if verb "" "2>&1") ) ) )
 		 (when verb (print cmd " ..."))
 		 cmd) ) ) ) )
     (when verb (print (if (zero? r) "succeeded." "failed.")))
-    ($system (sprintf "~A ~A" *remove-command* (shellpath fname)))
+    (ignore-errors ($system (sprintf "~A ~A" *remove-command* (shellpath fname))))
     (zero? r) ) )
 
 (define (required-chicken-version v)
@@ -659,7 +688,7 @@
 (define (upgrade-message ext msg #!optional version)
   (error
    (sprintf
-    "the required extension `~s' ~a - please run~%~%  chicken-install ~a~a~%~%and repeat the current installation operation."
+    "the currently installed extension `~s' ~a - please run~%~%  chicken-install ~a~a~%~%and repeat the current installation operation."
     ext msg ext (if version (conc ":" version) "")) ) )
 
 (define (required-extension-version . args)
@@ -676,7 +705,7 @@
 			 ((and (version>=? version ver) (not (string=? (->string version) (->string ver))))
 			  (upgrade-message 
 			   ext
-			   (sprintf "is older than ~a, which is what this extension requires"
+			   (sprintf "is older than ~a, which is the minimum version that this extension requires"
 				    version) 
                            version) )
 			 (else (loop more)) ) ) 
@@ -738,29 +767,18 @@
         (and defver (->string defver))
         ver ) ) )
 
-(define (read-info egg)
+(define (read-info egg #!optional (repo (repository-path)))
   (with-input-from-file 
-      (make-pathname (repository-path) egg ".setup-info")
+      (make-pathname repo egg setup-file-extension)
     read))
 
-(define (create-temporary-directory)
-  (let ((dir (or (get-environment-variable "TMPDIR") 
-		 (get-environment-variable "TEMP")
-		 (get-environment-variable "TMP") 
-		 "/tmp")))
-    (let loop ()
-      (let* ((n (##sys#fudge 16))	; current milliseconds
-	     (pn (make-pathname dir (string-append "chicken-install-" (number->string n 16)) "tmp")))
-	(cond ((file-exists? pn) (loop))
-	      (else (create-directory pn) pn))))))
-
 (define (remove-directory dir #!optional (strict #t))
   (cond ((not (file-exists? dir))
 	 (if strict
 	     (error 'remove-directory "cannot remove - directory not found" dir)
 	     #f))
 	(*sudo*
-	 ($system (sprintf "sudo rm -fr ~a" (shellpath dir))))
+	 (ignore-errors ($system (sprintf "sudo rm -fr ~a" (shellpath dir)))))
 	(else
 	 (let walk ((dir dir))
 	   (let ((files (directory dir #t)))
@@ -774,23 +792,26 @@
 	      files)
 	     (delete-directory dir)))) ))
 
-(define (remove-extension egg)
-  (and-let* ((files (assq 'files (read-info egg))))
+(define (remove-extension egg #!optional (repo (repository-path)))
+  (and-let* ((files (assq 'files (read-info egg repo))))
     (for-each remove-file* (cdr files)))
-  (remove-file* (make-pathname (repository-path) egg "setup-info")))
+  (remove-file* (make-pathname repo egg setup-file-extension)))
 
 (define ($system str)
   (let ((r (system
 	    (if *windows-shell*
-		(string-append "\"" str "\"")	; double quotes, yes - thanks to Matthew Flatt
+		(string-append "\"" str "\"") ; (sic) thanks to Matthew Flatt
 		str))))
     (unless (zero? r)
-      (quit "shell command failed with nonzero exit status ~a:~%~%  ~a" r str))))
+      (error
+       (sprintf "shell command failed with nonzero exit status ~a:~%~%  ~a" r str)))))
+
+(define (setup-error-handling)
+  (current-exception-handler
+   (lambda (c)
+     (print-error-message c (current-error-port))
+     (reset))))
 
-(define (quit fstr . args)
-  (flush-output)
-  (fprintf (current-error-port) "~%~?~%" fstr args)
-  (reset))
 
 ;;; Module Setup
 
