@@ -34,6 +34,7 @@
 			locate-egg/http
 			gather-egg-information
 			list-extensions
+			list-extension-versions
 			temporary-directory)
 
   (import scheme chicken)
@@ -49,6 +50,7 @@
   (define *quiet* #f)
   (define *chicken-install-user-agent* (conc "chicken-install " (chicken-version)))
   (define *trunk* #f)
+  (define *mode* 'default)
 
   (define (d fstr . args)
     (let ([port (if *quiet* (current-error-port) (current-output-port))])
@@ -78,6 +80,13 @@
   (define (list-eggs/local dir)
     (string-concatenate (map (cut string-append <> "\n") (directory dir))) )
 
+  (define (list-egg-versions/local name dir)
+    (let ((eggdir (make-pathname dir (string-append name "/tags"))))
+      (cond ((directory-exists? eggdir)
+	     (string-concatenate
+	      (map (cut string-append <> "\n") (directory eggdir))))
+	    (else "unknown\n"))))
+
   (define (locate-egg/local egg dir #!optional version destination)
     (let* ([eggdir (make-pathname dir egg)]
 	   [tagdir (make-pathname eggdir "tags")]
@@ -105,7 +114,9 @@
 			   (cons (list 'version version)
 				 (handle-exceptions ex
 				     (begin
-				       (warning "extension has syntactically invalid .meta file" egg)
+				       (warning 
+					"extension has syntactically invalid .meta file" 
+					egg)
 				       (return #f))
 				   (with-input-from-file meta read))))))))))
        ls)))
@@ -126,7 +137,18 @@
          (map (lambda (s) (string-append (string-chomp s "/") "\n"))
               (with-input-from-pipe cmd read-lines))) ) ) )
 
-  (define (locate-egg/svn egg repo #!optional version destination username  password)
+  (define (list-egg-versions/svn name repo #!optional username password)
+    (let* ((uarg (if username (string-append "--username='" username "'") ""))
+	   (parg (if password (string-append "--password='" password "'") ""))
+	   (cmd (make-svn-ls-cmd uarg parg (make-pathname repo (string-append name "/tags"))))
+	   (input (with-input-from-pipe cmd read-lines)))
+      (if (null? input)
+	  "unknown\n"
+	  (string-concatenate
+	   (map (lambda (s) (string-append (string-chomp s "/") "\n"))
+		(with-input-from-pipe cmd read-lines))) ) ))
+
+  (define (locate-egg/svn egg repo #!optional version destination username password)
     (let* ([uarg (if username (string-append "--username='" username "'") "")]
 	   [parg (if password (string-append "--password='" password "'") "")]
 	   [cmd (make-svn-ls-cmd uarg parg (make-pathname repo egg) recursive?: #t)])
@@ -145,13 +167,27 @@
                             (if (member "trunk/" files)
                                 (values "trunk" "trunk")
                                 (values "" "") ) ) ) ] )
-          (let* ([tmpdir (make-pathname (or destination (get-temporary-directory)) egg)]
-                 [cmd (make-svn-export-cmd uarg parg (conc repo #\/ egg #\/ filedir) tmpdir)])
+          (let* ((tmpdir (make-pathname (or destination (get-temporary-directory)) egg))
+                 (cmd (make-svn-export-cmd 
+		       uarg parg
+		       (conc 
+			repo #\/ egg #\/
+			(if (eq? *mode* 'meta)
+			    (metafile filedir egg)
+			    filedir))
+		       (if (eq? *mode* 'meta)
+			   (begin
+			     (create-directory tmpdir)
+			     (metafile tmpdir egg))
+			   tmpdir))))
 	    (d "  ~a~%" cmd)
 	    (if (zero? (system cmd))
                 (values tmpdir ver)
 	        (values #f "") ) ) ) ) ) )
 
+  (define (metafile dir egg)
+    (conc dir #\/ egg ".meta"))
+
   (define (deconstruct-url url)
     (let ([m (string-match "(http://)?([^/:]+)(:([^:/]+))?(/.+)" url)])
       (values
@@ -170,6 +206,7 @@
 		     locn
 		     "?name=" egg
 		     (if version (string-append "&version=" version) "")
+		     "&mode=" (->string *mode*)
 		     (if tests "&tests=yes" ""))]
 	      [eggdir (make-pathname tmpdir egg) ] )
 	  (unless (file-exists? eggdir) (create-directory eggdir))
@@ -243,7 +280,7 @@
 		(d "~a~%" ln)
 		(loop) ) ) ) )
 	(when chunked
-	  (d "reading chunks ...~%")
+	  (d "reading chunks ")
 	  (let ([data (read-chunks in)])
 	    (close-input-port in)
 	    (set! in (open-input-string data))) ) )
@@ -283,36 +320,50 @@
   (define (read-chunks in)
     (let get-chunks ([data '()])
       (let ([size (string->number (read-line in) 16)])
-	(if (zero? size)
-	    (string-concatenate-reverse data)
-	    (let ([chunk (read-string size in)])
-	      (read-line in)
-	      (get-chunks (cons chunk data)) ) ) ) ) )
+	(cond ((zero? size)
+	       (d "~%")
+	       (string-concatenate-reverse data))
+	      (else
+	       (let ([chunk (read-string size in)])
+		 (d ".")
+		 (read-line in)
+		 (get-chunks (cons chunk data)) ) ) ) ) ))
 
   (define (retrieve-extension name transport location
                               #!key version quiet destination username password tests
-			      proxy-host proxy-port trunk)
+			      proxy-host proxy-port trunk (mode 'default))
     (fluid-let ((*quiet* quiet)
-		(*trunk* trunk))
+		(*trunk* trunk)
+		(*mode* mode))
       (case transport
-	[(local)
+	((local)
 	 (when destination (warning "destination for transport `local' ignored"))
-	 (locate-egg/local name location version destination) ]
-	[(svn)
-	 (locate-egg/svn name location version destination username password) ]
-	[(http)
-	 (locate-egg/http name location version destination tests proxy-host proxy-port) ]
-	[else
-	 (error "cannot retrieve extension unsupported transport" transport) ] ) ) )
+	 (locate-egg/local name location version destination) )
+	((svn)
+	 (locate-egg/svn name location version destination username password) )
+	((http)
+	 (locate-egg/http name location version destination tests proxy-host proxy-port) )
+	(else
+	 (error "cannot retrieve extension unsupported transport" transport) ) ) ) )
 
   (define (list-extensions transport location #!key quiet username password)
-    (fluid-let ([*quiet* quiet])
+    (fluid-let ((*quiet* quiet))
       (case transport
-	[(local)
-	 (list-eggs/local location) ]
-	[(svn)
-	 (list-eggs/svn location username password) ]
-	[else
-	 (error "cannot list extensions - unsupported transport" transport) ] ) ) )
+	((local)
+	 (list-eggs/local location) )
+	((svn)
+	 (list-eggs/svn location username password) )
+	(else
+	 (error "cannot list extensions - unsupported transport" transport) ) ) ) )
+
+  (define (list-extension-versions name transport location #!key quiet username password)
+    (fluid-let ((*quiet* quiet))
+      (case transport
+	((local)
+	 (list-egg-versions/local name location) )
+	((svn)
+	 (list-egg-versions/svn name location username password) )
+	(else
+	 (error "cannot list extensions - unsupported transport" transport) ) ) ) )
 
 ) ;module setup-download
