@@ -26,38 +26,14 @@
 
 (declare
  (unit srfi-69)
- (usual-integrations)
- (disable-warning redef) ) ; hash-table-ref is an extended binding!
+ (hide
+  *eq?-hash *eqv?-hash *equal?-hash
+  *make-hash-table
+  *hash-table-copy *hash-table-merge! *hash-table-update!/default
+  *hash-table-for-each *hash-table-fold
+  hash-table-canonical-length hash-table-rehash! hash-table-check-resize! ) )
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (no-bound-checks)
-    (no-procedure-checks-for-usual-bindings) ) ] )
-
-(declare
-  (bound-to-procedure
-    ##sys#signal-hook
-    ##sys#peek-fixnum
-    ##sys#make-structure
-    ##sys#size
-    ##sys#slot ##sys#setslot
-    *equal?-hash )
-  (hide
-    *eq?-hash *eqv?-hash *equal?-hash
-    *make-hash-table
-    *hash-table-copy *hash-table-merge! *hash-table-update!/default
-    *hash-table-for-each *hash-table-fold
-    hash-table-canonical-length hash-table-rehash! hash-table-check-resize! ) )
-
-(declare
-  (bound-to-procedure
-   ##sys#check-string ##sys#check-symbol
-   ##sys#check-exact ##sy#check-inexact
-   ##sys#check-closure ##sys#check-structure ) )
-
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'srfi-69)
 
@@ -102,6 +78,11 @@
 (define-inline (%subbyte bytvec i)
   (##core#inline "C_subbyte" bytvec i) )
 
+(define-inline (exactify n)
+  (if (##sys#immediate? n)
+      n
+      (##core#inline "C_i_inexact_to_exact" n)))
+
 
 ;;; Generation of hash-values:
 
@@ -492,7 +473,7 @@
 		(##sys#check-exact arg 'make-hash-table)
 		(unless (fx< 0 arg)
 		  (error 'make-hash-table "invalid size" arg) )
-		(set! size (fxmin hash-table-max-size arg))
+		(set! size (fxmin hash-table-max-length arg))
 		(set! arguments (cdr arguments)) ) ) )
 	  ; Process keyword arguments
 	  (let loop ([args arguments])
@@ -517,7 +498,7 @@
 			    (##sys#check-exact val 'make-hash-table)
 			    (unless (fx< 0 val)
 			      (error 'make-hash-table "invalid size" val) )
-			    (set! size (fxmin hash-table-max-size val))]
+			    (set! size (fxmin hash-table-max-length val))]
 			  [(#:initial)
 			    (set! initial (lambda () val))]
 			  [(#:min-load)
@@ -631,8 +612,8 @@
         [min-load (##sys#slot ht 5)]
         [max-load (##sys#slot ht 6)] )
     (let ([len (##sys#size vec)] )
-      (let ([min-load-len (inexact->exact (floor (* len min-load)))]
-            [max-load-len (inexact->exact (floor (* len max-load)))] )
+      (let ([min-load-len (exactify (floor (* len min-load)))]
+            [max-load-len (exactify (floor (* len max-load)))] )
         (if (and (fx< len hash-table-max-length)
                  (fx<= min-load-len newsiz) (fx<= newsiz max-load-len))
           (hash-table-resize! ht vec len) ) ) ) ) )
@@ -832,7 +813,8 @@
 		        (if (test key (##sys#slot pare 0))
 			    (##sys#slot pare 1)
 			    (loop (##sys#slot bucket 1)) ) ) ) ) ) ) ) ) )
-   hash-table-set!) )
+   hash-table-set!
+   "(hash-table-ref ht key . def)") )
 
 (define hash-table-ref/default
   (let ([core-eq? eq?])
@@ -993,16 +975,15 @@
 			 (cons (cons (##sys#slot x 0) (##sys#slot x 1)) lst) ) ) ) ) ) ) ) )
 
 (define alist->hash-table
-  (let ([make-hash-table make-hash-table])
-    (lambda (alist . rest)
-      (##sys#check-list alist 'alist->hash-table)
-      (let ([ht (apply make-hash-table rest)])
-	(for-each
-	 (lambda (x)
-	   (##sys#check-pair x 'alist->hash-table)
-	   (*hash-table-update!/default  ht (##sys#slot x 0) (lambda (x) x) (##sys#slot x 1)) )
-	 alist)
-	ht ) ) ) )
+  (lambda (alist . rest)
+    (##sys#check-list alist 'alist->hash-table)
+    (let ([ht (apply make-hash-table rest)])
+      (for-each
+       (lambda (x)
+	 (##sys#check-pair x 'alist->hash-table)
+	 (*hash-table-update!/default  ht (##sys#slot x 0) (lambda (x) x) (##sys#slot x 1)) )
+       alist)
+      ht ) ) )
 
 ;; Hash-Table Keys & Values:
 
