@@ -61,12 +61,6 @@
 	 (flush-output)
 	 #t) ) )
 
-(define (compiler-warning class msg . args)	       
-  (when (and ##sys#warnings-enabled (not (memq class disabled-warnings)))
-    (let ((out (current-error-port)))
-      (apply fprintf out (string-append "\nWarning: " msg) args)
-      (newline out) ) ) )
-
 (define (quit msg . args)
   (let ([out (current-error-port)])
     (apply fprintf out (string-append "\nError: " msg) args)
@@ -82,8 +76,8 @@
 		      (set! args (cdr args))
 		      loc))))
       (if loc
-	  (fprintf out "Syntax error (~a): ~a~%~%" loc msg) 
-	  (fprintf out "Syntax error: ~a~%~%" msg) )
+	  (fprintf out "\nSyntax error (~a): ~a~%~%" loc msg) 
+	  (fprintf out "\nSyntax error: ~a~%~%" msg) )
       (for-each (cut fprintf out "\t~s~%" <>) args)
       (print-call-chain out 0 ##sys#current-thread "\n\tExpansion history:\n")
       (exit 70) ) ) )
@@ -478,9 +472,8 @@
 				  (eq? 'fixnum number-type)
 				  (not (integer? c)) )
 			     (begin
-			       (compiler-warning
-				'type
-				"literal '~s' is out of range - will be truncated to integer" c)
+			       (warning
+				"literal is out of range - will be truncated to integer" c)
 			       (inexact->exact (truncate c)) )
 			     c) ) ) )
 	       ((let)
@@ -671,33 +664,40 @@
     (make-node (car x) (cadr x) (map walk (cddr x)))))
 
 (define (emit-global-inline-file filename db)
-  (let ((lst '()))
-    (with-output-to-file filename
-      (lambda ()
-	(print "; GENERATED BY CHICKEN " (chicken-version) " FROM "
-	       source-filename "\n")
-	(##sys#hash-table-for-each
-	 (lambda (sym plist)
-	   (when (variable-visible? sym)
-	     (and-let* ((val (assq 'local-value plist))
-			((not (node? (variable-mark sym '##compiler#inline-global))))
-			((let ((val (assq 'value plist)))
-			   (or (not val)
-			       (not (eq? 'unknown (cdr val))))))
-			((assq 'inlinable plist))
-			(lparams (node-parameters (cdr val)))
-			;;((get db (first lparams) 'simple)) 
-			((not (get db sym 'hidden-refs)))
-			((case (variable-mark sym '##compiler#inline)
-			   ((yes) #t)
-			   ((no) #f)
-			   (else 
-			    (< (fourth lparams) inline-max-size) ) ) ) )
-	       (set! lst (cons sym lst))
-	       (pp (list sym (node->sexpr (cdr val))))
-	       (newline))))
-	 db)
-	(print "; END OF FILE")))
+  (let ((lst '())
+	(out '()))
+    (##sys#hash-table-for-each
+     (lambda (sym plist)
+       (when (variable-visible? sym)
+	 (and-let* ((val (assq 'local-value plist))
+		    ((not (node? (variable-mark sym '##compiler#inline-global))))
+		    ((let ((val (assq 'value plist)))
+		       (or (not val)
+			   (not (eq? 'unknown (cdr val))))))
+		    ((assq 'inlinable plist))
+		    (lparams (node-parameters (cdr val)))
+		    ;;((get db (first lparams) 'simple)) 
+		    ((not (get db sym 'hidden-refs)))
+		    ((case (variable-mark sym '##compiler#inline)
+		       ((yes) #t)
+		       ((no) #f)
+		       (else 
+			(< (fourth lparams) inline-max-size) ) ) ) )
+	   (set! lst (cons sym lst))
+	   (set! out (cons (list sym (node->sexpr (cdr val))) out)))))
+     db)
+    (if (null? out)
+	(delete-file* filename)
+	(with-output-to-file filename
+	  (lambda ()
+	    (print "; GENERATED BY CHICKEN " (chicken-version) " FROM "
+		   source-filename "\n")
+	    (for-each
+	     (lambda (x)
+	       (pp x)
+	       (newline))
+	     (reverse out))
+	    (print "; END OF FILE"))))
     (when (and (pair? lst)
 	       (debugging 'i "the following procedures can be globally inlined:"))
       (for-each (cut print "  " <>) (sort-symbols lst)))))
@@ -1209,6 +1209,7 @@ Usage: chicken FILENAME OPTION ...
   Language options:
 
     -feature SYMBOL              register feature identifier
+    -no-feature SYMBOL           disable built-in feature identifier
 
   Syntax related options:
 
@@ -1223,6 +1224,7 @@ Usage: chicken FILENAME OPTION ...
     -emit-import-library MODULE  write compile-time module information into
                                   separate file
     -emit-all-import-libraries   emit import-libraries for all defined modules
+    -no-module-registration      do not generate module registration code
     -no-compiler-syntax          disable expansion of compiler-macros
 
   Translation options:
@@ -1235,7 +1237,6 @@ Usage: chicken FILENAME OPTION ...
   Debugging options:
 
     -no-warnings                 disable warnings
-    -disable-warning CLASS       disable specific class of warnings
     -debug-level NUMBER          set level of available debugging information
     -no-trace                    disable tracing information
     -profile                     executable emits profiling information 
@@ -1275,6 +1276,9 @@ Usage: chicken FILENAME OPTION ...
     -no-procedure-checks-for-usual-bindings
                                  disable procedure call checks only for usual
                                   bindings
+    -no-procedure-checks-for-toplevel-bindings
+                                   disable procedure call checks for toplevel
+                                    bindings
 
   Configuration options:
 
@@ -1503,7 +1507,8 @@ EOF
       (printf "loading identifier database ~a ...~%" dbfile))
     (for-each
      (lambda (e)
-       (##sys#put! 
-	(car e) '##core#db
-	(append (or (##sys#get (car e) '##core#db) '()) (list (cdr e))) ))
+       (let ((id (car e)))
+	 (##sys#put! 
+	  id '##core#db
+	  (append (or (##sys#get id '##core#db) '()) (list (cdr e))) )))
      (read-file dbfile))))
