@@ -28,20 +28,9 @@
 (declare
   (unit tcp)
   (uses extras scheduler)
-  (usual-integrations)
-  (fixnum-arithmetic)
-  (no-bound-checks)
   (export tcp-close tcp-listen tcp-connect tcp-accept tcp-accept-ready? ##sys#tcp-port->fileno tcp-listener? tcp-addresses
 	  tcp-abandon-port tcp-listener-port tcp-listener-fileno tcp-port-numbers tcp-buffer-size
 	  tcp-read-timeout tcp-write-timeout tcp-accept-timeout tcp-connect-timeout)
-  (no-procedure-checks-for-usual-bindings)
-  (bound-to-procedure
-   ##net#socket ##net#bind ##net#connect ##net#listen ##net#accept make-parameter ##sys#string-append ##sys#tcp-port->fileno
-   ##sys#check-port ##sys#port-data ##sys#thread-block-for-i/o! make-string make-input-port make-output-port ##sys#substring
-   substring ##sys#make-c-string ##sys#schedule ##sys#set-port-data!
-   ##net#close ##net#recv ##net#send ##net#select ##net#select-write ##net#gethostaddr ##net#io-ports ##sys#update-errno
-   ##sys#error ##sys#signal-hook ##net#getservbyname ##net#parse-host ##net#fresh-addr
-   ##net#bind-socket ##net#shutdown)
   (foreign-declare #<<EOF
 #include <errno.h>
 #ifdef _WIN32
@@ -86,7 +75,7 @@ static char addr_buffer[ 20 ];
 EOF
 ) )
 
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'tcp)
 
@@ -118,56 +107,57 @@ EOF
 (define ##net#connect (foreign-lambda int "connect" int scheme-pointer int))
 
 (define ##net#send
-  (foreign-lambda* int ((int s) (scheme-pointer msg) (int offset) (int len) (int flags))
-		   "return(send(s, (char *)msg+offset, len, flags));"))
+  (foreign-lambda* 
+      int ((int s) (scheme-pointer msg) (int offset) (int len) (int flags))
+    "C_return(send(s, (char *)msg+offset, len, flags));"))
 
 (define ##net#make-nonblocking
   (foreign-lambda* bool ((int fd))
     "int val = fcntl(fd, F_GETFL, 0);"
-    "if(val == -1) return(0);"
-    "return(fcntl(fd, F_SETFL, val | O_NONBLOCK) != -1);") )
+    "if(val == -1) C_return(0);"
+    "C_return(fcntl(fd, F_SETFL, val | O_NONBLOCK) != -1);") )
 
 (define ##net#getsockname 
   (foreign-lambda* c-string ((int s))
     "struct sockaddr_in sa;"
     "unsigned char *ptr;"
     "int len = sizeof(struct sockaddr_in);"
-    "if(getsockname(s, (struct sockaddr *)&sa, (socklen_t *)&len) != 0) return(NULL);"
+    "if(getsockname(s, (struct sockaddr *)&sa, (socklen_t *)&len) != 0) C_return(NULL);"
     "ptr = (unsigned char *)&sa.sin_addr;"
     "sprintf(addr_buffer, \"%d.%d.%d.%d\", ptr[ 0 ], ptr[ 1 ], ptr[ 2 ], ptr[ 3 ]);"
-    "return(addr_buffer);") )
+    "C_return(addr_buffer);") )
 
 (define ##net#getsockport
   (foreign-lambda* int ((int s))
     "struct sockaddr_in sa;"
     "int len = sizeof(struct sockaddr_in);"
-    "if(getsockname(s, (struct sockaddr *)&sa, (socklen_t *)(&len)) != 0) return(-1);"
-    "else return(ntohs(sa.sin_port));") )
+    "if(getsockname(s, (struct sockaddr *)&sa, (socklen_t *)(&len)) != 0) C_return(-1);"
+    "else C_return(ntohs(sa.sin_port));") )
 
 (define ##net#getpeerport
  (foreign-lambda* int ((int s))
    "struct sockaddr_in sa;"
    "int len = sizeof(struct sockaddr_in);"
-   "if(getpeername(s, (struct sockaddr *)&sa, (socklen_t *)(&len)) != 0) return(-1);"
-   "else return(ntohs(sa.sin_port));") )
+   "if(getpeername(s, (struct sockaddr *)&sa, (socklen_t *)(&len)) != 0) C_return(-1);"
+   "else C_return(ntohs(sa.sin_port));") )
 
 (define ##net#getpeername 
   (foreign-lambda* c-string ((int s))
     "struct sockaddr_in sa;"
     "unsigned char *ptr;"
     "unsigned int len = sizeof(struct sockaddr_in);"
-    "if(getpeername(s, (struct sockaddr *)&sa, ((unsigned int *)&len)) != 0) return(NULL);"
+    "if(getpeername(s, (struct sockaddr *)&sa, ((unsigned int *)&len)) != 0) C_return(NULL);"
     "ptr = (unsigned char *)&sa.sin_addr;"
     "sprintf(addr_buffer, \"%d.%d.%d.%d\", ptr[ 0 ], ptr[ 1 ], ptr[ 2 ], ptr[ 3 ]);"
-    "return(addr_buffer);") )
+    "C_return(addr_buffer);") )
 
 (define ##net#startup
   (foreign-lambda* bool () #<<EOF
 #ifdef _WIN32
-     return(WSAStartup(MAKEWORD(1, 1), &wsa) == 0);
+     C_return(WSAStartup(MAKEWORD(1, 1), &wsa) == 0);
 #else
      signal(SIGPIPE, SIG_IGN);
-     return(1);
+     C_return(1);
 #endif
 EOF
 ) )
@@ -178,8 +168,8 @@ EOF
 (define ##net#getservbyname 
   (foreign-lambda* int ((c-string serv) (c-string proto))
     "struct servent *se;
-     if((se = getservbyname(serv, proto)) == NULL) return(0);
-     else return(ntohs(se->s_port));") )     
+     if((se = getservbyname(serv, proto)) == NULL) C_return(0);
+     else C_return(ntohs(se->s_port));") )     
 
 (define ##net#select
   (foreign-lambda* int ((int fd))
@@ -191,7 +181,7 @@ EOF
      tm.tv_sec = tm.tv_usec = 0;
      rv = select(fd + 1, &in, NULL, NULL, &tm);
      if(rv > 0) { rv = FD_ISSET(fd, &in) ? 1 : 0; }
-     return(rv);") )
+     C_return(rv);") )
 
 (define ##net#select-write
   (foreign-lambda* int ((int fd))
@@ -203,18 +193,18 @@ EOF
      tm.tv_sec = tm.tv_usec = 0;
      rv = select(fd + 1, NULL, &out, NULL, &tm);
      if(rv > 0) { rv = FD_ISSET(fd, &out) ? 1 : 0; }
-     return(rv);") )
+     C_return(rv);") )
 
 (define ##net#gethostaddr
   (foreign-lambda* bool ((scheme-pointer saddr) (c-string host) (unsigned-short port))
     "struct hostent *he = gethostbyname(host);"
     "struct sockaddr_in *addr = (struct sockaddr_in *)saddr;"
-    "if(he == NULL) return(0);"
+    "if(he == NULL) C_return(0);"
     "memset(addr, 0, sizeof(struct sockaddr_in));"
     "addr->sin_family = AF_INET;"
     "addr->sin_port = htons((short)port);"
     "addr->sin_addr = *((struct in_addr *)he->h_addr);"
-    "return(1);") )
+    "C_return(1);") )
 
 (define (yield)
   (##sys#call-with-current-continuation
@@ -233,7 +223,7 @@ EOF
 	      (let ((c (##core#inline "C_subchar" host i)))
 		(if (char=? c #\:)		    
 		    (values
-		     (substring host (add1 i) len)
+		     (substring host (fx+ i 1) len)
 		     (let* ((s (substring host 0 i))
 			    (p (##net#getservbyname s proto)) )
 		       (when (eq? 0 p)
@@ -264,7 +254,7 @@ EOF
     ;; PLT makes this an optional arg to tcp-listen. Should we as well?
     (when (eq? -1 ((foreign-lambda* int ((int socket)) 
 		     "int yes = 1; 
-                      return(setsockopt(socket, SOL_SOCKET, SO_REUSEADDR, (const char *)&yes, sizeof(int)));") 
+                      C_return(setsockopt(socket, SOL_SOCKET, SO_REUSEADDR, (const char *)&yes, sizeof(int)));") 
 		   s) )
       (##sys#update-errno)
       (##sys#signal-hook 
@@ -325,16 +315,14 @@ EOF
   (define ((check loc) x)
     (when x (##sys#check-exact x loc))
     x)
-  (define minute (* 60 1000))
+  (define minute (fx* 60 1000))
   (set! tcp-read-timeout (make-parameter minute (check 'tcp-read-timeout)))
   (set! tcp-write-timeout (make-parameter minute (check 'tcp-write-timeout))) 
   (set! tcp-connect-timeout (make-parameter #f (check 'tcp-connect-timeout))) 
   (set! tcp-accept-timeout (make-parameter #f (check 'tcp-accept-timeout))) )
 
 (define ##net#io-ports
-  (let ((make-input-port make-input-port)
-	(make-output-port make-output-port) 
-	(tbs tcp-buffer-size)
+  (let ((tbs tcp-buffer-size)
 	(make-string make-string) )
     (lambda (fd)
       (unless (##net#make-nonblocking fd)
@@ -360,7 +348,7 @@ EOF
 				  (when tmr
 				    (##sys#thread-block-for-timeout! 
 				     ##sys#current-thread
-				     (fx+ (##sys#fudge 16) tmr) ) )
+				     (+ (current-milliseconds) tmr) ) )
 				  (##sys#thread-block-for-i/o! ##sys#current-thread fd #t)
 				  (yield)
 				  (when (##sys#slot ##sys#current-thread 13)
@@ -468,7 +456,7 @@ EOF
 				  (when tmw
 				    (##sys#thread-block-for-timeout! 
 				     ##sys#current-thread
-				     (fx+ (##sys#fudge 16) tmw) ) )
+				     (+ (current-milliseconds) tmw) ) )
 				  (##sys#thread-block-for-i/o! ##sys#current-thread fd #f)
 				  (yield) 
 				  (when (##sys#slot ##sys#current-thread 13)
@@ -536,7 +524,7 @@ EOF
 	    (when tma
 	      (##sys#thread-block-for-timeout! 
 	       ##sys#current-thread
-	       (fx+ (##sys#fudge 16) tma) ) )
+	       (+ (current-milliseconds) tma) ) )
 	    (##sys#thread-block-for-i/o! ##sys#current-thread fd #t)
 	    (yield)
 	    (when (##sys#slot ##sys#current-thread 13)
@@ -561,8 +549,8 @@ EOF
     "int err, optlen;"
     "optlen = sizeof(err);"
     "if (typecorrect_getsockopt(socket, SOL_SOCKET, SO_ERROR, &err, (socklen_t *)&optlen) == -1)"
-    "return(-1);"
-    "return(err);"))
+    "C_return(-1);"
+    "C_return(err);"))
 
 (define general-strerror (foreign-lambda c-string "strerror" int))
 
@@ -601,7 +589,7 @@ EOF
 		  (when tmc
 		    (##sys#thread-block-for-timeout!
 		     ##sys#current-thread
-		     (fx+ (##sys#fudge 16) tmc) ) )
+		     (+ (current-milliseconds) tmc) ) )
 		  (##sys#thread-block-for-i/o! ##sys#current-thread s #:all)
 		  (yield)
 		  (when (##sys#slot ##sys#current-thread 13)
@@ -612,12 +600,12 @@ EOF
 		  (loop) ) ) )
 	    (fail) ) )
       (let ((err (get-socket-error s)))
-	(cond ((= err -1)
+	(cond ((fx= err -1)
 	       (##net#close s)
 	       (##sys#signal-hook 
 		#:network-error 'tcp-connect
 		(##sys#string-append "getsockopt() failed - " strerror)))
-	      ((> err 0)
+	      ((fx> err 0)
 	       (##net#close s)
 	       (##sys#signal-hook 
 		#:network-error 'tcp-connect
