@@ -6,8 +6,10 @@
 
 set -e
 TEST_DIR=`pwd`
+OS_NAME=`uname -s`
 export DYLD_LIBRARY_PATH=${TEST_DIR}/..
 export LD_LIBRARY_PATH=${TEST_DIR}/..
+export LIBRARY_PATH=${TEST_DIR}/..:${LIBRARY_PATH}
 
 mkdir -p test-repository
 
@@ -27,9 +29,12 @@ done
 "${TEST_DIR}/../chicken-install" -init test-repository export
 CHICKEN_REPOSITORY=${TEST_DIR}/test-repository
 CHICKEN=../chicken
+ASMFLAGS=
+FAST_OPTIONS="-O5 -d0 -b -disable-interrupts"
 
 if test -n "$MSYSTEM"; then
     CHICKEN="..\\chicken.exe"
+    ASMFLAGS=-Wa,-w
     # make compiled tests use proper library on Windows
     cp ../libchicken.dll .
 fi
@@ -68,6 +73,17 @@ fi
 
 diff -bu scrutiny.out scrutiny.expected
 
+echo "======================================== callback tests ..."
+$compile callback-tests.scm
+./a.out
+
+if ./a.out twice; then
+    echo "double-return from callback didn't fail"
+    exit 1
+else
+    echo "double-return from callback failed as it should."
+fi
+
 echo "======================================== runtime tests ..."
 $interpret -s apply-test.scm
 $compile test-gc-hooks.scm
@@ -79,6 +95,15 @@ $interpret -s records-and-setters-test.scm
 $compile records-and-setters-test.scm
 ./a.out
 
+echo "======================================== dynamic-wind tests ..."
+$interpret -s dwindtst.scm >dwindtst.out
+diff -bu dwindtst.expected dwindtst.out
+$compile dwindtst.scm
+./a.out >dwindtst.out
+diff -bu dwindtst.expected dwindtst.out
+echo "*** Skipping \"feeley-dynwind\" for now ***"
+# $interpret -s feeley-dynwind.scm
+
 echo "======================================== syntax tests ..."
 $interpret -s syntax-tests.scm
 
@@ -111,6 +136,9 @@ echo "======================================== compiler syntax tests ..."
 $compile compiler-syntax-tests.scm
 ./a.out
 
+echo "======================================== import tests ..."
+$interpret -bnq import-tests.scm
+
 echo "======================================== import library tests ..."
 rm -f ../foo.import.* foo.import.*
 $compile import-library-test1.scm -emit-import-library foo
@@ -185,8 +213,6 @@ $interpret -s srfi-4-tests.scm
 
 echo "======================================== srfi-18 tests ..."
 $interpret -s srfi-18-tests.scm
-echo "*** Skipping \"feeley-dynwind\" for now ***"
-# $interpret -s feeley-dynwind.scm
 
 echo "======================================== path tests ..."
 $interpret -bnq path-tests.scm
@@ -238,24 +264,4 @@ tmp/xxx $PWD/tmp
 PATH=$PWD/tmp:$PATH xxx $PWD/tmp
 # this may crash, if the PATH contains a non-matching libchicken.dll on Windows:
 #PATH=$PATH:$PWD/tmp xxx $PWD/tmp
-
-echo "======================================== timing compilation ..."
-time $compile compiler.scm -O5 -debug pb -v -C -Wa,-W
-echo "executing ..."
-time ./a.out
-
-echo "======================================== running slatex ..."
-$compile slatex.scm -O5
-mkdir -p slatexdir
-rm -f slatexdir/*
-time ./a.out
-
-echo "======================================== running floating-point benchmark ..."
-echo "boxed:"
-$compile fft.scm -O5
-time ./a.out
-echo "unboxed:"
-$compile fft.scm -O5 -D unboxed -debug oxi | tee fft.out
-time ./a.out
-
 echo "======================================== done."
