@@ -433,3 +433,48 @@
 (assert (= 3 (f1)))
 (assert (= 9 v1))
 (assert (= 10 v2))
+
+
+;;; redefining definition forms (disabled, since we can not catch this error easily)
+
+#|
+(module m0a () (import chicken) (reexport (only scheme define)))
+(module m0b () (import chicken) (reexport (only scheme define-syntax)))
+
+(module m1 ()
+(import (prefix scheme s:) (prefix m0b m:))
+;(s:define m:define 1)
+(s:define-syntax s:define-syntax (syntax-rules ()))
+)
+|#
+
+
+;;; renaming of keyword argument (#277)
+
+(define-syntax foo1
+  (syntax-rules ()
+    ((_ procname)
+     (define (procname #!key (who "world"))
+       (string-append "hello, " who)))))
+
+(foo1 bar)
+
+(assert (string=? "hello, XXX" (bar who: "XXX")))
+
+
+;;; import not seen, if explicitly exported and renamed:
+
+(module rfoo (rbar rbaz)
+(import scheme chicken)
+
+(define (rbaz x)
+  (print x))
+
+(define-syntax rbar
+  (syntax-rules ()
+    ((_ x) (rbaz x))))
+
+)
+
+(import (prefix rfoo f:))
+(f:rbar 1)
