@@ -28,26 +28,11 @@
 (declare
   (unit utils)
   (uses extras srfi-13 posix files regex)
-  (usual-integrations)
   (fixnum)
   (hide chop-pds)
   (disable-interrupts) )
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (always-bound
-      ##sys#windows-platform)
-    (bound-to-procedure
-      ##sys#check-port port? read-string read-line with-input-from-file
-      command-line-arguments
-      string-append
-      system)
-    (no-procedure-checks-for-usual-bindings)
-    (no-bound-checks))] )
-
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'utils)
 
@@ -55,13 +40,11 @@
 ;;; Like `system', but allows format-string and bombs on nonzero return code:
 
 (define system*
-  (let ([sprintf sprintf]
-	[system system] )
-    (lambda (fstr . args)
-      (let* ([str (apply sprintf fstr args)]
-	     [n (system str)] )
-	(unless (zero? n)
-	  (##sys#error "shell invocation failed with non-zero return status" str n) ) ) ) ) )
+  (lambda (fstr . args)
+    (let* ([str (apply sprintf fstr args)]
+	   [n (system str)] )
+      (unless (zero? n)
+	(##sys#error "shell invocation failed with non-zero return status" str n) ) ) ) )
 
 
 ;;; Read file as string from given filename or port:
@@ -98,8 +81,8 @@
 
 (define compile-file
   (let ((csc (foreign-value "C_CSC_PROGRAM" c-string))
-	(path (foreign-value "C_INSTALL_BIN_HOME" c-string)) 
-	(load-file load))
+	(load-file load)
+	(path (foreign-value "C_INSTALL_BIN_HOME" c-string)) )
     (lambda (filename #!key (options '()) output-file (load #t))
       (let ((cscpath (or (file-exists? (make-pathname path csc)) "csc"))
 	    (tmpfile (and (not output-file) (create-temporary-file "so")))
@@ -130,15 +113,89 @@
 ;;; Scan lines until regex or predicate matches
 
 (define scan-input-lines
-  (let ((regexp regexp)
-	(read-line read-line)
-	(string-search string-search))
-    (lambda (rx #!optional (port ##sys#standard-input))
-      (let ((rx (if (procedure? rx)
-		    rx
-		    (cut string-search (regexp rx) <>))))
+  (lambda (rx #!optional (port ##sys#standard-input))
+    (let ((rx (if (procedure? rx)
+		  rx
+		  (cut string-search (regexp rx) <>))))
+      (let loop ()
+	(let ((ln (read-line port)))
+	  (and (not (eof-object? ln))
+	       (or (rx ln)
+		   (loop))))))))
+
+
+;; Ask for confirmation
+
+#>
+#if defined(_WIN32) && !defined(__CYGWIN__)
+# include <windows.h>
+# define C_HAS_MESSAGE_BOX 1
+static int
+C_confirmation_dialog(char *msg, char *caption, int def, int abort)
+{
+  int d = 0, r;
+  int t = abort ? MB_YESNOCANCEL : MB_YESNO;
+
+  switch(def) {
+  case 0: d = MB_DEFBUTTON1; break;
+  case 1: d = MB_DEFBUTTON2; break;
+  case 2: d = MB_DEFBUTTON3;
+  }
+
+  r = MessageBox(NULL, msg, caption, t | MB_ICONQUESTION | d);
+
+  switch(r) {
+  case IDYES: return 1;
+  case IDNO: return 0;
+  default: return -1;
+  }
+}
+#else
+# define C_HAS_MESSAGE_BOX 0
+static int
+C_confirmation_dialog(char *msg, char *caption, int def, int abort) { return -1; }
+#endif
+<#
+
+;; Note: for Mac OS X, "CFUserNotificationDisplayAlert" could be used,
+;;       unless that requires linking any libraries. This would also
+;;       be useful for runtime error messages.
+
+(define-foreign-variable C_HAS_MESSAGE_BOX bool)
+
+(define yes-or-no?
+  (let ((dialog (foreign-lambda int "C_confirmation_dialog" c-string c-string int bool)))
+    (lambda (str #!key default title (abort reset))
+      (let ((gui (and C_HAS_MESSAGE_BOX (not (##sys#fudge 4))))) ; C_gui_mode
+	(define (get-input)
+	  (if gui
+	      (let ((r (dialog 
+			str
+			(or title "CHICKEN Runtime")
+			(cond ((not default) 3)
+			      ((string-ci=? default "yes") 0)
+			      ((string-ci=? default "no") 1)
+			      (else 2))
+			abort)))
+		(case r
+		  ((0) "no")
+		  ((1) "yes")
+		  (else "abort")))
+	      (string-trim-both (read-line))))
 	(let loop ()
-	  (let ((ln (read-line port)))
-	    (and (not (eof-object? ln))
-		 (or (rx ln)
-		     (loop)))))))))
+	  (unless gui
+	    (printf "~%~A (yes/no~a) " str (if abort "/abort" ""))
+	    (when default (printf "[~A] " default))
+	    (flush-output))
+	  (let ((ln (get-input)))
+	    (cond ((eof-object? ln) (set! ln "abort"))
+		  ((and default (string=? "" ln)) (set! ln default)) )
+	    (cond ((string-ci=? "yes" ln) #t)
+		  ((string-ci=? "no" ln) #f)
+		  ((and abort (string-ci=? "abort" ln)) (abort))
+		  (else
+		   (if abort
+		       (printf "~%Please enter \"yes\" or \"no\".~%")
+		       (printf "~%Please enter \"yes\", \"no\" or \"abort\".~%"))
+		   (loop) ) ) ) ) ) ) ) )
+  
